@@ -1,6 +1,6 @@
 /* MAPIStoreCalendarAttachment.h - this file is part of SOGo
  *
- * Copyright (C) 2011 Inverse inc
+ * Copyright (C) 2011-2012 Inverse inc
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -25,7 +25,18 @@
 
 #import "MAPIStoreAttachment.h"
 
-@interface MAPIStoreCalendarAttachment :  MAPIStoreAttachment
+@class NSTimeZone;
+
+@class iCalEvent;
+
+@interface MAPIStoreCalendarAttachment : MAPIStoreAttachment
+{
+  iCalEvent *event;
+  NSTimeZone *timeZone;
+}
+
+- (void) setEvent: (iCalEvent *) newEvent;
+- (iCalEvent *) event;
 
 @end
 
