@@ -33,7 +33,6 @@
 #import <SOGo/SOGoUser.h>
 
 #import "MAPIStoreAttachment.h"
-// #import "MAPIStoreAttachmentTable.h"
 #import "MAPIStoreFallbackContext.h"
 #import "MAPIStoreFolder.h"
 #import "MAPIStoreFolderTable.h"
@@ -51,6 +50,7 @@
 #import "MAPIStoreContext.h"
 
 #undef DEBUG
+#include <dlinklist.h>
 #include <stdbool.h>
 #include <gen_ndr/exchange.h>
 #include <util/attr.h>
@@ -88,32 +88,31 @@ static NSMutableDictionary *contextClassMapping;
       currentClass = [classes objectAtIndex: count];
       moduleName = [currentClass MAPIModuleName];
       if (moduleName)
-	{
-	  [contextClassMapping setObject: currentClass
+        {
+          [contextClassMapping setObject: currentClass
                                   forKey: moduleName];
-	  NSLog (@"  registered class '%@' as handler of '%@' contexts",
-		 NSStringFromClass (currentClass), moduleName);
-	}
+          NSLog (@"  registered class '%@' as handler of '%@' contexts",
+                 NSStringFromClass (currentClass), moduleName);
+        }
     }
 
   MAPIStoreFallbackContextK = [MAPIStoreFallbackContext class];
 }
 
 + (struct mapistore_contexts_list *) listAllContextsForUser: (NSString *)  userName
-                                            withIndexing: (struct indexing_context *) indexing
+                                               withIndexing: (struct indexing_context *) indexing
                                                    inMemCtx: (TALLOC_CTX *) memCtx
 {
   struct mapistore_contexts_list *list, *current;
   NSArray *classes;
   Class currentClass;
   NSUInteger count, max;
-  MAPIStoreUserContext *userContext;
 
   list = NULL;
 
-  userContext = [MAPIStoreUserContext userContextWithUsername: userName
-                                               andTDBIndexing: indexing];
-  [userContext activateWithUser: [userContext sogoUser]];
+  // User context is activated on initialization
+  [MAPIStoreUserContext userContextWithUsername: userName
+                                 andTDBIndexing: indexing];
 
   classes = GSObjCAllSubclassesOfClass (self);
   max = [classes count];
@@ -121,7 +120,7 @@ static NSMutableDictionary *contextClassMapping;
     {
       currentClass = [classes objectAtIndex: count];
       current = [currentClass listContextsForUser: userName
-                                  withIndexing: indexing
+                                     withIndexing: indexing
                                          inMemCtx: memCtx];
       if (current)
         DLIST_CONCATENATE(list, current, void);
@@ -131,7 +130,7 @@ static NSMutableDictionary *contextClassMapping;
 }
 
 + (struct mapistore_contexts_list *) listContextsForUser: (NSString *) userName
-                                         withIndexing: (struct indexing_context *) indexing
+                                            withIndexing: (struct indexing_context *) indexing
                                                 inMemCtx: (TALLOC_CTX *) memCtx
 {
   return NULL;
@@ -213,18 +212,16 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   return completeURL;
 }
 
-+ (int) openContext: (MAPIStoreContext **) contextPtr
-            withURI: (const char *) newUri
-     connectionInfo: (struct mapistore_connection_info *) newConnInfo
-     andTDBIndexing: (struct indexing_context *) indexing
++ (enum mapistore_error) openContext: (MAPIStoreContext **) contextPtr
+                             withURI: (const char *) newUri
+                      connectionInfo: (struct mapistore_connection_info *) newConnInfo
+                      andTDBIndexing: (struct indexing_context *) indexing
 {
   MAPIStoreContext *context;
   Class contextClass;
   NSString *module;
   NSURL *baseURL;
-  int rc = MAPISTORE_ERR_NOT_FOUND;
-
-  //NSLog (@"METHOD '%s' (%d) -- uri: '%s'", __FUNCTION__, __LINE__, newUri);
+  enum mapistore_error rc = MAPISTORE_ERR_NOT_FOUND;
 
   context = nil;
 
@@ -276,6 +273,11 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
 {
   NSString *username;
 
+  if (newConnInfo == NULL)
+    {
+      return nil;
+    }
+
   if ((self = [self init]))
     {
       ASSIGN (contextUrl, newUrl);
@@ -292,13 +294,6 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
       ASSIGN (userContext,
               [MAPIStoreUserContext userContextWithUsername: username
                                              andTDBIndexing: indexing]);
-
-#if 0
-      mapistore_mgmt_backend_register_user (newConnInfo,
-                                            "SOGo",
-                                            [username UTF8String]);
-#endif
-
       connInfo = newConnInfo;
       username = [NSString stringWithUTF8String: newConnInfo->username];
       ASSIGN (activeUser, [SOGoUser userWithLogin: username]);
@@ -316,12 +311,6 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
 
 - (void) dealloc
 {
-#if 0
-  mapistore_mgmt_backend_unregister_user ([self connectionInfo], "SOGo", 
-                                          [[userContext username]
-                                            UTF8String]);
-#endif
-
   [contextUrl release];
   [userContext release];
   [containersBag release];
@@ -349,26 +338,16 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   return activeUser;
 }
 
-// - (void) logRestriction: (struct mapi_SRestriction *) res
-// 	      withState: (MAPIRestrictionState) state
-// {
-//   NSString *resStr;
-
-//   resStr = MAPIStringForRestriction (res);
-
-//   [self logWithFormat: @"%@  -->  %@", resStr, MAPIStringForRestrictionState (state)];
-// }
-
-- (int) getPath: (char **) path
-         ofFMID: (uint64_t) fmid
-       inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPath: (char **) path
+                          ofFMID: (uint64_t) fmid
+                        inMemCtx: (TALLOC_CTX *) memCtx
 {
-  int rc;
+  enum mapistore_error rc;
   NSString *objectURL, *url;
-  // TDB_DATA key, dbuf;
 
-  url = [[contextUrl absoluteString]
-            stringByReplacingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
+  url = [contextUrl absoluteString];
+  // FIXME transform percent escapes but not for user part of the url
+  //[xxxx stringByReplacingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
   objectURL = [[userContext mapping] urlFromID: fmid];
   if (objectURL)
     {
@@ -389,14 +368,6 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   else
     {
       [self errorWithFormat: @"%s: you should *never* get here", __PRETTY_FUNCTION__];
-      // /* attempt to populate our mapping dict with data from indexing.tdb */
-      // key.dptr = (unsigned char *) talloc_asprintf (memCtx, "0x%.16llx",
-      //                                               (long long unsigned int )fmid);
-      // key.dsize = strlen ((const char *) key.dptr);
-
-      // dbuf = tdb_fetch (memCtx->indexing_list->index_ctx->tdb, key);
-      // talloc_free (key.dptr);
-      // uri = talloc_strndup (memCtx, (const char *)dbuf.dptr, dbuf.dsize);
       *path = NULL;
       rc = MAPISTORE_SUCCESS;
     }
@@ -408,11 +379,10 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
 {
 }
 
-- (int) getRootFolder: (MAPIStoreFolder **) folderPtr
-              withFID: (uint64_t) newFid
+- (enum mapistore_error) getRootFolder: (MAPIStoreFolder **) folderPtr
+                               withFID: (uint64_t) newFid
 {
   enum mapistore_error rc;
-  MAPIStoreMapping *mapping;
   MAPIStoreFolder *baseFolder;
   SOGoFolder *currentFolder;
   WOContext *woContext;
@@ -420,11 +390,7 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   NSArray *pathComponents;
   NSUInteger count, max;
 
-  mapping = [userContext mapping];
-  if (![mapping urlFromID: newFid])
-    [mapping registerURL: [contextUrl absoluteString]
-                  withID: newFid];
-  [userContext activateWithUser: activeUser];
+  [userContext activate];
   woContext = [userContext woContext];
 
   [self ensureContextFolder];
@@ -472,8 +438,16 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
                      mapiStoreObjectWithSOGoObject: currentFolder
                                        inContainer: nil];
       [baseFolder setContext: self];
-      *folderPtr = baseFolder;
-      rc = MAPISTORE_SUCCESS;
+
+      if ([[userContext sogoUser] isEqual: activeUser]
+          || [baseFolder subscriberCanReadMessages])
+        {
+          *folderPtr = baseFolder;
+          rc = MAPISTORE_SUCCESS;
+        }
+      else
+        rc = MAPISTORE_ERR_DENIED;
+
     }
   else if ([[userContext sogoUser] isEqual: activeUser])
     rc = MAPISTORE_ERR_NOT_FOUND;
@@ -486,7 +460,7 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
 /* utils */
 
 - (NSString *) extractChildNameFromURL: (NSString *) objectURL
-			andFolderURLAt: (NSString **) folderURL;
+                        andFolderURLAt: (NSString **) folderURL;
 {
   NSString *childKey;
   NSRange lastSlash;
@@ -495,15 +469,15 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   if ([objectURL hasSuffix: @"/"])
     objectURL = [objectURL substringToIndex: [objectURL length] - 2];
   lastSlash = [objectURL rangeOfString: @"/"
-			       options: NSBackwardsSearch];
+                               options: NSBackwardsSearch];
   if (lastSlash.location != NSNotFound)
     {
       slashPtr = NSMaxRange (lastSlash);
       childKey = [objectURL substringFromIndex: slashPtr];
       if ([childKey length] == 0)
-	childKey = nil;
+        childKey = nil;
       if (folderURL)
-	*folderURL = [objectURL substringToIndex: slashPtr];
+        *folderURL = [objectURL substringToIndex: slashPtr];
     }
   else
     childKey = nil;
@@ -514,11 +488,10 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
 - (uint64_t) idForObjectWithKey: (NSString *) key
                     inFolderURL: (NSString *) folderURL
 {
-  NSString *childURL, *owner;
+  NSString *childURL;
   MAPIStoreMapping *mapping;
   uint64_t mappingId;
-  uint32_t contextId;
-  void *rootObject;
+  enum mapistore_error ret;
 
   if (key)
     childURL = [NSString stringWithFormat: @"%@%@", folderURL,
@@ -529,52 +502,71 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   mappingId = [mapping idFromURL: childURL];
   if (mappingId == NSNotFound)
     {
-      //[self warnWithFormat: @"no id exist yet for '%@', requesting one...",
-      //      childURL];
-      mapistore_indexing_get_new_folderID (connInfo->mstore_ctx, &mappingId);
-      [mapping registerURL: childURL withID: mappingId];
-      contextId = 0;
+      const char *owner;
 
-      mapistore_search_context_by_uri (connInfo->mstore_ctx, [folderURL UTF8String],
-                                       &contextId, &rootObject);
-      owner = [userContext username];
-      mapistore_indexing_record_add_mid (connInfo->mstore_ctx, contextId,
-                                         [owner UTF8String], mappingId);
+      [self logWithFormat: @"No id exist yet for '%@', requesting one", childURL];
+      owner = [[userContext username] UTF8String];
+      ret = mapistore_indexing_get_new_folderID_as_user (connInfo->mstore_ctx, owner, &mappingId);
+      if (ret == MAPISTORE_SUCCESS)
+        [mapping registerURL: childURL withID: mappingId];
+      else
+        [self errorWithFormat: @"Error trying to get new folder id (%d): %s",
+              ret, mapistore_errstr (ret)];
     }
 
   return mappingId;
 }
 
+/* Get new change number from openchange db interface using
+   resource's owner user */
 - (uint64_t) getNewChangeNumber
 {
+  const char *owner;
+  enum MAPISTATUS retval;
   uint64_t newVersionNumber;
 
-  if (openchangedb_get_new_changeNumber (connInfo->oc_ctx, &newVersionNumber)
-      != MAPI_E_SUCCESS)
-    abort ();
+  owner = [[userContext username] UTF8String];
+  retval = openchangedb_get_new_changeNumber (connInfo->oc_ctx, owner, &newVersionNumber);
+  if (retval != MAPI_E_SUCCESS)
+    [NSException raise: @"MAPIStoreIOException"
+                format: @"Impossible to get new change number for %s: %s", owner,
+                 mapi_get_errstr (retval)];
 
   return newVersionNumber;
 }
 
+/* Get new change numbers from openchange db interface using
+   resource's owner user */
 - (NSArray *) getNewChangeNumbers: (uint64_t) max
 {
+  const char *owner;
+  enum MAPISTATUS retval;
   TALLOC_CTX *memCtx;
   NSMutableArray *newChangeNumbers;
   uint64_t count;
   struct UI8Array_r *numbers;
   NSString *newNumber;
 
-  memCtx = talloc_zero(NULL, TALLOC_CTX);
+  memCtx = talloc_new (NULL);
+  if (!memCtx)
+    [NSException raise: @"MAPIStoreIOException"
+                format: @"Not enough memory to allocate change numbers"];
+
   newChangeNumbers = [NSMutableArray arrayWithCapacity: max];
-  
-  if (openchangedb_get_new_changeNumbers (connInfo->oc_ctx,
-                                          memCtx, max, &numbers)
-      != MAPI_E_SUCCESS || numbers->cValues != max)
-    abort ();
+  owner = [[userContext username] UTF8String];
+
+  retval = openchangedb_get_new_changeNumbers (connInfo->oc_ctx, memCtx, owner, max, &numbers);
+  if (retval != MAPI_E_SUCCESS || numbers->cValues != max)
+    {
+      talloc_free (memCtx);
+      [NSException raise: @"MAPIStoreIOException"
+                  format: @"Failing to get %d new change numbers: %s", max,
+                          mapi_get_errstr (retval)];
+    }
+
   for (count = 0; count < max; count++)
     {
-      newNumber
-        = [NSString stringWithUnsignedLongLong: numbers->lpui8[count]];
+      newNumber = [NSString stringWithUnsignedLongLong: numbers->lpui8[count]];
       [newChangeNumbers addObject: newNumber];
     }
 
@@ -583,30 +575,31 @@ static inline NSURL *CompleteURLFromMapistoreURI (const char *uri)
   return newChangeNumbers;
 }
 
+/* Get new fmids from mapistore_indexing interface using resource's
+   owner user */
 - (NSArray *) getNewFMIDs: (uint64_t) max
 {
-  TALLOC_CTX *memCtx;
+  const char *owner;
+  enum mapistore_error ret;
   NSMutableArray *newFMIDs;
-  uint64_t count;
-  struct UI8Array_r *numbers;
   NSString *newNumber;
+  uint64_t count, newFID;
 
-  memCtx = talloc_zero(NULL, TALLOC_CTX);
   newFMIDs = [NSMutableArray arrayWithCapacity: max];
-  
-  if (mapistore_indexing_get_new_folderIDs (connInfo->mstore_ctx,
-                                            memCtx, max, &numbers)
-      != MAPISTORE_SUCCESS || numbers->cValues != max)
-    abort ();
+  /* Get the resource's owner name */
+  owner = [[userContext username] UTF8String];
+
   for (count = 0; count < max; count++)
     {
-      newNumber
-        = [NSString stringWithUnsignedLongLong: numbers->lpui8[count]];
+      ret = mapistore_indexing_get_new_folderID_as_user (connInfo->mstore_ctx, owner, &newFID);
+      if (ret != MAPISTORE_SUCCESS)
+          [NSException raise: @"MAPIStoreIOException"
+                      format: @"Impossible to get new fmid for %s", owner];
+
+      newNumber = [NSString stringWithUnsignedLongLong: newFID];
       [newFMIDs addObject: newNumber];
     }
 
-  talloc_free (memCtx);
-
   return newFMIDs;
 }
 
