@@ -67,7 +67,7 @@ MakeDisplayFolderName (NSString *folderName)
   NSArray *parts;
   NSString *lastFolder;
   NSUInteger max;
- 
+
   parts = [folderName componentsSeparatedByString: @"/"];
   max = [parts count];
   if (max > 1)
@@ -87,7 +87,7 @@ MakeDisplayFolderName (NSString *folderName)
                                                 inMemCtx: (TALLOC_CTX *) memCtx
 {
   struct mapistore_contexts_list *firstContext = NULL, *context;
-  NSString *urlBase, *stringData, *currentName, *inboxName, *draftsName, *sentName, *trashName;
+  NSString *urlBase, *stringData, *currentName, *realName, *inboxName, *draftsName, *sentName, *trashName;
   NSArray *unprefixedFolders;
   NSMutableArray *secondaryFolders;
   enum mapistore_context_role role[] = {MAPISTORE_MAIL_ROLE,
@@ -126,7 +126,11 @@ MakeDisplayFolderName (NSString *folderName)
   trashName = [NSString stringWithFormat: @"folder%@",
                        [unprefixedFolders componentsJoinedByString: @"/folder"]];
 
-  urlBase = [NSString stringWithFormat: @"sogo://%@:%@@mail/", userName, userName];
+  urlBase = [NSString stringWithFormat: @"sogo://%@:%@@mail/",
+                      [userName stringByReplacingOccurrencesOfString: @"@"
+                                withString: @"%40"],
+                      [userName stringByReplacingOccurrencesOfString: @"@"
+                                withString: @"%40"]];
   for (count = 0; count < 3; count++)
     {
       context = talloc_zero (memCtx, struct mapistore_contexts_list);
@@ -142,6 +146,9 @@ MakeDisplayFolderName (NSString *folderName)
       DLIST_ADD_END (firstContext, context, void);
     }
 
+  /* FIXME: Flush any cache before retrieving the hierarchy */
+  [accountFolder flushMailCaches];
+
   secondaryFolders = [[accountFolder toManyRelationshipKeysWithNamespaces: NO]
                        mutableCopy];
   [secondaryFolders autorelease];
@@ -153,12 +160,25 @@ MakeDisplayFolderName (NSString *folderName)
   for (count = 0; count < max; count++)
     {
       context = talloc_zero (memCtx, struct mapistore_contexts_list);
+      // secondaryFolders has the names (1) Imap4Encoded ,(2) asCSSIdentifier and (3) "folder"-prefixed
+      //     e.g.: folderProbl&AOg-mes_SP_de_SP_synchronisation
       currentName = [secondaryFolders objectAtIndex: count];
-      stringData = [NSString stringWithFormat: @"%@%@",
-                             urlBase, [currentName stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding]];
-      context->url = [stringData asUnicodeInMemCtx: context];
-      stringData = [[[currentName substringFromIndex: 6] fromCSSIdentifier] stringByDecodingImap4FolderName];
-      context->name = [stringData asUnicodeInMemCtx: context];
+      // To get the real name we have to revert that (applying the decode functions
+      // in reverse order)
+      //     e.g.: Problèmes de synchronisation
+      realName = [[[currentName substringFromIndex: 6]
+                                fromCSSIdentifier]
+                                stringByDecodingImap4FolderName];
+      // And finally to represent that as URI we have to (1) asCSSIdentifier,
+      // (2) Imap4Encode (3) AddPercentEscapes and (4) add the "folder" prefix
+      //     e.g.: folderProbl&AOg-mes_SP_de_SP_synchronisation
+      // In the example there are no percent escapes added because is already ok
+      stringData = [[[realName asCSSIdentifier]
+                               stringByEncodingImap4FolderName]
+                               stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
+      stringData = [NSString stringWithFormat: @"folder%@", stringData];
+      context->url = [[NSString stringWithFormat: @"%@%@", urlBase, stringData] asUnicodeInMemCtx: context];
+      context->name = [realName asUnicodeInMemCtx: context];
       context->main_folder = false;
       context->role = MAPISTORE_MAIL_ROLE;
       context->tag = "tag";
@@ -180,19 +200,20 @@ MakeDisplayFolderName (NSString *folderName)
 
   userContext = [MAPIStoreUserContext userContextWithUsername: userName
                                                andTDBIndexing: NULL];
-  [MAPIApp setUserContext: userContext];
   accountFolder = [[userContext rootFolders] objectForKey: @"mail"];
   folderName = [NSString stringWithFormat: @"folder%@",
-                         [newFolderName asCSSIdentifier]];
+                         [[newFolderName stringByEncodingImap4FolderName] asCSSIdentifier]];
   newFolder = [SOGoMailFolder objectWithName: folderName
                                  inContainer: accountFolder];
   if ([newFolder create])
     mapistoreURI = [NSString stringWithFormat: @"sogo://%@:%@@mail/%@/",
-                             userName, userName,
-                             [[folderName stringByEncodingImap4FolderName] stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding]];
+                             [userName stringByReplacingOccurrencesOfString: @"@"
+                                       withString: @"%40"],
+                             [userName stringByReplacingOccurrencesOfString: @"@"
+                                       withString: @"%40"],
+                             [folderName stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding]];
   else
     mapistoreURI = nil;
-  [MAPIApp setUserContext: nil];
 
   return mapistoreURI;
 }
@@ -264,8 +285,12 @@ MakeDisplayFolderName (NSString *folderName)
                         componentsSeparatedByString: @"/"];
   folderName = [NSString stringWithFormat: @"folder%@",
                          [unprefixedFolders componentsJoinedByString: @"/folder"]];
-  url = [NSString stringWithFormat: @"sogo://%@:%@@outbox/%@", userName,
-                  userName, folderName];
+  url = [NSString stringWithFormat: @"sogo://%@:%@@outbox/%@",
+                  [userName stringByReplacingOccurrencesOfString: @"@"
+                            withString: @"%40"],
+                  [userName stringByReplacingOccurrencesOfString: @"@"
+                            withString: @"%40"],
+                  folderName];
 
   context = talloc_zero (memCtx, struct mapistore_contexts_list);
   context->url = [url asUnicodeInMemCtx: context];
