@@ -84,16 +84,16 @@
   return memberId;
 }
 
-- (int) getPidTagMemberId: (void **) data
-                 inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagMemberId: (void **) data
+                                  inMemCtx: (TALLOC_CTX *) memCtx
 {
   *data = MAPILongLongValue (memCtx, memberId);
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagEntryId: (void **) data
-                inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagEntryId: (void **) data
+                                 inMemCtx: (TALLOC_CTX *) memCtx
 {
   NSData *entryId;
   struct mapistore_connection_info *connInfo;
@@ -103,15 +103,15 @@
   else
     {
       connInfo = [(MAPIStoreContext *) [container context] connectionInfo];
-      entryId = MAPIStoreInternalEntryId (connInfo->sam_ctx, userId);
+      entryId = MAPIStoreInternalEntryId (connInfo, userId);
     }
   *data = [entryId asBinaryInMemCtx: memCtx];
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagMemberName: (void **) data
-                   inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagMemberName: (void **) data
+                                    inMemCtx: (TALLOC_CTX *) memCtx
 {
   NSString *displayName;
 
@@ -127,13 +127,14 @@
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagMemberRights: (void **) data
-                     inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagMemberRights: (void **) data
+                                      inMemCtx: (TALLOC_CTX *) memCtx
 {
   uint32_t rights;
   NSArray *roles;
 
   roles = [[(MAPIStoreFolder *) container aclFolder] aclsForUser: userId];
+  roles = [(MAPIStoreFolder *) container expandRoles: roles];
   rights = [(MAPIStoreFolder *) container exchangeRightsForRoles: roles];
 
   *data = MAPILongValue (memCtx, rights);
