@@ -47,6 +47,7 @@
 #import "NSObject+MAPIStore.h"
 #import "NSString+MAPIStore.h"
 
+#include <libmapi/libmapi.h>
 #include <mapistore/mapistore.h>
 #include <mapistore/mapistore_errors.h>
 #include <execinfo.h>
@@ -58,22 +59,27 @@ static BOOL initialization_done = NO;
 #define NS_CURRENT_THREAD_REGISTER() \
     BOOL __nsrct_thread_registered = GSRegisterCurrentThread(); \
     if (!initialization_done) { \
-        DEBUG(5, ("[SOGo: %s:%d] You should call sogo_backend_init() first. Current thread: %p, pid: %d\n", \
-		  __FUNCTION__, __LINE__, GSCurrentThread(), getpid())); \
-    }
-#define NS_CURRENT_THREAD_TRY_UNREGISTER() \
+        OC_DEBUG(5, "[SOGo] You should call sogo_backend_init() first. Current thread: %p, pid: %d\n", \
+                 GSCurrentThread(), getpid());                         \
+    } \
+    OC_DEBUG(5, "[SOGo] --->");
+
+#define NS_CURRENT_THREAD_TRY_UNREGISTER(rc) \
     if (__nsrct_thread_registered) { \
         GSUnregisterCurrentThread(); \
-    }
+    } \
+    OC_DEBUG(6, "[SOGo] <--- [%s]", mapistore_errstr (rc));
 
 #define TRYCATCH_START @try {
 #define TRYCATCH_END(pool)  \
           } @catch (NSException * e) { \
             enum mapistore_error ret = sogo_backend_handle_objc_exception(e, __PRETTY_FUNCTION__, __LINE__); \
+            mapiapp_cleanup(); \
             [pool release]; \
-            NS_CURRENT_THREAD_TRY_UNREGISTER(); \
+            NS_CURRENT_THREAD_TRY_UNREGISTER(ret); \
             return ret; \
-          }
+          } \
+        mapiapp_cleanup();
 
 
 static enum mapistore_error
@@ -140,11 +146,13 @@ sogo_backend_init (void)
   NSUserDefaults *ud;
   SoProductRegistry *registry;
   char *argv[] = { SAMBA_PREFIX "/sbin/samba", NULL };
+  NSString *debugLevel;
+  uint8_t parentLogLevel;
 
   GSRegisterCurrentThread();
 
   if (initialization_done) {
-      DEBUG(0, ("SOGo backend already initialized.\n"));
+      OC_DEBUG(5, "[SOGo] SOGo backend already initialized.\n");
       return MAPISTORE_SUCCESS;
   }
 
@@ -164,6 +172,11 @@ sogo_backend_init (void)
 
   /* We force the plugin to base its configuration on the SOGo tree. */
   ud = [NSUserDefaults standardUserDefaults];
+
+  /* Ensure imap4Connection calls raise Exception if
+    IMAP connection is not established. See NGImap4Connection.m */
+  [ud setBool: YES forKey: @"SoIMAP4ExceptionsEnabled"];
+
   if (!leakDebugging && [ud boolForKey: @"SOGoDebugLeaks"])
     {
       NSLog (@"  leak debugging on");
@@ -172,6 +185,22 @@ sogo_backend_init (void)
       leakDebugging = YES;
     }
 
+  /* Set debug level according to samba */
+  parentLogLevel = DEBUGLEVEL_CLASS[DBGC_ALL]; // FIXME: samba logger specific code
+  if (parentLogLevel >= 4)
+    debugLevel = @"DEBUG";
+  else if (parentLogLevel >= 3)
+    debugLevel = @"INFO";
+  else if (parentLogLevel >= 2)
+    debugLevel = @"WARN";
+  else if (parentLogLevel >= 1)
+    debugLevel = @"ERROR";
+  else
+    debugLevel = @"FATAL";
+  OC_DEBUG(3, "[SOGo] Setting log level to %s", [debugLevel UTF8String]);
+  [ud setObject: debugLevel forKey: @"NGLogDefaultLogLevel"];
+  [ud synchronize];
+
   registry = [SoProductRegistry sharedProductRegistry];
   [registry scanForProductsInDirectory: SOGO_BUNDLES_DIR];
 
@@ -189,18 +218,39 @@ sogo_backend_init (void)
 
   [pool release];
 
-  DEBUG(0, ("[SOGo: %s:%d] backend init SUCCESS. Current thread: %p, pid: %d\n", __FUNCTION__, __LINE__, GSCurrentThread(), getpid()));
+  OC_DEBUG(5, "[SOGo] backend init SUCCESS. Current thread: %p, pid: %d\n", GSCurrentThread(), getpid());
   initialization_done = YES;
 
   return MAPISTORE_SUCCESS;
 }
 
+/**
+   \details Cleanup operation to execute after an action has been performed
+   so there won't be any conflicts with future calls.
+   In practice this will deactivate the current user context set on MAPIApp
+   (which is the current WOApplication), this means two things: (1) set nil
+   as current user context on MAPIApp and (2) remove woContext from current
+   thread dictionary (this is used on WOContext.m).
+*/
+static void mapiapp_cleanup(void)
+{
+  Class MAPIApplicationK;
+  MAPIApplicationK = NSClassFromString (@"MAPIApplication");
+  if (MAPIApplicationK)
+    [[MAPIApplicationK application] cleanup];
+}
+
 /**
    \details Create a connection context to the sogo backend
 
    \param mem_ctx pointer to the memory context
+   \param conn_info pointer to the connection information available for this context
+                    (database connection, connected user, replica server info)
+   \param indexing pointer to the indexing database connection
    \param uri pointer to the sogo path
-   \param private_data pointer to the private backend context 
+   \param private_data pointer to the private backend context
+
+   \note the developer must free allocated private_data
 */
 
 static enum mapistore_error
@@ -211,9 +261,7 @@ sogo_backend_create_context(TALLOC_CTX *mem_ctx,
 {
   NSAutoreleasePool *pool;
   MAPIStoreContext *context;
-  int rc;
-
-  DEBUG(0, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   NS_CURRENT_THREAD_REGISTER();
   pool = [NSAutoreleasePool new];
@@ -233,7 +281,7 @@ sogo_backend_create_context(TALLOC_CTX *mem_ctx,
     rc = MAPISTORE_ERROR;
 
   [pool release];
-  NS_CURRENT_THREAD_TRY_UNREGISTER();
+  NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
 
   return rc;
 }
@@ -248,9 +296,7 @@ sogo_backend_create_root_folder (const char *username,
   NSAutoreleasePool *pool;
   NSString *userName, *folderName;
   NSString *mapistoreUri;
-  int rc;
-
-  DEBUG(0, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   NS_CURRENT_THREAD_REGISTER();
   pool = [NSAutoreleasePool new];
@@ -273,7 +319,7 @@ sogo_backend_create_root_folder (const char *username,
     rc = MAPISTORE_ERROR;
 
   [pool release];
-  NS_CURRENT_THREAD_TRY_UNREGISTER();
+  NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
 
   return rc;
 }
@@ -285,9 +331,7 @@ sogo_backend_list_contexts(const char *username, struct indexing_context *indexi
 {
   NSAutoreleasePool *pool;
   NSString *userName;
-  int rc;
-
-  DEBUG(0, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   NS_CURRENT_THREAD_REGISTER();
   pool = [NSAutoreleasePool new];
@@ -306,7 +350,7 @@ sogo_backend_list_contexts(const char *username, struct indexing_context *indexi
     rc = MAPISTORE_ERROR;
 
   [pool release];
-  NS_CURRENT_THREAD_TRY_UNREGISTER();
+  NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
 
   return rc;
 }
@@ -333,9 +377,7 @@ sogo_context_get_path(void *backend_object, TALLOC_CTX *mem_ctx,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreContext *context;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (backend_object)
     {
@@ -349,7 +391,7 @@ sogo_context_get_path(void *backend_object, TALLOC_CTX *mem_ctx,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -367,9 +409,7 @@ sogo_context_get_root_folder(void *backend_object, TALLOC_CTX *mem_ctx,
   NSAutoreleasePool *pool;
   MAPIStoreContext *context;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (backend_object)
     {
@@ -385,7 +425,7 @@ sogo_context_get_root_folder(void *backend_object, TALLOC_CTX *mem_ctx,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -410,9 +450,7 @@ sogo_folder_open_folder(void *folder_object, TALLOC_CTX *mem_ctx, uint64_t fid,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder, *childFolder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -429,7 +467,7 @@ sogo_folder_open_folder(void *folder_object, TALLOC_CTX *mem_ctx, uint64_t fid,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -441,7 +479,7 @@ sogo_folder_open_folder(void *folder_object, TALLOC_CTX *mem_ctx, uint64_t fid,
 
 /**
    \details Create a folder in the sogo backend
-   
+
    \param private_data pointer to the current sogo context
 
    \return MAPISTORE_SUCCESS on success, otherwise MAPISTORE_ERROR
@@ -454,9 +492,7 @@ sogo_folder_create_folder(void *folder_object, TALLOC_CTX *mem_ctx,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder, *childFolder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -472,7 +508,7 @@ sogo_folder_create_folder(void *folder_object, TALLOC_CTX *mem_ctx,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -497,9 +533,7 @@ sogo_folder_delete(void *folder_object)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -513,7 +547,7 @@ sogo_folder_delete(void *folder_object)
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -529,9 +563,7 @@ sogo_folder_get_child_count(void *folder_object, enum mapistore_table_type table
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -545,7 +577,7 @@ sogo_folder_get_child_count(void *folder_object, enum mapistore_table_type table
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -565,9 +597,7 @@ sogo_folder_open_message(void *folder_object,
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
   MAPIStoreMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -586,7 +616,7 @@ sogo_folder_open_message(void *folder_object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -607,9 +637,7 @@ sogo_folder_create_message(void *folder_object,
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
   MAPIStoreMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -621,13 +649,13 @@ sogo_folder_create_message(void *folder_object,
       TRYCATCH_START
       rc = [folder createMessage: &message
                          withMID: mid
-		    isAssociated: associated];
+                    isAssociated: associated];
       if (rc == MAPISTORE_SUCCESS)
         *message_object = [message tallocWrapper: mem_ctx];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -643,9 +671,7 @@ sogo_folder_delete_message(void *folder_object, uint64_t mid, uint8_t flags)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -659,7 +685,7 @@ sogo_folder_delete_message(void *folder_object, uint64_t mid, uint8_t flags)
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -676,14 +702,13 @@ sogo_folder_move_copy_messages(void *folder_object,
                                uint32_t mid_count,
                                uint64_t *src_mids, uint64_t *t_mids,
                                struct Binary_r **target_change_keys,
+                               struct Binary_r **target_predecessor_change_lists,
                                uint8_t want_copy)
 {
   MAPIStoreFolder *sourceFolder, *targetFolder;
   NSAutoreleasePool *pool;
   struct MAPIStoreTallocWrapper *wrapper;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -702,12 +727,13 @@ sogo_folder_move_copy_messages(void *folder_object,
                                        fromFolder: sourceFolder
                                          withMIDs: t_mids
                                     andChangeKeys: target_change_keys
+                        andPredecessorChangeLists: target_predecessor_change_lists
                                          wantCopy: want_copy
                                          inMemCtx: mem_ctx];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -725,9 +751,7 @@ sogo_folder_move_folder(void *folder_object, void *target_folder_object,
   MAPIStoreFolder *moveFolder, *targetFolder;
   NSString *newFolderName;
   struct MAPIStoreTallocWrapper *wrapper;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -757,7 +781,7 @@ sogo_folder_move_folder(void *folder_object, void *target_folder_object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -775,9 +799,7 @@ sogo_folder_copy_folder(void *folder_object, void *target_folder_object, TALLOC_
   MAPIStoreFolder *copyFolder, *targetFolder;
   NSString *newFolderName;
   struct MAPIStoreTallocWrapper *wrapper;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -801,7 +823,7 @@ sogo_folder_copy_folder(void *folder_object, void *target_folder_object, TALLOC_
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -819,9 +841,7 @@ sogo_folder_get_deleted_fmids(void *folder_object, TALLOC_CTX *mem_ctx,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -839,7 +859,7 @@ sogo_folder_get_deleted_fmids(void *folder_object, TALLOC_CTX *mem_ctx,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -858,9 +878,7 @@ sogo_folder_open_table(void *folder_object, TALLOC_CTX *mem_ctx,
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -879,7 +897,7 @@ sogo_folder_open_table(void *folder_object, TALLOC_CTX *mem_ctx,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -897,9 +915,7 @@ sogo_folder_modify_permissions(void *folder_object, uint8_t flags,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -915,7 +931,7 @@ sogo_folder_modify_permissions(void *folder_object, uint8_t flags,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -931,9 +947,7 @@ sogo_folder_preload_message_bodies(void *folder_object, enum mapistore_table_typ
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreFolder *folder;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (folder_object)
     {
@@ -948,7 +962,7 @@ sogo_folder_preload_message_bodies(void *folder_object, enum mapistore_table_typ
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -966,9 +980,7 @@ sogo_message_get_message_data(void *message_object,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -983,8 +995,8 @@ sogo_message_get_message_data(void *message_object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
       rc = MAPISTORE_SUCCESS;
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1001,9 +1013,7 @@ sogo_message_create_attachment (void *message_object, TALLOC_CTX *mem_ctx, void
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
   MAPIStoreAttachment *attachment;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1016,11 +1026,10 @@ sogo_message_create_attachment (void *message_object, TALLOC_CTX *mem_ctx, void
       rc = [message createAttachment: &attachment inAID: aidp];
       if (rc == MAPISTORE_SUCCESS)
         *attachment_object = [attachment tallocWrapper: mem_ctx];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1038,9 +1047,7 @@ sogo_message_open_attachment (void *message_object, TALLOC_CTX *mem_ctx,
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
   MAPIStoreAttachment *attachment;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1053,11 +1060,10 @@ sogo_message_open_attachment (void *message_object, TALLOC_CTX *mem_ctx,
       rc = [message getAttachment: &attachment withAID: aid];
       if (rc == MAPISTORE_SUCCESS)
         *attachment_object = [attachment tallocWrapper: mem_ctx];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1074,9 +1080,7 @@ sogo_message_get_attachment_table (void *message_object, TALLOC_CTX *mem_ctx, vo
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
   MAPIStoreAttachmentTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1090,11 +1094,10 @@ sogo_message_get_attachment_table (void *message_object, TALLOC_CTX *mem_ctx, vo
                            andRowCount: row_count];
       if (rc == MAPISTORE_SUCCESS)
         *table_object = [table tallocWrapper: mem_ctx];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1113,9 +1116,7 @@ sogo_message_modify_recipients (void *message_object,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1128,11 +1129,10 @@ sogo_message_modify_recipients (void *message_object,
       rc = [message modifyRecipientsWithRecipients: recipients
                                           andCount: count
                                         andColumns: columns];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1148,9 +1148,7 @@ sogo_message_set_read_flag (void *message_object, uint8_t flag)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1161,11 +1159,10 @@ sogo_message_set_read_flag (void *message_object, uint8_t flag)
 
       TRYCATCH_START
       rc = [message setReadFlag: flag];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1181,9 +1178,7 @@ sogo_message_save (void *message_object, TALLOC_CTX *mem_ctx)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1194,11 +1189,10 @@ sogo_message_save (void *message_object, TALLOC_CTX *mem_ctx)
 
       TRYCATCH_START
       rc = [message saveMessage: mem_ctx];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1214,9 +1208,7 @@ sogo_message_submit (void *message_object, enum SubmitFlags flags)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreMailVolatileMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (message_object)
     {
@@ -1227,11 +1219,10 @@ sogo_message_submit (void *message_object, enum SubmitFlags flags)
 
       TRYCATCH_START
       rc = [message submitWithFlags: flags];
-      // [context tearDownRequest];
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1252,9 +1243,7 @@ sogo_message_attachment_open_embedded_message (void *attachment_object,
   NSAutoreleasePool *pool;
   MAPIStoreAttachment *attachment;
   MAPIStoreEmbeddedMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (attachment_object)
     {
@@ -1273,7 +1262,7 @@ sogo_message_attachment_open_embedded_message (void *attachment_object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1293,9 +1282,7 @@ sogo_message_attachment_create_embedded_message (void *attachment_object,
   NSAutoreleasePool *pool;
   MAPIStoreAttachment *attachment;
   MAPIStoreEmbeddedMessage *message;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (attachment_object)
     {
@@ -1313,7 +1300,7 @@ sogo_message_attachment_create_embedded_message (void *attachment_object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1329,9 +1316,7 @@ static enum mapistore_error sogo_table_get_available_properties(void *table_obje
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1345,7 +1330,7 @@ static enum mapistore_error sogo_table_get_available_properties(void *table_obje
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1361,9 +1346,7 @@ sogo_table_set_columns (void *table_object, uint16_t count, enum MAPITAGS *prope
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1378,7 +1361,7 @@ sogo_table_set_columns (void *table_object, uint16_t count, enum MAPITAGS *prope
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1394,9 +1377,7 @@ sogo_table_set_restrictions (void *table_object, struct mapi_SRestriction *restr
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1413,7 +1394,7 @@ sogo_table_set_restrictions (void *table_object, struct mapi_SRestriction *restr
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1429,9 +1410,7 @@ sogo_table_set_sort_order (void *table_object, struct SSortOrderSet *sort_order,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1448,7 +1427,7 @@ sogo_table_set_sort_order (void *table_object, struct SSortOrderSet *sort_order,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1466,9 +1445,7 @@ sogo_table_get_row (void *table_object, TALLOC_CTX *mem_ctx,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1483,7 +1460,7 @@ sogo_table_get_row (void *table_object, TALLOC_CTX *mem_ctx,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1501,9 +1478,7 @@ sogo_table_get_row_count (void *table_object,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1518,7 +1493,7 @@ sogo_table_get_row_count (void *table_object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1534,9 +1509,7 @@ sogo_table_handle_destructor (void *table_object, uint32_t handle_id)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreTable *table;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (table_object)
     {
@@ -1550,8 +1523,8 @@ sogo_table_handle_destructor (void *table_object, uint32_t handle_id)
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
       rc = MAPISTORE_SUCCESS;
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1568,9 +1541,7 @@ static enum mapistore_error sogo_properties_get_available_properties(void *objec
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreObject *propObject;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (object)
     {
@@ -1584,7 +1555,7 @@ static enum mapistore_error sogo_properties_get_available_properties(void *objec
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1603,9 +1574,7 @@ sogo_properties_get_properties (void *object,
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreObject *propObject;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (object)
     {
@@ -1621,7 +1590,7 @@ sogo_properties_get_properties (void *object,
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1637,9 +1606,7 @@ sogo_properties_set_properties (void *object, struct SRow *aRow)
   struct MAPIStoreTallocWrapper *wrapper;
   NSAutoreleasePool *pool;
   MAPIStoreObject *propObject;
-  int rc;
-
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
+  enum mapistore_error rc;
 
   if (object)
     {
@@ -1653,7 +1620,7 @@ sogo_properties_set_properties (void *object, struct SRow *aRow)
       TRYCATCH_END(pool)
 
       [pool release];
-      NS_CURRENT_THREAD_TRY_UNREGISTER();
+      NS_CURRENT_THREAD_TRY_UNREGISTER(rc);
     }
   else
     {
@@ -1664,18 +1631,16 @@ sogo_properties_set_properties (void *object, struct SRow *aRow)
 }
 
 static enum mapistore_error
-sogo_manager_generate_uri (TALLOC_CTX *mem_ctx, 
-                           const char *user, 
-                           const char *folder, 
-                           const char *message, 
+sogo_manager_generate_uri (TALLOC_CTX *mem_ctx,
+                           const char *user,
+                           const char *folder,
+                           const char *message,
                            const char *rootURI,
                            char **uri)
 {
   NSAutoreleasePool *pool;
   NSString *partialURLString, *username, *directory;
 
-  DEBUG (5, ("[SOGo: %s:%d]\n", __FUNCTION__, __LINE__));
-
   if (uri == NULL) return MAPISTORE_ERR_INVALID_PARAMETER;
 
   /* This fixes a crash occurring during the instantiation of the
@@ -1693,7 +1658,10 @@ sogo_manager_generate_uri (TALLOC_CTX *mem_ctx,
       username = [NSString stringWithUTF8String: (user ? user : "*")];
       /* Do proper directory lookup here */
       directory = [NSString stringWithUTF8String: (folder ? folder : "*")];
-      partialURLString = [NSString stringWithFormat: @"sogo://%@:*@%@", username, directory];
+      partialURLString = [NSString stringWithFormat: @"sogo://%@:*@%@",
+                                   [username stringByReplacingOccurrencesOfString: @"@"
+                                             withString: @"%40"],
+                                   directory];
     }
   if (![partialURLString hasSuffix: @"/"])
     partialURLString = [partialURLString stringByAppendingString: @"/"];
@@ -1706,7 +1674,7 @@ sogo_manager_generate_uri (TALLOC_CTX *mem_ctx,
   TRYCATCH_END(pool)
 
   [pool release];
-  NS_CURRENT_THREAD_TRY_UNREGISTER();
+  NS_CURRENT_THREAD_TRY_UNREGISTER(MAPISTORE_SUCCESS);
 
   return MAPISTORE_SUCCESS;
 }
@@ -1716,11 +1684,11 @@ sogo_manager_generate_uri (TALLOC_CTX *mem_ctx,
 
    \return MAPISTORE_SUCCESS on success, otherwise MAPISTORE error
 */
-int mapistore_init_backend(void)
+enum mapistore_error mapistore_init_backend(void)
 {
-  struct mapistore_backend	backend;
-  int				ret;
-  static BOOL                   registered = NO;
+  struct mapistore_backend backend;
+  enum mapistore_error ret;
+  static BOOL registered = NO;
 
   if (registered)
     ret = MAPISTORE_SUCCESS;
@@ -1776,7 +1744,7 @@ int mapistore_init_backend(void)
       /* Register ourselves with the MAPISTORE subsystem */
       ret = mapistore_backend_register (&backend);
       if (ret != MAPISTORE_SUCCESS)
-        DEBUG(0, ("Failed to register the '%s' mapistore backend!\n", backend.backend.name));
+        OC_DEBUG(0, "[SOGo] Failed to register the '%s' mapistore backend!\n", backend.backend.name);
     }
 
   return ret;
