@@ -1,8 +1,6 @@
 /* MAPIStoreUserContext.m - this file is part of SOGo
  *
- * Copyright (C) 2012 Inverse inc
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2012-2016 Inverse inc
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,10 +33,12 @@
 #import <NGImap4/NGImap4Connection.h>
 
 #import <GDLContentStore/GCSChannelManager.h>
+#import <GDLContentStore/GCSFolderManager.h>
 #import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserFolder.h>
 #import <SOGo/NSString+Utilities.h>
+#import <NGExtensions/NSObject+Logs.h>
 #import <Mailer/SOGoMailAccount.h>
 #import <Mailer/SOGoMailAccounts.h>
 
@@ -72,6 +72,8 @@ static NSMapTable *contextsTable = nil;
       [userContext autorelease];
       [contextsTable setObject: userContext forKey: username];
     }
+  else
+    [userContext activate];
 
   return userContext;
 }
@@ -158,6 +160,8 @@ static NSMapTable *contextsTable = nil;
       if ([userPassword length] == 0)
         userPassword = username;
       [authenticator setPassword: userPassword];
+      // Activate the profile on initialization
+      [self activate];
     }
 
   return self;
@@ -214,7 +218,7 @@ static NSMapTable *contextsTable = nil;
   if (!userFolder)
     {
        userFolder = [SOGoUserFolder objectWithName: username
-                                       inContainer: MAPIApp];
+                                       inContainer: nil];
        [userFolder retain];
     }
 
@@ -232,7 +236,8 @@ static NSMapTable *contextsTable = nil;
   if (!rootFolders)
     {
       rootFolders = [NSMutableDictionary new];
-      [self userFolder];
+      [self activate];
+      [self userFolder]; // force lazy initialization
       [woContext setClientObject: userFolder];
 
       /* Calendar */
@@ -257,10 +262,10 @@ static NSMapTable *contextsTable = nil;
                                       acquire: NO];
       [containersBag addObject: accountsFolder];
       [woContext setClientObject: accountsFolder];
+
       currentFolder = [accountsFolder lookupName: @"0"
                                        inContext: woContext
                                          acquire: NO];
-
       [rootFolders setObject: currentFolder
                       forKey: @"mail"];
       connection = [currentFolder imap4Connection];
@@ -268,8 +273,7 @@ static NSMapTable *contextsTable = nil;
 
       /* ensure the folder cache is filled */
       [currentFolder toManyRelationshipKeysWithNamespaces: YES];
-      hierarchy = [connection
-                    cachedHierarchyResultsForURL: [currentFolder imap4URL]];
+      hierarchy = [connection cachedHierarchyResultsForURL: [currentFolder imap4URL]];
       flags = [[hierarchy  objectForKey: @"list"] objectForKey: @"/INBOX"];
       inboxHasNoInferiors = [flags containsObject: @"noinferiors"];
     }
@@ -311,9 +315,9 @@ static NSMapTable *contextsTable = nil;
              parts in this url. We strip the '-' character in case we have
              this in the domain part - like foo@bar-zot.com */
           ocFSTableName = [NSString stringWithFormat: @"sogo_cache_folder_%@",
-                                    [[[user loginInDomain] asCSSIdentifier] 
-                                      stringByReplacingOccurrencesOfString: @"-"
-                                                                withString: @"_"]];
+                                    [[[user login] asCSSIdentifier]
+                                     stringByReplacingOccurrencesOfString: @"-"
+                                                               withString: @"_"]];
           [parts replaceObjectAtIndex: 4 withObject: ocFSTableName];
           folderTableURL
             = [NSURL URLWithString: [parts componentsJoinedByString: @"/"]];
@@ -334,11 +338,14 @@ static NSMapTable *contextsTable = nil;
   NSString *tableName, *query;
   GCSSpecialQueries *queries;
 
+  if ([GCSFolderManager singleStoreMode])
+    return;
+
   [self folderTableURL];
 
   cm = [GCSChannelManager defaultChannelManager];
   channel = [cm acquireOpenChannelForURL: folderTableURL];
-  
+
   /* FIXME: make use of [EOChannelAdaptor describeTableNames] instead */
   tableName = [[folderTableURL path] lastPathComponent];
   if ([channel evaluateExpressionX:
@@ -355,7 +362,7 @@ static NSMapTable *contextsTable = nil;
     [channel cancelFetch];
 
 
-  [cm releaseChannel: channel]; 
+  [cm releaseChannel: channel];
 }
 
 /* SOGo context objects */
@@ -369,6 +376,11 @@ static NSMapTable *contextsTable = nil;
   return authenticator;
 }
 
+- (void) activate
+{
+  [self activateWithUser: [self sogoUser]];
+}
+
 - (void) activateWithUser: (SOGoUser *) activeUser;
 {
   NSMutableDictionary *info;
@@ -379,4 +391,24 @@ static NSMapTable *contextsTable = nil;
   [info setObject: woContext forKey: @"WOContext"];
 }
 
+- (void) deactivate
+{
+  NSMutableDictionary *info;
+
+  if (self == [MAPIApp userContext])
+    [MAPIApp setUserContext: nil];
+  else
+    [self errorWithFormat: @"Error: Tried to deactivate an user context "
+                           @"not enabled (%@ vs %@)",
+                           [self username], [[MAPIApp userContext] username]];
+
+  info = [[NSThread currentThread] threadDictionary];
+  if (woContext == [info objectForKey: @"WOContext"])
+    [info removeObjectForKey: @"WOContext"];
+  else
+    [self errorWithFormat: @"Error: Tried to deactivate a WOContext "
+                           @"not enabled (%@ vs %@)",
+                           woContext, [info objectForKey: @"WOContext"]];
+}
+
 @end
