@@ -1818,6 +1818,7 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
   NSArray *allEvents;
   iCalEvent *event;
   NSUInteger i;
+  int j;
 
   allEvents = [rqCalendar events];
 
@@ -1849,13 +1850,32 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
           uid = [[event organizer] uidInContext: context];
           if (uid)
             {
+	      iCalPerson *attendee, *organizer;
               NSDictionary *defaultIdentity;
-              SOGoUser *organizer;
+	      SOGoUser *organizerUser;
+	      NSArray *allAttendees;
 
-              organizer = [SOGoUser userWithLogin: uid];
-              defaultIdentity = [organizer defaultIdentity];
-              [[event organizer] setCn: [defaultIdentity objectForKey: @"fullName"]];
-              [[event organizer] setEmail: [defaultIdentity objectForKey: @"email"]];
+	      organizerUser = [SOGoUser userWithLogin: uid];
+              defaultIdentity = [organizerUser defaultIdentity];
+	      organizer = [[event organizer] copy];
+              [organizer setCn: [defaultIdentity objectForKey: @"fullName"]];
+              [organizer setEmail: [defaultIdentity objectForKey: @"email"]];
+
+	      // We now check if one of the attendee is also the organizer. If so,
+	      // we remove it. See bug #3905 (https://sogo.nu/bugs/view.php?id=3905)
+	      // for more details. This is a Calendar app bug on Apple Yosemite.
+	      allAttendees = [event attendees];
+
+	      for (j = [allAttendees count]-1; j >= 0; j--)
+		{
+		  attendee = [allAttendees objectAtIndex: j];
+		  if ([organizerUser hasEmail: [attendee rfc822Email]])
+		    [event removeFromAttendees: attendee];
+		}
+
+	      // We reset the organizer
+	      [event setOrganizer: organizer];
+	      RELEASE(organizer);
             }
         }
     }
@@ -1990,6 +2010,21 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
   return [super DELETEAction: _ctx];
 }
 
+//
+// Let's check if our CalDAV client has sent us a broken SENT-BY. When Lightning is identity-aware,
+// it'll stupidly send something like this:
+// ORGANIZER;RSVP=TRUE;CN=John Doe;PARTSTAT=ACCEPTED;ROLE=CHAIR;SENT-BY="mail
+// to:mailto:sogo3@example.com":mailto:sogo1@example.com
+//
+- (void) _fixupSentByForPerson: (iCalPerson *) person
+{
+  NSString *sentBy;
+
+  sentBy = [person sentBy];
+  if ([sentBy hasPrefix: @"mailto:"])
+    [person setSentBy: [sentBy substringFromIndex: 7]];
+}
+
 //
 // This method is meant to be the common point of any save operation from web
 // and DAV requests, as well as from code making use of SOGo as a library
@@ -2025,9 +2060,7 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
         [self _decomposeGroupsInRequestCalendar: calendar];
       
       if ([[ownerUser domainDefaults] iPhoneForceAllDayTransparency] && [rq isIPhone])
-        {
-          [self _adjustTransparencyInRequestCalendar: calendar];
-        }
+	[self _adjustTransparencyInRequestCalendar: calendar];
       
       [self _adjustEventsInRequestCalendar: calendar];
       [self adjustClassificationInRequestCalendar: calendar];
@@ -2067,7 +2100,7 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
       // broken CalDAV client that aren't identity-aware will create the event in Annie's calendar
       // and set Bob as the organizer. We fix this for them. See #3368 for details.
       if (!userIsOrganizer &&
-	  [[context activeUser] hasEmail:  [[event organizer] rfc822Email]])
+	  [[context activeUser] hasEmail: [[event organizer] rfc822Email]])
 	{
 	  [[event organizer] setCn: [ownerUser cn]];
 	  [[event organizer] setEmail: [[ownerUser allEmails] objectAtIndex: 0]];
@@ -2080,6 +2113,8 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
 	  attendees = [event attendeesWithoutUser: ownerUser];
 	  if ([attendees count])
 	    {
+	      [self _fixupSentByForPerson: [event organizer]];
+
 	      if ((ex = [self _handleAddedUsers: attendees fromEvent: event  force: YES]))
 		return ex;
 	      else
@@ -2224,7 +2259,7 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
 	  // broken CalDAV client that aren't identity-aware will create the event in Annie's calendar
 	  // and set Bob as the organizer. We fix this for them.  See #3368 for details.
 	  if (!userIsOrganizer &&
-	      [[context activeUser] hasEmail:  [[newEvent organizer] rfc822Email]])
+	      [[context activeUser] hasEmail: [[newEvent organizer] rfc822Email]])
 	    {
 	      [[newEvent organizer] setCn: [ownerUser cn]];
 	      [[newEvent organizer] setEmail: [[ownerUser allEmails] objectAtIndex: 0]];
@@ -2244,6 +2279,8 @@ inRecurrenceExceptionsForEvent: (iCalEvent *) theEvent
 	      // We check ACLs of the 'organizer' - in case someone forges the SENT-BY
 	      NSString *uid;
 
+	      [self _fixupSentByForPerson: [newEvent organizer]];
+
 	      uid = [[oldEvent organizer] uidInContext: context];
 
 	      if (uid && [[[context activeUser] login] caseInsensitiveCompare: uid] != NSOrderedSame)
