@@ -580,17 +580,6 @@
 		[allAttendees addObject: person];
 	    }
 	}
-      else
-	{
-	  // We remove any attendees matching the organizer. Apple iCal will do that when
-	  // you invite someone. It'll add the organizer in the attendee list, which will
-	  // confuse itself!
-	  if ([[currentAttendee rfc822Email] caseInsensitiveCompare: organizerEmail] == NSOrderedSame)
-	    {
-	      [allAttendees removeObject: currentAttendee];
-	      eventWasModified = YES;
-	    }
-	}
       
       j++;
     } // while (currentAttendee ...
@@ -797,12 +786,12 @@
   if ([object firstChildWithTag: @"X-SOGo-Send-Appointment-Notifications"])
     return;
 
-  // We never send IMIP inivitaton/deletion/update when the "initiator" is Outlook 2013/2016 over
-  // the EAS protocol. That is because Outlook will always issue a SendMail command
+  // We never send IMIP inivitaton/deletion/update when the "initiator" is an EAS client.
+  // That is because Outlook, iOS and Android will always issue a SendMail command
   // with the meeting details (ie., IMIP message with METHOD:REQUEST) so there's
   // no need to send it twice. Moreover, Outlook users can also choose to NOT send
   // the IMIP messsage at all, so SOGo won't send one without user's consent
-  if ([[context objectForKey: @"DeviceType"] isEqualToString: @"WindowsOutlook15"])
+  if ([[[context request] requestHandlerKey] isEqualToString: @"Microsoft-Server-ActiveSync"])
     return;
 
   ownerUser = [SOGoUser userWithLogin: owner];
