@@ -309,6 +309,7 @@
                                   to: (NSCalendarDate *) endDate
 {
   SOGoAppointmentFolder *calFolder;
+  SOGoAppointmentFolders *calFolders;
   SOGoUser *user;
   SOGoUserDefaults *ud;
   NSArray *folders;
@@ -318,9 +319,12 @@
 
   infos = [NSMutableArray array];
 
-  folders = [[container lookupName: @"Calendar"
-			inContext: context
-			acquire: NO] subFolders];
+  calFolders = [container lookupName: @"Calendar"
+                           inContext: context
+                             acquire: NO];
+  [calFolders appendSubscribedSources];
+  folders = [calFolders subFolders];
+
   max = [folders count];
   for (count = 0; count < max; count++)
     {
