@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2006-2012 Inverse inc.
-  Copyright (C) 2004-2005 SKYRIX Software AG
+  Copyright (C) 2006-2014-2016 Inverse inc.
 
   This file is part of SOGo.
 
@@ -20,22 +19,19 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSException.h>
+#import <Foundation/NSCalendarDate.h>
 
+#import <NGObjWeb/WORequest.h>
+#import <NGObjWeb/WOResponse.h>
+#import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGCards/iCalCalendar.h>
 #import <NGCards/iCalToDo.h>
-#import <NGCards/iCalEventChanges.h>
-#import <NGCards/iCalPerson.h>
-#import <SOGo/SOGoUserManager.h>
 
-#import <SoObjects/SOGo/SOGoMailer.h>
 
 #import "iCalCalendar+SOGo.h"
 #import "NSArray+Appointments.h"
-#import "SOGoAptMailNotification.h"
-#import "SOGoAppointmentFolder.h"
 #import "SOGoTaskOccurence.h"
 
 #import "SOGoTaskObject.h"
@@ -65,12 +61,12 @@
 
 #warning this code should be put in SOGoCalendarComponent once the UID hack\
   in SOGoAppointmentObject is resolved
-- (NSException *) saveContentString: (NSString *) newContent
-                        baseVersion: (unsigned int) newVersion
+- (NSException *) saveComponent: (id) theComponent
+                    baseVersion: (unsigned int) newVersion
 {
   NSException *ex;
 
-  ex = [super saveContentString: newContent baseVersion: newVersion];
+  ex = [super saveComponent: theComponent baseVersion: newVersion];
   [fullCalendar release];
   fullCalendar = nil;
   [safeCalendar release];
@@ -81,4 +77,51 @@
   return ex;
 }
 
+- (iCalRepeatableEntityObject *) newOccurenceWithID: (NSString *) theRecurrenceID
+{
+  iCalToDo *newOccurence, *master;
+  NSCalendarDate *date, *firstDate;
+  NSTimeInterval interval;
+
+  newOccurence = (iCalToDo *) [super newOccurenceWithID: theRecurrenceID];
+  date = [newOccurence recurrenceId];
+  [newOccurence setStartDate: date];
+
+  master = [self component: NO secure: NO];
+
+  if ([master due])
+    {
+      firstDate = [master startDate];
+      interval = [[master due]
+                   timeIntervalSinceDate: (NSDate *)firstDate];
+  
+      [newOccurence setDue: [date addYear: 0
+                                    month: 0
+                                      day: 0
+                                     hour: 0
+                                   minute: 0
+                                   second: interval]];
+    }
+
+  return newOccurence;
+}
+
+- (void) prepareDeleteOccurence: (iCalToDo *) occurence
+{
+
+}
+
+- (id) PUTAction: (WOContext *) _ctx
+{
+  iCalCalendar *rqCalendar;
+  WORequest *rq;
+
+  rq = [_ctx request];
+  rqCalendar = [iCalCalendar parseSingleFromSource: [rq contentAsString]];
+  [self adjustClassificationInRequestCalendar: rqCalendar];
+  [rq setContent: [[rqCalendar versitString] dataUsingEncoding: [rq contentEncoding]]];
+
+  return [super PUTAction: _ctx];
+}
+
 @end /* SOGoTaskObject */
