@@ -1,6 +1,6 @@
 /* iCalEvent+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2014 Inverse inc.
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 @interface iCalEvent (SOGoExtensions)
 
 - (BOOL) isStillRelevant;
-- (unsigned int) occurenceInterval;
+- (NSTimeInterval) occurenceInterval;
 - (void) updateRecurrenceRulesUntilDate: (NSCalendarDate *) previousEndDate;
 
 @end
