@@ -1,6 +1,6 @@
 /* iCalEvent+SOGo.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2014 Inverse inc.
+ * Copyright (C) 2007-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,25 +18,27 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
+#import <Foundation/NSTimeZone.h>
 #import <Foundation/NSValue.h>
 
-#import <NGExtensions/NGCalendarDateRange.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
 
+#import <NGCards/iCalCalendar.h>
 #import <NGCards/iCalDateTime.h>
 #import <NGCards/iCalTimeZone.h>
-#import <NGCards/iCalEvent.h>
 #import <NGCards/iCalPerson.h>
 #import <NGCards/iCalRecurrenceRule.h>
-#import <NGCards/iCalTrigger.h>
 #import <NGCards/NSString+NGCards.h>
 
-#import "SOGoAppointmentFolder.h"
+#import <SoObjects/SOGo/CardElement+SOGo.h>
+#import <SoObjects/SOGo/SOGoUser.h>
+#import <SoObjects/SOGo/SOGoUserDefaults.h>
+#import <SoObjects/SOGo/WOContext+SOGo.h>
+
+#import <SOGo/NSCalendarDate+SOGo.h>
+
 #import "iCalRepeatableEntityObject+SOGo.h"
 
 #import "iCalEvent+SOGo.h"
@@ -47,7 +49,7 @@
 {
   NSCalendarDate *now, *lastRecurrence;
   BOOL isStillRelevent;
-  
+
   now = [NSCalendarDate calendarDate];
 
   if ([self isRecurrent])
@@ -57,7 +59,7 @@
     }
   else
     isStillRelevent = ([[self endDate] earlierDate: now] == now);
-  
+
   return isStillRelevent;
 }
 
@@ -66,6 +68,7 @@
 //
 - (NSMutableDictionary *) quickRecordFromContent: (NSString *) theContent
                                        container: (id) theContainer
+				 nameInContainer: (NSString *) nameInContainer
 {
   NSMutableDictionary *row;
   NSCalendarDate *startDate, *endDate;
@@ -114,8 +117,8 @@
   row = [NSMutableDictionary dictionaryWithCapacity:8];
 
   [row setObject: @"vevent" forKey: @"c_component"];
- 
-  if ([uid isNotNull]) 
+
+  if ([uid isNotNull])
     [row setObject:uid forKey: @"c_uid"];
   else
     [self logWithFormat: @"WARNING: could not extract a uid from event!"];
@@ -168,21 +171,21 @@
 					  forAllDay: isAllDay]
 	      forKey: @"c_enddate"];
     }
-  
+
   if ([self isRecurrent])
     {
       NSCalendarDate *date;
-      
+
       date = [self lastPossibleRecurrenceStartDate];
-      if (!date)
-	{
-	  /* this could also be *nil*, but in the end it makes the fetchspecs
-	     more complex - thus we set it to a "reasonable" distant future */
-	  date = iCalDistantFuture;
-	}
+      if (date)
+        date = [date addTimeInterval: [self durationAsTimeInterval]];
+      else
+        /* this could also be *nil*, but in the end it makes the fetchspecs
+           more complex - thus we set it to a "reasonable" distant future */
+        date = iCalDistantFuture;
       [row setObject: [self quickRecordDateAsNumber: date
-			    withOffset: 0 forAllDay: NO]
-	   forKey: @"c_cycleenddate"];
+                                         withOffset: 0 forAllDay: NO]
+              forKey: @"c_cycleenddate"];
       [row setObject: [self cycleInfo] forKey: @"c_cycleinfo"];
     }
 
@@ -194,7 +197,7 @@
   if ([status isNotNull])
     {
       int code = 1;
- 
+
       if ([status isEqualToString: @"TENTATIVE"])
 	code = 2;
       else if ([status isEqualToString: @"CANCELLED"])
@@ -214,12 +217,12 @@
   if (organizer)
     {
       NSString *email;
- 
+
       email = [organizer valueForKey: @"rfc822Email"];
       if (email)
 	[row setObject:email forKey: @"c_orgmail"];
     }
- 
+
   /* construct partstates */
   count = [attendees count];
   partstates = [[NSMutableString alloc] initWithCapacity:count * 2];
@@ -227,7 +230,7 @@
     {
       iCalPerson *p;
       iCalPersonPartStat stat;
- 
+
       p = [attendees objectAtIndex:i];
       stat = [p participationStatus];
       if (i)
@@ -238,7 +241,7 @@
   [partstates release];
 
   /* handle alarms */
-  [self updateNextAlarmDateInRow: row  forContainer: theContainer];
+  [self updateNextAlarmDateInRow: row  forContainer: theContainer  nameInContainer: nameInContainer];
 
   /* handle categories */
   categories = [self categories];
@@ -248,16 +251,22 @@
   else
     [row setObject: [NSNull null] forKey: @"c_category"];
 
+  /* handle description */
+  if ([self comment])
+    [row setObject: [self comment]  forKey: @"c_description"];
+  else
+    [row setObject: [NSNull null] forKey: @"c_description"];
+  
   return row;
 }
 
 - (NSTimeInterval) occurenceInterval
 {
-  return [[self endDate] timeIntervalSinceDate: [self startDate]];
+  return (NSTimeInterval) [[self endDate] timeIntervalSinceDate: [self startDate]];
 }
 
 /**
- * Shift the "until dates" of the recurrence rules of the event 
+ * Shift the "until dates" of the recurrence rules of the event
  * with respect to the previous end date of the event.
  * @param previousEndDate the previous end date of the event
  */
@@ -306,5 +315,115 @@
 	}
     }
 }
-  
+
+/**
+ * @see [iCalRepeatableEntityObject+SOGo attributes]
+ * @see [iCalEntityObject+SOGo attributes]
+ * @see [UIxAppointmentEditor viewAction]
+ */
+- (NSDictionary *) attributesInContext: (WOContext *) context
+{
+  NSMutableDictionary *data;
+
+  data = [NSMutableDictionary dictionaryWithDictionary: [super attributesInContext: context]];
+
+  [data setObject: [NSNumber numberWithBool: ![self isOpaque]] forKey: @"isTransparent"];
+
+  return data;
+}
+
+/**
+ * @see [iCalRepeatableEntityObject+SOGo setAttributes:inContext:]
+ * @see [iCalEntityObject+SOGo setAttributes:inContext:]
+ * @see [UIxAppointmentEditor saveAction]
+ */
+- (void) setAttributes: (NSDictionary *) data
+             inContext: (WOContext *) context
+{
+  NSCalendarDate *aptStartDate, *aptEndDate, *allDayStartDate;
+  NSInteger offset, nbrDays;
+  NSTimeZone *timeZone;
+  iCalDateTime *startDate;
+  iCalTimeZone *tz;
+  SOGoUserDefaults *ud;
+  BOOL isAllDay;
+  id o;
+
+  [super setAttributes: data inContext: context];
+
+  aptStartDate = aptEndDate = nil;
+
+  // Handle start/end dates
+  o = [data objectForKey: @"startDate"];
+  if ([o isKindOfClass: [NSString class]] && [o length])
+    aptStartDate = [self dateFromString: o inContext: context];
+
+  o = [data objectForKey: @"endDate"];
+  if ([o isKindOfClass: [NSString class]] && [o length])
+    aptEndDate = [self dateFromString: o inContext: context];
+
+  o = [data objectForKey: @"isTransparent"];
+  if ([o isKindOfClass: [NSNumber class]] && [o boolValue])
+    [self setTransparency: @"TRANSPARENT"];
+  else
+    [self setTransparency: @"OPAQUE"];
+
+  isAllDay = [[data objectForKey: @"isAllDay"] boolValue];
+
+  if (aptStartDate && aptEndDate && [aptStartDate earlierDate: aptEndDate] == aptStartDate)
+    {
+      if (isAllDay)
+        {
+          // Remove the vTimeZone when dealing with an all-day event.
+          startDate = (iCalDateTime *)[self uniqueChildWithTag: @"dtstart"];
+          tz = [startDate timeZone];
+          if (tz)
+            {
+              [startDate setTimeZone: nil];
+              [(iCalDateTime *)[self uniqueChildWithTag: @"dtend"] setTimeZone: nil];
+              [[self parent] removeChild: tz];
+            }
+
+          nbrDays = round ([aptEndDate timeIntervalSinceDate: aptStartDate] / 86400) + 1;
+          timeZone = [aptStartDate timeZone];
+          offset = [timeZone secondsFromGMTForDate: aptStartDate];
+          allDayStartDate = [aptStartDate dateByAddingYears: 0 months: 0 days: 0
+                                                      hours: 0 minutes: 0
+                                                    seconds: offset];
+          [self setAllDayWithStartDate: allDayStartDate
+                              duration: nbrDays];
+        }
+      else
+        {
+          o = [data objectForKey: @"startTime"];
+          if ([o isKindOfClass: [NSString class]] && [o length])
+            [self adjustDate: &aptStartDate withTimeString: o inContext: context];
+
+          o = [data objectForKey: @"endTime"];
+          if ([o isKindOfClass: [NSString class]] && [o length])
+            [self adjustDate: &aptEndDate withTimeString: o inContext: context];
+
+          [self setStartDate: aptStartDate];
+          [self setEndDate: aptEndDate];
+        }
+    }
+
+  if (!isAllDay)
+    {
+      // Make sure there's a vTimeZone associated to the event unless it
+      // is an all-day event.
+      startDate = (iCalDateTime *)[self uniqueChildWithTag: @"dtstart"];
+      if (![startDate timeZone])
+        {
+          ud = [[context activeUser] userDefaults];
+          tz = [iCalTimeZone timeZoneForName: [ud timeZoneName]];
+          if ([[self parent] addTimeZone: tz])
+            {
+              [startDate setTimeZone: tz];
+              [(iCalDateTime *)[self uniqueChildWithTag: @"dtend"] setTimeZone: tz];
+            }
+        }
+    }
+}
+
 @end
