@@ -1,9 +1,6 @@
 /* iCalPerson+SOGo.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2012 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *         Ludovic Marcotte <lmarcotte@inverse.ca>
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +19,10 @@
  */
 
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSString.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserManager.h>
+#import <SOGo/SOGoSystemDefaults.h>
 
 #import "iCalPerson+SOGo.h"
 
@@ -67,6 +64,46 @@ static SOGoUserManager *um = nil;
   return [um getUIDForEmail: [self rfc822Email]];
 }
 
+/*
+ It returns the login if the email of the iCalPerson exists on the
+ domain of the current active user
+*/
+- (NSString *) uidInContext: (WOContext *) context
+{
+  NSString *domain;
+
+  domain = [[context activeUser] domain];
+
+  return [self uidInDomain: domain];
+}
+
+/*
+ It returns the login if the email of the iCalPerson exists on the
+ given domain
+*/
+- (NSString *) uidInDomain: (NSString *) domain
+{
+  NSDictionary *contact;
+  NSString *uid = nil;
+
+  if (!um)
+    um = [SOGoUserManager sharedUserManager];
+
+  contact = [um contactInfosForUserWithUIDorEmail: [self rfc822Email]
+                                         inDomain: domain];
+  if (!contact) return nil;
+
+  uid = [contact valueForKey: @"c_uid"];
+
+  // On multidomain environment without DomainLessLogin enabled the login
+  // must have the @domain suffix
+  if ([[SOGoSystemDefaults sharedSystemDefaults] enableDomainBasedUID]
+      && ![[contact objectForKey: @"DomainLessLogin"] boolValue])
+    uid = [NSString stringWithFormat:@"%@@%@", uid, domain];
+
+  return uid;
+}
+
 - (NSString *) contactIDInContext: (WOContext *) context
 {
   NSString *domain, *uid;
