@@ -236,19 +236,40 @@ convention:
 - (void) _setEmails: (NSDictionary *) ldifRecord
 {
   CardElement *homeMail;
-  NSString* mail;
+  NSArray *emails;
+  NSEnumerator *mailList;
+  id mail;
 
   // Emails from the configured mail fields of the source have already been extracted in
   // [LDAPSource _fillEmailsOfEntry:intoLDIFRecord:]
   [self addElementWithTag: @"email"
                    ofType: @"work"
                 withValue: [ldifRecord objectForKey: @"c_emails"]];
+
   // When importing an LDIF file, add the default mail attribute
   mail = [ldifRecord objectForKey: @"mail"];
-  if ([mail length] && ![[self emails] containsObject: mail])
-    [self addElementWithTag: @"email"
-                     ofType: @"work"
-                  withValue: [ldifRecord objectForKey: @"mail"]];
+  if (mail)
+    {
+      emails = [self emails];
+      if ([mail isKindOfClass: [NSArray class]])
+        {
+          mailList = [(NSArray *)mail objectEnumerator];
+          while ((mail = [mailList nextObject]))
+            {
+              if ([mail length] && ![emails containsObject: mail])
+                [self addElementWithTag: @"email"
+                                 ofType: @"work"
+                              withValue: mail];
+            }
+        }
+      else if ([mail length] && ![emails containsObject: mail])
+        {
+          [self addElementWithTag: @"email"
+                           ofType: @"work"
+                        withValue: mail];
+        }
+    }
+
   homeMail = [self elementWithTag: @"email" ofType: @"home"];
   [homeMail setSingleValue: [ldifRecord objectForKey: @"mozillasecondemail"] forKey: @""];
   [[self uniqueChildWithTag: @"x-mozilla-html"]
@@ -261,9 +282,9 @@ convention:
   NSInteger year, yearOfToday, month, day;
   CardElement *element;
   NSCalendarDate *now;
-  NSArray *units;
-  NSString *fn, *ou;
-  id o;
+  NSMutableArray *units;
+  NSString *fn;
+  id o, ou;
 
   [self setNWithFamily: [ldifRecord objectForKey: @"sn"]
                  given: [ldifRecord objectForKey: @"givenname"]
@@ -277,53 +298,58 @@ convention:
   [self setFn: fn];
 
   element = [self elementWithTag: @"adr" ofType: @"home"];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillahomestreet2"]
-                  atIndex: 1 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillahomestreet"]
-                  atIndex: 2 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillahomelocalityname"]
-                  atIndex: 3 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillahomestate"]
-                  atIndex: 4 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillahomepostalcode"]
-                  atIndex: 5 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillahomecountryname"]
-                  atIndex: 6 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillahomestreet2"]
+	     atIndex: 1 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillahomestreet"]
+	     atIndex: 2 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillahomelocalityname"]
+	     atIndex: 3 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillahomestate"]
+	     atIndex: 4 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillahomepostalcode"]
+	     atIndex: 5 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillahomecountryname"]
+	     atIndex: 6 forKey: @""];
 
   element = [self elementWithTag: @"adr" ofType: @"work"];
-  [element setSingleValue: [ldifRecord objectForKey: @"mozillaworkstreet2"]
-                  atIndex: 1 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"street"]
-                  atIndex: 2 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"l"]
-                  atIndex: 3 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"st"]
-                  atIndex: 4 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"postalcode"]
-                  atIndex: 5 forKey: @""];
-  [element setSingleValue: [ldifRecord objectForKey: @"c"]
-                  atIndex: 6 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"mozillaworkstreet2"]
+	     atIndex: 1 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"street"]
+	     atIndex: 2 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"l"]
+	     atIndex: 3 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"st"]
+	     atIndex: 4 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"postalcode"]
+	     atIndex: 5 forKey: @""];
+  [element setValues: [ldifRecord objectForKey: @"c"]
+	     atIndex: 6 forKey: @""];
 
   ou = [ldifRecord objectForKey: @"ou"];
   if ([ou isKindOfClass: [NSArray class]])
-    units = [NSArray arrayWithArray: (NSArray *)ou];
+    units = [NSMutableArray arrayWithArray: (NSArray *)ou];
   else if (ou)
-    units = [NSArray arrayWithObject: ou];
+    units = [NSMutableArray arrayWithObject: ou];
   else
-    units = nil;
-  [self setOrg: [ldifRecord objectForKey: @"o"]
-         units: units];
+    units = [NSMutableArray array];
+  o = [ldifRecord objectForKey: @"o"];
+  if ([o isKindOfClass: [NSArray class]])
+    [units addObjectsFromArray: (NSArray *)o];
+  else if (o)
+    [units addObject: o];
+  [self setOrganizations: units];
 
   [self _setPhoneValues: ldifRecord];
   [self _setEmails: ldifRecord];
   [[self elementWithTag: @"url" ofType: @"home"]
-    setSingleValue: [ldifRecord objectForKey: @"mozillahomeurl"] forKey: @""];
+    setValues: [ldifRecord objectForKey: @"mozillahomeurl"] atIndex: 0  forKey: @""];
   [[self elementWithTag: @"url" ofType: @"work"]
-    setSingleValue: [ldifRecord objectForKey: @"mozillaworkurl"] forKey: @""];
+    setValues: [ldifRecord objectForKey: @"mozillaworkurl"] atIndex: 0  forKey: @""];
 
   [[self uniqueChildWithTag: @"x-aim"]
-    setSingleValue: [ldifRecord objectForKey: @"nsaimid"]
-            forKey: @""];
+    setValues: [ldifRecord objectForKey: @"nsaimid"]
+      atIndex: 0
+       forKey: @""];
 
   now = [NSCalendarDate date];
   year = [[ldifRecord objectForKey: @"birthyear"] intValue];
@@ -712,6 +738,53 @@ convention:
   return company;
 }
 
+- (void) setOrganizations: (NSArray *) newOrganizations
+{
+  CardElement *org;
+  NSArray *elements;
+  NSUInteger count, max;
+
+  // First remove all current org properties
+  elements = [self childrenWithTag: @"org"];
+  [self removeChildren: elements];
+
+  org = [self uniqueChildWithTag: @"org"];
+  max = [newOrganizations count];
+  for (count = 0; count < max; count++)
+    {
+      [org setSingleValue: [newOrganizations objectAtIndex: count]
+                  atIndex: count forKey: @""];
+    }
+}
+
+- (NSArray *) organizations
+{
+  CardElement *org;
+  NSArray *organizations;
+  NSEnumerator *orgs;
+  NSMutableArray *flattenedOrganizations;
+  NSString *value;
+  NSUInteger count, max;
+
+  // Get all org properties
+  orgs = [[self childrenWithTag: @"org"] objectEnumerator];
+  flattenedOrganizations = [NSMutableArray array];
+  while ((org = [orgs nextObject]))
+    {
+      // Get all values of each org property
+      organizations = [org valuesForKey: @""];
+      max = [organizations count];
+      for (count = 0; count < max; count++)
+        {
+          value = [[organizations objectAtIndex: count] lastObject];
+          if ([value length])
+            [flattenedOrganizations addObject: value];
+        }
+    }
+
+  return flattenedOrganizations;
+}
+
 - (NSString *) fullName
 {
   CardElement *n;
