@@ -1,6 +1,6 @@
-/* SOGoContactGCSEntry.h - this file is part of SOGo
+/* SOGoContactGCSEntry.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2014 Inverse inc.
+ * Copyright (C) 2006-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,12 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSString.h>
 
-#import <NGCards/NGVCard.h>
-#import <NGCards/NGVCardPhoto.h>
 
 #import "NGVCard+SOGo.h"
 #import "SOGoContactEntryPhoto.h"
@@ -50,7 +47,7 @@
 
 - (Class *) parsingClass
 {
-  return [NGVCard class];
+  return (Class *)[NGVCard class];
 }
 
 /* content */
@@ -187,6 +184,10 @@
 {
   NSException *ex;
 
+  // We make sure new cards always have a UID - see #3819
+  if (![[newCard uid] length])
+    [newCard setUid: [self globallyUniqueObjectId]];
+
   ex = [super saveComponent: newCard baseVersion: newVersion];
   [card release];
   card = nil;
