@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2006-2011 Inverse inc.
-  Copyright (C) 2004-2005 SKYRIX Software AG
+  Copyright (C) 2006-2016 Inverse inc.
 
   This file is part of SOGo.
 
@@ -15,24 +14,19 @@
   License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with OGo; see the file COPYING.  If not, write to the
+  License along with SOGo; see the file COPYING.  If not, write to the
   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSURL.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/SoObject+SoDAV.h>
 #import <NGObjWeb/WOContext.h>
-#import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
 #import <SaxObjC/XMLNamespaces.h>
+#import <EOControl/EOFetchSpecification.h>
 #import <EOControl/EOQualifier.h>
 #import <EOControl/EOSortOrdering.h>
 
@@ -67,6 +61,24 @@ static NSArray *folderListingFields = nil;
                                            @"c_component", nil];
 }
 
+- (id) init
+{
+  if ((self = [super init]))
+    {
+      baseCardDAVURL = nil;
+      basePublicCardDAVURL = nil;
+    }
+
+  return self;
+}
+
+- (void) dealloc
+{
+  [baseCardDAVURL release];
+  [basePublicCardDAVURL release];
+  [super dealloc];
+}
+
 - (Class) objectClassForContent: (NSString *) content
 {
   CardGroup *cardEntry;
@@ -109,7 +121,7 @@ static NSArray *folderListingFields = nil;
   NSString *component;
   Class objectClass;
 
-  qualifier = [EOQualifier qualifierWithQualifierFormat:@"c_name = %@", name];
+  qualifier = [EOQualifier qualifierWithQualifierFormat: @"c_name = %@", [name asSafeSQLString]];
   records = [[self ocsFolder] fetchFields: [NSArray arrayWithObject: @"c_component"]
                               matchingQualifier: qualifier];
 
@@ -170,16 +182,15 @@ static NSArray *folderListingFields = nil;
   return obj;
 }
 
-- (EOQualifier *) _qualifierForFilter: (NSString *) filter
-                           onCriteria: (NSString *) criteria
+- (EOQualifier *) qualifierForFilter: (NSString *) filter
+                          onCriteria: (NSString *) criteria
 {
   NSString *qs;
   EOQualifier *qualifier;
 
   if ([filter length] > 0)
     {
-      filter = [[filter stringByReplacingString: @"\\"  withString: @"\\\\"]
-                     stringByReplacingString: @"'"  withString: @"\\'\\'"];
+      filter = [filter asSafeSQLString];
       if ([criteria isEqualToString: @"name_or_address"])
         qs = [NSString stringWithFormat:
                          @"(c_sn isCaseInsensitiveLike: '%%%@%%') OR "
@@ -209,10 +220,26 @@ static NSArray *folderListingFields = nil;
   return qualifier;
 }
 
+/**
+ * Normalize keys of dictionary representing a contact.
+ * @param contactRecord a dictionary with pairs from the quick table
+ * @see [UIxContactView dataAction]
+ */
 - (void) fixupContactRecord: (NSMutableDictionary *) contactRecord
 {
   NSString *data;
 
+  // c_categories => categories
+  data = [contactRecord objectForKey: @"c_categories"];
+  if ([data length])
+    [contactRecord setObject: data forKey: @"categories"];
+
+  // c_name => id
+  data = [contactRecord objectForKey: @"c_name"];
+  if ([data length])
+    [contactRecord setObject: data forKey: @"id"];
+
+  // c_cn
   data = [contactRecord objectForKey: @"c_cn"];
   if (![data length])
     {
@@ -224,18 +251,55 @@ static NSArray *folderListingFields = nil;
       else
         {
           data = [contactRecord objectForKey: @"c_o"];
-          [contactRecord setObject: data forKey: @"c_cn"];          
+          [contactRecord setObject: data forKey: @"c_cn"];
         }
     }
 
-  if (![contactRecord objectForKey: @"c_mail"])
-    [contactRecord setObject: @"" forKey: @"c_mail"];
+  // c_screenname
   if (![contactRecord objectForKey: @"c_screenname"])
     [contactRecord setObject: @"" forKey: @"c_screenname"];
-  if (![contactRecord objectForKey: @"c_o"])
-    [contactRecord setObject: @"" forKey: @"c_o"];
-  if (![contactRecord objectForKey: @"c_telephonenumber"])
-    [contactRecord setObject: @"" forKey: @"c_telephonenumber"];
+
+  // c_mail => emails[]
+  data = [contactRecord objectForKey: @"c_mail"];
+  if ([data length])
+    {
+      NSArray *values;
+      NSDictionary *email;
+      NSMutableArray *emails;
+      NSString *type, *value;
+      int i, max;
+
+      values = [data componentsSeparatedByString: @","];
+      max = [values count];
+      emails = [NSMutableArray arrayWithCapacity: max];
+      for (i = 0; i < max; i++)
+        {
+          type = (i == 0)? @"pref" : @"home";
+          value = [values objectAtIndex: i];
+          email = [NSDictionary dictionaryWithObjectsAndKeys: type, @"type", value, @"value", nil];
+          [emails addObject: email];
+        }
+      [contactRecord setObject: emails forKey: @"emails"];
+    }
+  else
+    {
+      [contactRecord setObject: @"" forKey: @"c_mail"];
+      [contactRecord setObject: [NSArray array] forKey: @"emails"];
+    }
+
+  // c_telephonenumber => phones[]
+  data = [contactRecord objectForKey: @"c_telephonenumber"];
+  if ([data length])
+    {
+      NSDictionary *phonenumber;
+      phonenumber = [NSDictionary dictionaryWithObjectsAndKeys: @"pref", @"type", data, @"value", nil];
+      [contactRecord setObject: [NSArray arrayWithObject: phonenumber] forKey: @"phones"];
+    }
+  else
+    {
+      [contactRecord setObject: @"" forKey: @"c_telephonenumber"];
+      [contactRecord setObject: [NSArray array] forKey: @"phones"];
+    }
 }
 
 - (NSArray *) _flattenedRecords: (NSArray *) records
@@ -273,8 +337,7 @@ static NSArray *folderListingFields = nil;
  
   if (aName && [aName length] > 0)
     {
-      aName = [[aName stringByReplacingString: @"\\"  withString: @"\\\\"]
-                 stringByReplacingString: @"'"  withString: @"\\'\\'"];
+      aName = [aName asSafeSQLString];
       qs = [NSString stringWithFormat: @"(c_name='%@')", aName];
       qualifier = [EOQualifier qualifierWithQualifierFormat: qs];
       dbRecords = [[self ocsFolder] fetchFields: folderListingFields
@@ -303,9 +366,9 @@ static NSArray *folderListingFields = nil;
   EOQualifier *qualifier;
   EOSortOrdering *ordering;
 
-  qualifier = [self _qualifierForFilter: filter onCriteria: criteria];
+  qualifier = [self qualifierForFilter: filter onCriteria: criteria];
   dbRecords = [[self ocsFolder] fetchFields: folderListingFields
-				matchingQualifier: qualifier];
+                          matchingQualifier: qualifier];
 
   if ([dbRecords count] > 0)
     {
@@ -326,6 +389,37 @@ static NSArray *folderListingFields = nil;
   return records;
 }
 
+- (NSArray *) lookupContactsWithQualifier: (EOQualifier *) qualifier
+{
+  return [self lookupContactsFields: folderListingFields
+                      withQualifier: qualifier
+                       andOrderings: nil];
+}
+
+- (NSArray *) lookupContactsFields: (NSArray *) fields
+                     withQualifier: (EOQualifier *) qualifier
+                      andOrderings: (NSArray *) orderings
+{
+  NSArray *dbRecords, *records;
+  EOFetchSpecification *spec;
+
+  spec = [EOFetchSpecification fetchSpecificationWithEntityName: [[self ocsFolder] folderName]
+                                                      qualifier: qualifier
+                                                  sortOrderings: orderings];
+
+  dbRecords = [[self ocsFolder] fetchFields: fields
+                         fetchSpecification: spec
+                              ignoreDeleted: YES];
+
+  if ([dbRecords count] > 0 && fields == folderListingFields)
+    records = [self _flattenedRecords: dbRecords];
+  else
+    records = dbRecords;
+
+  [self debugWithFormat:@"fetched %i records.", [records count]];
+  return records;
+}
+
 - (NSDictionary *) davSQLFieldsTable
 {
   static NSMutableDictionary *davSQLFieldsTable = nil;
@@ -403,4 +497,48 @@ static NSArray *folderListingFields = nil;
   return [NSString stringWithFormat: @"%@:%@", nsRep, nodeName];
 }
 
+- (NSString *) _baseCardDAVURL
+{
+  NSString *davURL;
+
+  if (!baseCardDAVURL)
+    {
+      davURL = [[self realDavURL] absoluteString];
+      if ([davURL hasSuffix: @"/"])
+        baseCardDAVURL = [davURL substringToIndex: [davURL length] - 1];
+      else
+        baseCardDAVURL = davURL;
+      [baseCardDAVURL retain];
+    }
+
+  return baseCardDAVURL;
+}
+
+- (NSString *) cardDavURL
+{
+  return [NSString stringWithFormat: @"%@/", [self _baseCardDAVURL]];
+}
+
+- (NSString *) _basePublicCardDAVURL
+{
+  NSString *davURL;
+
+  if (!basePublicCardDAVURL)
+    {
+      davURL = [[self publicDavURL] absoluteString];
+      if ([davURL hasSuffix: @"/"])
+        basePublicCardDAVURL = [davURL substringToIndex: [davURL length] - 1];
+      else
+        basePublicCardDAVURL = davURL;
+      [basePublicCardDAVURL retain];
+    }
+
+  return basePublicCardDAVURL;
+}
+
+- (NSString *) publicCardDavURL
+{
+  return [NSString stringWithFormat: @"%@/", [self _basePublicCardDAVURL]];
+}
+
 @end /* SOGoContactGCSFolder */
