@@ -18,7 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSDictionary.h>
 #import <Foundation/NSString.h>
 
 #import <NGCards/NGVList.h>
@@ -45,7 +44,7 @@
 
 - (Class *) parsingClass
 {
-  return [NGVList class];
+  return (Class *)[NGVList class];
 }
 
 
