@@ -1,6 +1,6 @@
 /* SOGoContactSourceFolder.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2013 Inverse inc.
+ * Copyright (C) 2006-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,39 +18,27 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSURL.h>
 #import <Foundation/NSValue.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
-#import <NGObjWeb/WOApplication.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
-#import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
-#import <NGObjWeb/SoObject.h>
-#import <NGObjWeb/SoSelectorInvocation.h>
-#import <NGObjWeb/SoUser.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
-#import <DOM/DOMElement.h>
-#import <DOM/DOMProtocols.h>
 #import <EOControl/EOSortOrdering.h>
 #import <SaxObjC/XMLNamespaces.h>
 
 #import <SOGo/DOMNode+SOGo.h>
+#import <SOGo/LDAPSource.h>
 #import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/NSObject+DAV.h>
 #import <SOGo/SOGoPermissions.h>
-#import <SOGo/SOGoSource.h>
-#import <SOGo/SOGoUserSettings.h>
+#import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/WORequest+SOGo.h>
 #import <SOGo/WOResponse+SOGo.h>
 
+#import "NSArray+Contacts.h"
 #import "SOGoContactFolders.h"
 #import "SOGoContactGCSFolder.h"
 #import "SOGoContactLDIFEntry.h"
@@ -94,7 +82,7 @@
     {
       if (![newDisplayName length])
         newDisplayName = newName;
-      ASSIGN (displayName, newDisplayName);
+      ASSIGN (displayName, (NSMutableString *)newDisplayName);
     }
 
   return self;
@@ -127,6 +115,11 @@
   return isPersonalSource;
 }
 
+- (BOOL) listRequiresDot
+{
+  return [source listRequiresDot];
+}
+
 - (NSString *) groupDavResourceType
 {
   return @"vcard-collection";
@@ -164,9 +157,10 @@
       ldifEntry = [childRecords objectForKey: objectName];
       if (!ldifEntry)
         {
-          ldifEntry = [source lookupContactEntry: objectName];
-	  if (ldifEntry)
-	    [childRecords setObject: ldifEntry forKey: objectName];
+          ldifEntry = [source lookupContactEntry: objectName
+                                        inDomain: [[context activeUser] domain]];
+          if (ldifEntry)
+            [childRecords setObject: ldifEntry forKey: objectName];
           else if ([self isValidContentName: objectName])
             {
               url = [[[lookupContext request] uri] urlWithoutParameters];
@@ -217,6 +211,11 @@
   return [source removeContactEntryWithID: [ldifEntry nameInContainer]];
 }
 
+/**
+ * Normalize keys of dictionary representing a contact.
+ * @param oldRecord a dictionary with pairs from the source folder (LDAP or SQL)
+ * @see [SOGoContactGCSFolder _fixupContactRecord]
+ */
 - (NSDictionary *) _flattenedRecord: (NSDictionary *) oldRecord
 {
   NSMutableDictionary *newRecord;
@@ -226,54 +225,96 @@
   newRecord = [NSMutableDictionary dictionaryWithCapacity: 8];
   [newRecord setObject: [oldRecord objectForKey: @"c_uid"]
                 forKey: @"c_uid"];
+
+  // c_name => id
   [newRecord setObject: [oldRecord objectForKey: @"c_name"]
                 forKey: @"c_name"];
+  [newRecord setObject: [oldRecord objectForKey: @"c_name"]
+                forKey: @"id"];
 
+  // displayname || c_cn => fn
   data = [oldRecord objectForKey: @"displayname"];
   if (!data)
     data = [oldRecord objectForKey: @"c_cn"];
-  if (!data)
+  if (data)
+    [newRecord setObject: data forKey: @"fn"];
+  else
     data = @"";
   [newRecord setObject: data forKey: @"c_cn"];
 
-  data = [oldRecord objectForKey: @"mail"];
-  if (!data)
-    data = @"";
-  else if ([data isKindOfClass: [NSArray class]])
+  if ([[SOGoSystemDefaults sharedSystemDefaults] enableDomainBasedUID])
     {
-      if ([data count] > 0)
-        data = [data objectAtIndex: 0];
+      data = [oldRecord objectForKey: @"c_domain"];
+      if (data)
+        [newRecord setObject: data forKey: @"c_domain"];
+    }
+
+  // mail => emails[]
+  data = [oldRecord objectForKey: @"c_emails"];
+  if (!data)
+    data = [oldRecord objectForKey: @"mail"];
+  if (data)
+    {
+      if ([data isKindOfClass: [NSArray class]])
+        {
+          if ([data count] > 0)
+            {
+              NSEnumerator *emails;
+              NSMutableArray *recordEmails;
+              NSString *email;
+              emails = [(NSArray *)data objectEnumerator];
+              recordEmails = [NSMutableArray arrayWithCapacity: [data count]];
+              while ((email = [emails nextObject]))
+                {
+                  [recordEmails addObject: [NSDictionary dictionaryWithObject: email forKey: @"value"]];
+                }
+              [newRecord setObject: recordEmails forKey: @"emails"];
+            }
+        }
+      else if (data)
+        {
+          NSDictionary *email;
+          email = [NSDictionary dictionaryWithObjectsAndKeys: @"pref", @"type", data, @"value", nil];
+          [newRecord setObject: [NSArray arrayWithObject: email] forKey: @"emails"];
+        }
       else
         data = @"";
     }
+  else
+    data = @"";
   [newRecord setObject: data forKey: @"c_mail"];
 
-  data = [oldRecord objectForKey: @"nsaimid"];
+  data = [oldRecord objectForKey: @"mozillanickname"];
+  if (![data length])
+    data = [oldRecord objectForKey: @"nsaimid"];
   if (![data length])
     data = [oldRecord objectForKey: @"nscpaimscreenname"];
   if (![data length])
     data = @"";
   [newRecord setObject: data forKey: @"c_screenname"];
 
+  // o => org
   data = [oldRecord objectForKey: @"o"];
-  if (!data)
+  if (data)
+    [newRecord setObject: data forKey: @"org"];
+  else
     data = @"";
   [newRecord setObject: data forKey: @"c_o"];
 
+  // telephonenumber || cellphone || homephone => phones[]
   data = [oldRecord objectForKey: @"telephonenumber"];
   if (![data length])
     data = [oldRecord objectForKey: @"cellphone"];
   if (![data length])
     data = [oldRecord objectForKey: @"homephone"];
-  if (![data length])
-    data = @"";
-  else if ([data isKindOfClass: [NSArray class]])
+  if (data)
     {
-      if ([data count] > 0)
-        data = [data objectAtIndex: 0];
-      else
-        data = @"";
+      NSDictionary *phonenumber;
+      phonenumber = [NSDictionary dictionaryWithObjectsAndKeys: @"pref", @"type", data, @"value", nil];
+      [newRecord setObject: [NSArray arrayWithObject: phonenumber] forKey: @"phones"];
     }
+  else
+    data = @"";
   [newRecord setObject: data forKey: @"c_telephonenumber"];
 
   // Custom attribute for group-lookups. See LDAPSource.m where
@@ -284,15 +325,22 @@
       [newRecord setObject: data forKey: @"isGroup"];
       [newRecord setObject: @"vlist" forKey: @"c_component"];
     }
-#warning TODO: create a custom icon for resources
   else
     {
       [newRecord setObject: @"vcard" forKey: @"c_component"];
     }
   
+  // c_info => note
   data = [oldRecord objectForKey: @"c_info"];
   if ([data length] > 0)
-    [newRecord setObject: data forKey: @"contactInfo"];
+    {
+      [newRecord setObject: data forKey: @"note"];
+      [newRecord setObject: data forKey: @"contactInfo"];
+    }
+
+  // photo => hasPhoto
+  data = [oldRecord objectForKey: @"photo"];
+  [newRecord setObject: [NSNumber numberWithInt: [data length] ? 1 : 0] forKey: @"hasPhoto"];
 
   recordSource = [oldRecord objectForKey: @"source"];
   if ([recordSource conformsToProtocol: @protocol (SOGoDNSource)] &&
@@ -304,15 +352,22 @@
 
 - (NSArray *) _flattenedRecords: (NSArray *) records
 {
+  NSMutableDictionary *oldRecord;
   NSMutableArray *newRecords;
   NSEnumerator *oldRecords;
-  NSDictionary *oldRecord;
+  NSDictionary *o;
 
   newRecords = [NSMutableArray arrayWithCapacity: [records count]];
 
   oldRecords = [records objectEnumerator];
-  while ((oldRecord = [oldRecords nextObject]))
-    [newRecords addObject: [self _flattenedRecord: oldRecord]];
+  while ((o = [oldRecords nextObject]))
+    {
+      oldRecord = [NSMutableDictionary dictionary];
+      [oldRecord addEntriesFromDictionary: o];
+      if ([source isKindOfClass: [LDAPSource class]])
+        [(LDAPSource *)source applyContactMappingToResult: oldRecord];
+      [newRecords addObject: [self _flattenedRecord: oldRecord]];
+    }
 
   return newRecords;
 }
@@ -324,10 +379,14 @@
   NSDictionary *record;
 
   if (aName && [aName length] > 0)
-    record = [self _flattenedRecord: [source lookupContactEntry: aName]];
+    {
+      record = [source lookupContactEntry: aName
+                                 inDomain: [[context activeUser] domain]];
+      record = [self _flattenedRecord: record];
+    }
   else
     record = nil;
-  
+
   return record;
 }
 
@@ -444,7 +503,7 @@
       methodSel = SOGoSelectorForPropertyGetter (*currentProperty);
       if (methodSel && [ldifEntry respondsToSelector: methodSel])
         *currentValue = [[ldifEntry performSelector: methodSel]
-                          stringByEscapingXMLString];
+                          safeStringByEscapingXMLString];
       currentProperty++;
       currentValue++;
     }
@@ -562,7 +621,7 @@
                          toResponse: (WOResponse *) response
 {
   NSObject <DOMElement> *element;
-  NSString *url, *baseURL, *cname;
+  NSString *url, *baseURL, *cname, *domain;
   NSString **propertiesArray;
   NSMutableString *buffer;
   NSDictionary *object;
@@ -579,13 +638,13 @@
 
   max = [refs length];
   buffer = [NSMutableString stringWithCapacity: max*512];
-
+  domain = [[context activeUser] domain];
   for (count = 0; count < max; count++)
     {
       element = [refs objectAtIndex: count];
       url = [[[element firstChild] nodeValue] stringByUnescapingURL];
       cname = [self _deduceObjectNameFromURL: url fromBaseURL: baseURL];
-      object = [source lookupContactEntry: cname];
+      object = [source lookupContactEntry: cname inDomain: domain];
       if (object)
         [self appendObject: object
                 properties: propertiesArray
@@ -657,7 +716,7 @@
   BOOL otherIsPersonal;
 
   otherIsPersonal = ([otherFolder isKindOfClass: [SOGoContactGCSFolder class]]
-                     || ([otherFolder isKindOfClass: isa] && [otherFolder isPersonalSource]));
+                     || ([otherFolder isKindOfClass: object_getClass(self)] && [otherFolder isPersonalSource]));
 
   if (isPersonalSource)
     {
