@@ -1,8 +1,6 @@
 /* NSObject+CardDAV.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2011 Inverse inc.
- *
- * Author: Ludovic Marcotte <ludovic@inverse.ca>
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,22 +18,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
 
-#import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
-#import <DOM/DOMProtocols.h>
 #import <DOM/DOMNode.h>
 #import <SaxObjC/SaxObjC.h>
-#import <SaxObjC/XMLNamespaces.h>
 
+#import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/WOResponse+SOGo.h>
 
@@ -75,7 +68,7 @@
                            [component davEntityTag]];
       [r appendContentString: etagLine];
       [r appendContentString: @"<C:address-data>"];
-      contactString = [[component contentAsString] stringByEscapingXMLString];
+      contactString = [[component contentAsString] safeStringByEscapingXMLString];
       [r appendContentString: contactString];
       [r appendContentString: @"</C:address-data>"
                         @"<C:addressbook-data>"];
@@ -187,6 +180,13 @@
         [filters addObject: filter];
     }
 
+  // If no filters are provided, we return everything.
+  if (![filters count])
+  {
+      [filters addObject: [NSDictionary dictionaryWithObject: @"." forKey: @"email"]];
+      [filters addObject: [NSDictionary dictionaryWithObject: @"%" forKey: @"name"]];
+  }
+
   return filters;
 }
 
