@@ -22,7 +22,6 @@
 #define NSSTRING_MAIL_H
 
 #import <Foundation/NSArray.h>
-#import <Foundation/NSString.h>
 
 @interface NSString (SOGoExtension)
 
@@ -34,6 +33,7 @@
       fromIndex: (int) start;
 - (int) indexOf: (unichar) _c;
 - (NSString *) decodedHeader;
+- (NSString *) asSafeFilename;
 
 @end
 
