@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2009-2014 Inverse inc.
-  Copyright (C) 2004-2005 SKYRIX Software AG
+  Copyright (C) 2009-2016 Inverse inc.
 
   This file is part of SOGo.
 
@@ -35,6 +34,7 @@
 */
 
 @class NSArray;
+@class NSMutableDictionary;
 @class NSMutableArray;
 @class NSString;
 
@@ -42,6 +42,7 @@
 @class SOGoDraftsFolder;
 @class SOGoSentFolder;
 @class SOGoTrashFolder;
+@class SOGoJunkFolder;
 
 typedef enum {
   undefined = -1,
@@ -49,16 +50,24 @@ typedef enum {
   rfc4314
 } SOGoIMAPAclStyle;
 
+typedef enum {
+  SOGoMailStandardListing = 0,
+  SOGoMailSubscriptionsManagementListing = 1
+} SOGoMailListingMode;
+
 @interface SOGoMailAccount : SOGoMailBaseObject
 {
   SOGoMailFolder *inboxFolder;
   SOGoDraftsFolder *draftsFolder;
   SOGoSentFolder *sentFolder;
   SOGoTrashFolder *trashFolder;
+  SOGoJunkFolder *junkFolder;
   SOGoIMAPAclStyle imapAclStyle;
   NSMutableArray *identities;
   NSString *otherUsersFolderName;
   NSString *sharedFoldersName;
+  NSMutableDictionary *subscribedFolders;
+  BOOL sieveFolderUTF8Encoding;
 }
 
 - (SOGoIMAPAclStyle) imapAclStyle;
@@ -81,25 +90,31 @@ typedef enum {
 /* folder pathes */
 - (NSArray *) toManyRelationshipKeysWithNamespaces: (BOOL) withNSs;
 
-- (NSArray *) allFolderPaths;
-- (NSArray *) allFoldersMetadata;
+- (NSArray *) allFolderPaths: (SOGoMailListingMode) theListingMode;
+- (NSArray *) allFoldersMetadata: (SOGoMailListingMode) theListingMode;
 
 - (NSDictionary *) imapFolderGUIDs;
 
 - (BOOL) isInDraftsFolder;
 
-/* special folders */
+- (id) lookupNameByPaths: (NSArray *) _paths
+               inContext: (id)_ctx
+                 acquire: (BOOL) _flag;
+
+  /* special folders */
 - (NSString *) inboxFolderNameInContext: (id)_ctx;
 - (NSString *) draftsFolderNameInContext: (id)_ctx;
 - (NSString *) sentFolderNameInContext: (id)_ctx;
 - (NSString *) trashFolderNameInContext: (id)_ctx;
 - (NSString *) otherUsersFolderNameInContext: (id)_ctx;
 - (NSString *) sharedFoldersNameInContext: (id)_ctx;
+- (NSString *) junkFolderNameInContext: (id)_ctx;
 
 - (SOGoMailFolder *) inboxFolderInContext: (id)_ctx;
 - (SOGoDraftsFolder *) draftsFolderInContext: (id)_ctx;
 - (SOGoSentFolder *) sentFolderInContext: (id)_ctx;
 - (SOGoTrashFolder *) trashFolderInContext: (id)_ctx;
+- (SOGoJunkFolder *) junkFolderInContext: (id)_ctx;
 
 /* namespaces */
 
