@@ -20,10 +20,6 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCharacterSet.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSDictionary.h>
 
 #import <NGObjWeb/SoObject+SoDAV.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
@@ -32,22 +28,37 @@
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSURL+misc.h>
 #import <NGImap4/NGImap4Connection.h>
+#import <NGImap4/NGImap4Client.h>
 
 #import <SOGo/SOGoCache.h>
 #import <SOGo/SOGoUser.h>
+#import <SOGo/WORequest+SOGo.h>
 
 #import "SOGoMailAccount.h"
 #import "SOGoMailManager.h"
 
-#import "SOGoMailBaseObject.h"
 
 @implementation SOGoMailBaseObject
 
+
+- (id)init
+{
+  if ((self = [super init]))
+    {
+      NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
+      // With this to YES, imap4Connection will raise exception if a working
+      // connection cannot be provided
+      imap4ExceptionsEnabled = [ud boolForKey:@"SoIMAP4ExceptionsEnabled"];
+    }
+
+  return self;
+}
+
 - (id) initWithImap4URL: (NSURL *) _url
 	    inContainer: (id) _container
 {
   NSString *n;
-  
+
   n = [[_url path] lastPathComponent];
   if ((self = [self initWithName: n inContainer:_container]))
     {
@@ -121,10 +132,23 @@
 - (NGImap4Connection *) _createIMAP4Connection
 {
   NGImap4ConnectionManager *manager;
-  NSString *password;
   NGImap4Connection *newConnection;
+  NSString *password;
+  NSHost *host;
 
   [self imap4URL];
+
+  // We first check if we're trying to establish an IMAP connection to localhost
+  // for an account number greater than 0 (default account). We prevent that
+  // for security reasons if admins use an IMAP trust.
+  host = [NSHost hostWithName: [[self imap4URL] host]];
+  if (![[[self mailAccountFolder] nameInContainer] isEqualToString: @"0"] &&
+      [[host address] isEqualToString: @"127.0.0.1"])
+    {
+      [self errorWithFormat: @"Trying to use localhost for additional IMAP account - aborting."];
+      return nil;
+    }
+
   manager = [self mailManager];
   password = [self imap4PasswordRenewed: NO];
   if (password)
@@ -146,7 +170,10 @@
   if (!newConnection)
     {
       newConnection = (NGImap4Connection *) [NSNull null];
-      [self errorWithFormat:@"Could not connect IMAP4"];
+      if (imap4ExceptionsEnabled)
+        [NSException raise: @"IOException" format: @"IMAP connection failed"];
+      else
+        [self errorWithFormat:@"Could not connect IMAP4"];
     }
   else
     {
@@ -198,12 +225,28 @@
       if (!imap4)
         {
           imap4 = [self _createIMAP4Connection];
-          [sogoCache registerIMAP4Connection: imap4
-                                      forKey: cacheKey];
+	  [sogoCache registerIMAP4Connection: imap4
+				      forKey: cacheKey];
         }
       [imap4 retain];
     }
 
+   // Connection broken, try to reconnect
+   if (![imap4 isKindOfClass: [NSNull class]] && ![[[imap4 client] isConnected] boolValue])
+     {
+       [self warnWithFormat: @"IMAP connection is broken, trying to reconnect..."];
+       [[imap4 client] reconnect];
+
+       // Still broken, give up
+       if (![[[imap4 client] isConnected] boolValue])
+         {
+           if (imap4ExceptionsEnabled)
+             [NSException raise: @"IOException" format: @"IMAP connection failed"];
+           else
+             [self errorWithFormat: @"Could not get a valid IMAP connection"];
+         }
+     }
+
   return [imap4 isKindOfClass: [NSNull class]] ? nil : imap4;
 }
 
