@@ -20,9 +20,6 @@
 */
 
 #import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSURL.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
@@ -40,7 +37,6 @@
 
 #import "NSDictionary+Mail.h"
 #import "SOGoMailObject.h"
-#import "SOGoMailManager.h"
 
 #import "SOGoMailBodyPart.h"
 
@@ -278,7 +274,8 @@ static BOOL debugOn = NO;
       else if ([enc isEqualToString: @"quoted-printable"])
 	data = [data dataByDecodingQuotedPrintableTransferEncoding];
       else if ([enc isEqualToString: @"7bit"]
-	       || [enc isEqualToString: @"8bit"])
+	       || [enc isEqualToString: @"8bit"]
+	       || [enc isEqualToString: @"binary"])
 	; /* keep data as is */ // TODO: do we need to change encodings?
       else
 	{
@@ -396,7 +393,7 @@ static BOOL debugOn = NO;
 	    mimeType = @"application/octet-stream";
 	  
 	  [response setHeader: mimeType forKey: @"content-type"];
-	  [response setHeader: [NSString stringWithFormat:@"%d", [data length]]
+	  [response setHeader: [NSString stringWithFormat:@"%d", (int)[data length]]
 		    forKey: @"content-length"];
   
 	  if (asAttachment)
