@@ -76,6 +76,9 @@
 @interface SOGoMailGermanForward : SOGoMailForward
 @end
 
+@interface SOGoMailHebrewForward : SOGoMailForward
+@end
+
 @interface SOGoMailHungarianForward : SOGoMailForward
 @end
 
@@ -85,6 +88,9 @@
 @interface SOGoMailItalianForward : SOGoMailForward
 @end
 
+@interface SOGoMailLatvianForward : SOGoMailForward
+@end
+
 @interface SOGoMailLithuanianForward : SOGoMailForward
 @end
 
