@@ -287,6 +287,9 @@
 @implementation SOGoMailGermanForward
 @end
 
+@implementation SOGoMailHebrewForward
+@end
+
 @implementation SOGoMailHungarianForward
 @end
 
@@ -296,6 +299,9 @@
 @implementation SOGoMailItalianForward
 @end
 
+@implementation SOGoMailLatvianForward
+@end
+
 @implementation SOGoMailLithuanianForward
 @end
 
