@@ -1,4 +1,5 @@
 /*
+  Copyright (C) 2009-2016 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of OpenGroupware.org.
@@ -48,6 +49,8 @@
 @class NGImap4Envelope;
 @class NGImap4EnvelopeAddress;
 
+@class WOResponse;
+
 NSArray *SOGoMailCoreInfoKeys;
 
 @interface SOGoMailObject : SOGoMailBaseObject
@@ -107,6 +110,8 @@ NSArray *SOGoMailCoreInfoKeys;
 - (BOOL) hasAttachment;
 - (NSDictionary *) fetchFileAttachmentIds;
 - (NSArray *) fetchFileAttachmentKeys;
+- (NSArray *) fetchFileAttachments;
+- (WOResponse *) archiveAllFilesinArchiveNamed: (NSString *) archiveName;
 
 /* flags */
 
@@ -114,6 +119,7 @@ NSArray *SOGoMailCoreInfoKeys;
 - (NSException *) removeFlags:(id)_f;
 
 - (BOOL) isNewMail;  /* \Recent */
+- (BOOL) flagged;    /* \Flagged */
 - (BOOL) read;       /* \Unseen */
 - (BOOL) replied;    /* \Answered */
 - (BOOL) forwarded;  /* $forwarded */
@@ -128,9 +134,9 @@ NSArray *SOGoMailCoreInfoKeys;
                           inContext: (id)_ctx;
 
 - (void) addRequiredKeysOfStructure: (NSDictionary *) info
-			       path: (NSString *) p
-			    toArray: (NSMutableArray *) keys
-		      acceptedTypes: (NSArray *) types
+                               path: (NSString *) p
+                            toArray: (NSMutableArray *) keys
+                      acceptedTypes: (NSArray *) types
                            withPeek: (BOOL) withPeek;
 
 @end
