@@ -75,6 +75,9 @@
 @interface SOGoMailGermanReply : SOGoMailReply
 @end
 
+@interface SOGoMailHebrewReply : SOGoMailReply
+@end
+
 @interface SOGoMailHungarianReply : SOGoMailReply
 @end
 
@@ -84,6 +87,9 @@
 @interface SOGoMailItalianReply : SOGoMailReply
 @end
 
+@interface SOGoMailLatvianReply : SOGoMailReply
+@end
+
 @interface SOGoMailLithuanianReply : SOGoMailReply
 @end
 
