@@ -123,6 +123,9 @@
 @implementation SOGoMailGermanReply
 @end
 
+@implementation SOGoMailHebrewReply
+@end
+
 @implementation SOGoMailHungarianReply
 @end
 
@@ -132,6 +135,9 @@
 @implementation SOGoMailItalianReply
 @end
 
+@implementation SOGoMailLatvianReply
+@end
+
 @implementation SOGoMailLithuanianReply
 @end
 
