@@ -1,8 +1,6 @@
 /* NSObject+Utilities.h - this file is part of SOGo
  *
- * Copyright (C) 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +21,6 @@
 #ifndef NSOBJECT_UTILITIES_H
 #define NSOBJECT_UTILITIES_H
 
-#import <Foundation/NSObject.h>
 
 #import <DOM/DOMProtocols.h>
 
