@@ -1,6 +1,6 @@
 /* NSString+Utilities.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2014 Inverse inc.
+ * Copyright (C) 2006-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,11 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCharacterSet.h>
-#import <Foundation/NSData.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSValue.h>
 
 #import <EOControl/EOQualifier.h>
 
@@ -46,10 +41,6 @@ static NSString **cssEscapingStrings = NULL;
 static unichar *cssEscapingCharacters = NULL;
 static int cssEscapingCount;
 
-static unichar thisCharCode[30];
-static NSString *controlCharString = nil;
-static NSCharacterSet *controlCharSet = nil;
-
 @implementation NSString (SOGoURLExtension)
 
 - (NSString *) composeURLWithAction: (NSString *) action
@@ -116,7 +107,6 @@ static NSCharacterSet *controlCharSet = nil;
 
 //       [urlNonEndingChars addCharactersInString: @">&=,.:;\t \r\n"];
 //       [urlAfterEndingChars addCharactersInString: @"()[]{}&;<\t \r\n"];
-
   if (!urlNonEndingChars)
     {
       urlNonEndingChars = [NSMutableCharacterSet new];
@@ -262,7 +252,7 @@ static NSCharacterSet *controlCharSet = nil;
   return selfCopy;
 }
 
-- (NSString *) doubleQuotedString
+- (NSString *) asSafeJSString
 {
   NSMutableString *representation;
 
@@ -275,34 +265,159 @@ static NSCharacterSet *controlCharSet = nil;
   [representation replaceString: @"\r" withString: @"\\r"];
   [representation replaceString: @"\t" withString: @"\\t"];
 
-  return [NSString stringWithFormat: @"\"%@\"", representation];
+  return representation;
 }
 
-- (NSCharacterSet *) safeCharacterSet
+- (NSString *) doubleQuotedString
 {
-  if (!controlCharSet)
+  return [NSString stringWithFormat: @"\"%@\"", [self asSafeJSString]];
+}
+
+//
+// See http://www.hackcraft.net/xmlUnicode/
+//
+// XML1.0 and XML1.1 allow different characters in different contexts, but for the most part I will only describe the XML1.0 usage, XML1.1 usage is analogous.
+// The first definition that is relevant here is that of a Char:
+// [2]	Char	::=	#x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]	/* any Unicode character, excluding the surrogate blocks, FFFE, and FFFF. */
+//
+// This defines which characters can be used in an XML1.0 document. It is clearly very liberal, banning only some of the control characters and the noncharacters U+FFFE and U+FFFF.
+// Indeed it is somewhat too liberal in my view since it allows other noncharacters (the code points from U+FDD0 to U+FDEF inclusive and the last 2 code points in each plane,
+// from U+1FFFE & U+1FFFF through to U+10FFFE & U+10FFFF, are noncharacters) but the production quoted above allows them.
+//
+- (NSString *) safeString
+{
+  NSData *data;
+  NSString *s;
+
+  const wchar_t *buf;
+  wchar_t *start, c;
+  int len, i, j;
+
+  data = [self dataUsingEncoding: NSUTF32LittleEndianStringEncoding];
+  len = [data length];
+  buf = [data bytes];
+  start = (wchar_t *)calloc(len, sizeof(wchar_t));
+
+  for (i = 0, j = 0; i < len/4; i++)
     {
-      int i, j;
-      
-      // Create an array of chars for all control characters between 0x00 and 0x1F,
-      // apart from \t, \n, \f and \r (0x09, 0x0A, 0x0C and 0x0D)
-      for (i = 0, j = 0x00; j <= 0x08; i++, j++) {
-        thisCharCode[i] = j;
-      }
-      thisCharCode[i++] = 0x0B;
-      for (j = 0x0E; j <= 0x1F; i++, j++) {
-        thisCharCode[i] = j;
-      }
-      
-      // Also add some unicode separators
-      thisCharCode[i++] = 0x2028; // line separator
-      thisCharCode[i++] = 0x2029; // paragraph separator
-      controlCharString = [NSString stringWithCharacters:thisCharCode length:i];
-      controlCharSet = [NSCharacterSet characterSetWithCharactersInString: controlCharString];
-      [controlCharSet retain];
+      c = buf[i];
+
+      if (c == 0x0 ||
+	  c == 0x9 ||
+          c == 0xA ||
+          c == 0xD ||
+          (c >= 0x20 && c <= 0xD7FF) ||
+          (c >= 0xE000 && c <= 0xFFFD) ||
+          (c >= (wchar_t)0x10000 && c <= (wchar_t)0x10FFFF))
+        {
+          start[j] = c;
+          j++;
+        }
     }
 
-  return controlCharSet;
+  s = [[NSString alloc] initWithBytesNoCopy: start  length: j*sizeof(wchar_t)  encoding: NSUTF32LittleEndianStringEncoding  freeWhenDone: YES];
+
+  return AUTORELEASE(s);
+}
+
+- (NSString *) safeStringByEscapingXMLString
+{
+  return [self safeStringByEscapingXMLString: NO];
+}
+//
+// This is a copy from NSString+XMLEscaping.m from SOPE.
+// The difference here is that we use wchar_t instead of unichar.
+// This is needed to get the rigth numeric character reference.
+// e.g. SMILING FACE WITH OPEN MOUTH
+//      ok: wchar_t -> &#128515;   wrong:  unichar -> &#55357; &#56835;
+//
+// We avoid naming it like the one in SOPE since if the ActiveSync
+// bundle is loaded, it'll overwrite the one provided by SOPE.
+//
+- (NSString *) safeStringByEscapingXMLString: (BOOL) encoreCR
+{
+  register unsigned i, len, j;
+  register wchar_t *buf;
+  const wchar_t *chars;
+  unsigned escapeCount;
+
+  if ([self length] == 0) return @"";
+
+  NSData *data = [self dataUsingEncoding:NSUTF32StringEncoding];
+  chars = [data bytes];
+  len = [data length]/4;
+
+  /* check for characters to escape ... */
+  for (i = 0, escapeCount = 0; i < len; i++) {
+    switch (chars[i]) {
+      case '&': case '"': case '<': case '>': case '\r':
+        escapeCount++;
+        break;
+      default:
+        if (chars[i] < 0x20 || chars[i] > 127)
+          escapeCount++;
+        break;
+    }
+  }
+  if (escapeCount == 0 ) {
+    /* nothing to escape ... */
+    return [[self copy] autorelease];
+  }
+
+  buf = calloc((len + 5) + (escapeCount * 16), sizeof(wchar_t));
+  for (i = 0, j = 0; i < len; i++) {
+    switch (chars[i]) {
+      /* escape special chars */
+      case '&':
+        buf[j] = '&'; j++; buf[j] = 'a'; j++; buf[j] = 'm'; j++;
+        buf[j] = 'p'; j++; buf[j] = ';'; j++;
+        break;
+      case '"':
+        buf[j] = '&'; j++; buf[j] = 'q'; j++; buf[j] = 'u'; j++;
+        buf[j] = 'o'; j++; buf[j] = 't'; j++; buf[j] = ';'; j++;
+        break;
+      case '<':
+        buf[j] = '&'; j++; buf[j] = 'l'; j++; buf[j] = 't'; j++;
+        buf[j] = ';'; j++;
+        break;
+      case '>':
+        buf[j] = '&'; j++; buf[j] = 'g'; j++; buf[j] = 't'; j++;
+        buf[j] = ';'; j++;
+        break;
+      case '\r':
+        if (encoreCR) // falls back to default if we don't encode
+          {
+            buf[j] = '&'; j++; buf[j] = '#'; j++; buf[j] = '1'; j++;
+            buf[j] = '3'; j++; buf[j] = ';'; j++;
+            break;
+          }
+      default:
+        /* escape big chars */
+        if (chars[i] > 127) {
+          unsigned char nbuf[32];
+          unsigned int k;
+
+          sprintf((char *)nbuf, "&#%i;", (int)chars[i]);
+          for (k = 0; nbuf[k] != '\0'; k++) {
+            buf[j] = nbuf[k];
+            j++;
+          }
+        }
+        else if (chars[i] == 0x9 || chars[i] == 0xA || chars[i] == 0xD || chars[i] >= 0x20) { // ignore any unsupported control character
+          /* nothing to escape */
+          buf[j] = chars[i];
+          j++;
+        }
+        break;
+    }
+  }
+
+  self = [[NSString alloc] initWithBytesNoCopy: buf
+                                        length: (j*sizeof(wchar_t))
+                                      encoding: NSUTF32StringEncoding
+                                  freeWhenDone: YES];
+
+  return [self autorelease];
 }
 
 - (NSString *) jsonRepresentation
@@ -310,8 +425,7 @@ static NSCharacterSet *controlCharSet = nil;
   NSString *cleanedString;
 
   // Escape double quotes and remove control characters
-  cleanedString = [[[self doubleQuotedString] componentsSeparatedByCharactersInSet: [self safeCharacterSet]]
-                              componentsJoinedByString: @""];
+  cleanedString = [[self doubleQuotedString] safeString];
   return cleanedString;
 }
 
@@ -321,12 +435,18 @@ static NSCharacterSet *controlCharSet = nil;
   int count;
 
   strings = [NSArray arrayWithObjects: @"_U_", @"_D_", @"_H_", @"_A_", @"_S_",
-                     @"_C_", @"_CO_", @"_SP_", @"_SQ_", @"_AM_", @"_P_", @"_DS_", nil];
+                     @"_C_", @"_SC_",
+                     @"_CO_", @"_SP_", @"_SQ_", @"_DQ_",
+                     @"_LP_", @"_RP_", @"_LS_", @"_RS_", @"_LC_", @"_RC_",
+                     @"_AM_", @"_P_", @"_DS_", nil];
   [strings retain];
   cssEscapingStrings = [strings asPointersOfObjects];
 
-  characters = [NSArray arrayWithObjects: @"_", @".", @"#", @"@", @"*", @":",
-                        @",", @" ", @"'", @"&", @"+", @"$", nil];
+  characters = [NSArray arrayWithObjects: @"_", @".", @"#", @"@", @"*",
+                        @":", @";",
+                        @",", @" ", @"'", @"\"",
+                        @"(", @")", @"[", @"]", @"{", @"}",
+                        @"&", @"+", @"$", nil];
   cssEscapingCount = [strings count];
   cssEscapingCharacters = NSZoneMalloc (NULL,
                                         (cssEscapingCount + 1)
@@ -350,6 +470,7 @@ static NSCharacterSet *controlCharSet = nil;
 
 - (NSString *) asCSSIdentifier
 {
+  NSCharacterSet *numericSet;
   NSMutableString *cssIdentifier;
   unichar currentChar;
   int count, max, idx;
@@ -358,10 +479,12 @@ static NSCharacterSet *controlCharSet = nil;
     [self _setupCSSEscaping];
 
   cssIdentifier = [NSMutableString string];
+  numericSet = [NSCharacterSet decimalDigitCharacterSet];
   max = [self length];
+
   if (max > 0)
     {
-      if (isdigit([self characterAtIndex: 0]))
+      if ([numericSet characterIsMember: [self characterAtIndex: 0]])
         // A CSS identifier can't start with a digit; we add an underscore
         [cssIdentifier appendString: @"_"];
       for (count = 0; count < max; count++)
@@ -392,6 +515,7 @@ static NSCharacterSet *controlCharSet = nil;
 
 - (NSString *) fromCSSIdentifier
 {
+  NSCharacterSet *numericSet;
   NSMutableString *newString;
   NSString *currentString;
   int count, length, max, idx;
@@ -400,12 +524,14 @@ static NSCharacterSet *controlCharSet = nil;
   if (!cssEscapingStrings)
     [self _setupCSSEscaping];
 
+  numericSet = [NSCharacterSet decimalDigitCharacterSet];
   newString = [NSMutableString string];
   max = [self length];
   count = 0;
+
   if (max > 0
       && [self characterAtIndex: 0] == '_'
-      && isdigit([self characterAtIndex: 1]))
+      && [numericSet characterIsMember: [self characterAtIndex: 1]])
     {
       /* If the identifier starts with an underscore followed by a digit,
          we remove the underscore */
@@ -463,36 +589,7 @@ static NSCharacterSet *controlCharSet = nil;
 
 - (NSString *) asQPSubjectString: (NSString *) encoding
 {
-  NSString *qpString, *subjectString;
-  NSData *subjectData, *destSubjectData;
-  NSUInteger length, destLength;
-  unsigned char *destString;
-
-#warning "encoding" parameter is not useful
-  subjectData = [self dataUsingEncoding: NSUTF8StringEncoding];
-  length = [subjectData length];
-  destLength = length * 3;
-  destString = calloc (destLength, sizeof (char));
-
-  NGEncodeQuotedPrintableMime ([subjectData bytes], length,
-                               destString, destLength);
-
-  destSubjectData = [NSData dataWithBytesNoCopy: destString
-                                         length: strlen ((char *) destString)
-                                   freeWhenDone: YES];
-  qpString = [[NSString alloc] initWithData: destSubjectData
-			       encoding: NSASCIIStringEncoding];
-  [qpString autorelease];
-  if ([qpString length] > [self length])
-    {
-      qpString = [qpString stringByReplacingString: @" " withString: @"_"];
-      subjectString = [NSString stringWithFormat: @"=?%@?q?%@?=",
-				encoding, qpString];
-    }
-  else
-    subjectString = self;
-
-  return subjectString;
+   return [NGMimeHeaderFieldGenerator encodeQuotedPrintableText: self];
 }
 
 - (BOOL) caseInsensitiveMatches: (NSString *) match
@@ -548,9 +645,9 @@ static NSCharacterSet *controlCharSet = nil;
 - (id) objectFromJSONString
 {
   SBJsonParser *parser;
-  NSObject *object;
+  NSArray *object;
   NSError *error;
-  NSString *unescaped;
+  NSString *unescaped, *json;
 
   object = nil;
 
@@ -559,13 +656,16 @@ static NSCharacterSet *controlCharSet = nil;
       parser = [SBJsonParser new];
       [parser autorelease];
       error = nil;
-      object = [parser objectWithString: self
+
+      /* Parse it this way so we can parse simple values, like "null" */
+      json = [NSString stringWithFormat: @"[%@]", self];
+      object = [parser objectWithString: json
                                   error: &error];
       if (error)
         {
           [self errorWithFormat: @"json parser: %@,"
                 @" attempting once more after unescaping...", error];
-          unescaped = [self stringByReplacingString: @"\\\\"
+          unescaped = [json stringByReplacingString: @"\\\\"
                                          withString: @"\\"];
           object = [parser objectWithString: unescaped
                                       error: &error];
@@ -579,13 +679,14 @@ static NSCharacterSet *controlCharSet = nil;
         }
     }
 
-  return object;
+  return [object objectAtIndex: 0];
 }
 
 - (NSString *) asSafeSQLString
 {
-  return [[self stringByReplacingString: @"\\" withString: @"\\\\"]
-           stringByReplacingString: @"'" withString: @"\\'"];
+  return [[[self stringByReplacingString: @"\\" withString: @"\\\\"]
+           stringByReplacingString: @"'" withString: @"\\'"]
+	   stringByReplacingString: @"\%" withString: @"\\%"];
 }
 
 - (NSUInteger) countOccurrencesOfString: (NSString *) substring
