@@ -373,7 +373,9 @@ static NSArray *childRecordFields = nil;
       dd = [[context activeUser] domainDefaults];
       displayNameFormat = [dd subscriptionFolderFormat];
 
-      name = [folderSubscriptionValues keysWithFormat: displayNameFormat];
+      // Use a format only if it was defined by the user
+      if (displayNameFormat)
+	name = [folderSubscriptionValues keysWithFormat: displayNameFormat];
     }
 
   return name;
@@ -424,7 +426,25 @@ static NSArray *childRecordFields = nil;
 
 - (GCSFolder *) ocsFolderForPath: (NSString *) _path
 {
-  return [[self folderManager] folderAtPath: _path];
+  NSDictionary *record;
+  SOGoCache *cache;
+
+  cache = [SOGoCache sharedCache];
+  record = [[cache valueForKey: _path] objectFromJSONString];
+
+  // We check if we got a cache miss or a potentially bogus
+  // entry from the cache
+  if (!record || ![record objectForKey: @"c_folder_type"])
+    {
+      record = [[self folderManager] recordAtPath: _path];
+
+      if (!record)
+	return nil;
+
+      [cache setValue: [record jsonRepresentation]  forKey: _path];
+    }
+
+  return [[self folderManager] folderForRecord: record];
 }
 
 - (BOOL) folderIsMandatory
@@ -471,10 +491,17 @@ static NSArray *childRecordFields = nil;
 {
   NSMutableArray *newPath;
   NSURL *davURL;
+  unsigned int max, count;
 
   davURL = [self realDavURL];
   newPath = [NSMutableArray arrayWithArray: [[davURL path] componentsSeparatedByString: @"/"]];
   [newPath insertObject: @"public" atIndex: 3];
+
+  max = [newPath count];
+  for (count = 0; count < max; count++)
+    [newPath replaceObjectAtIndex: count
+                       withObject: [[newPath objectAtIndex: count] stringByEscapingURL]];
+
   davURL = [NSURL URLWithString: [newPath componentsJoinedByString: @"/"]
                   relativeToURL: davURL];
 
@@ -495,9 +522,9 @@ static NSArray *childRecordFields = nil;
         publicParticle = @"";
       path = [NSString stringWithFormat: @"/%@/dav%@/%@/%@/%@/",
                        appName, publicParticle,
-                       [self ownerInContext: nil],
-                       [container nameInContainer],
-                       [self realNameInContainer]];
+                       [[self ownerInContext: nil] stringByEscapingURL],
+                       [[container nameInContainer] stringByEscapingURL],
+                       [[self realNameInContainer] stringByEscapingURL]];
       currentDavURL = [self davURL];
       realDavURL = [NSURL URLWithString: path relativeToURL: currentDavURL];
     }
@@ -571,6 +598,7 @@ static NSArray *childRecordFields = nil;
       [self removeFolderSettings: moduleSettings
                    withReference: [self folderReference]];
       [us synchronize];
+      [[SOGoCache sharedCache] removeValueForKey: ocsPath];
 
       if ([[context request] handledByDefaultHandler])
         [self sendFolderAdvisoryTemplate: @"Removal"];
@@ -932,8 +960,12 @@ static NSArray *childRecordFields = nil;
 
       folderSubscription = [moduleSettings objectForKey: @"SubscribedFolders"];
       subscriptionPointer = [self folderReference];
-      
+
+      // We used to set "show alarms" for any type of folder, so we remove it
+      // here and let the subclass handle it (SOGoAppointmentFolder).
       folderShowAlarms = [moduleSettings objectForKey: @"FolderShowAlarms"];
+      if (folderShowAlarms)
+        [folderShowAlarms removeObjectForKey: subscriptionPointer];
 
       if (reallyDo)
         {
@@ -945,30 +977,17 @@ static NSArray *childRecordFields = nil;
                                  forKey: @"SubscribedFolders"];
             }
 
-	  if (!(folderShowAlarms
-                && [folderShowAlarms isKindOfClass: [NSMutableDictionary class]]))
-            {
-              folderShowAlarms = [NSMutableDictionary dictionary];
-              [moduleSettings setObject: folderShowAlarms
-                                 forKey: @"FolderShowAlarms"];
-            }
-
           [self setFolderPropertyValue: [self _displayNameFromSubscriber]
                             inCategory: @"FolderDisplayNames"
                               settings: us];
 
           [folderSubscription addObjectUniquely: subscriptionPointer];
-	  
-	  // By default, we disable alarms on subscribed calendars
-	  [folderShowAlarms setObject: [NSNumber numberWithBool: NO]
-			       forKey: subscriptionPointer];
         }
       else
         {
           [self removeFolderSettings: moduleSettings
                        withReference: subscriptionPointer];
           [folderSubscription removeObject: subscriptionPointer];
-	  [folderShowAlarms removeObjectForKey: subscriptionPointer];
 	}
 
       [us synchronize];
