@@ -1,6 +1,6 @@
 /* SOGoParentFolder.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2015 Inverse inc.
+ * Copyright (C) 2006-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -291,14 +291,8 @@ static SoSecurityManager *sm = nil;
   // This is important because user A could delete folder X, and user B has subscribed to it.
   // If the "default roles" are enabled for calendars/address books, -validatePersmission:.. will
   // work (grabbing the default role) and the deleted resource will be incorrectly returned.
-  //
-  // FIXME - 2015/01/29 - this fix (24c6c8c91d421594bd51f07904d4cd3911cd187c) was reverted. Normally, we should add
-  // [subscribedFolder ocsFolderForPath: [subscribedFolder ocsPath]] to check the existence of the folder but it causes
-  // massive SQL traffic.
-  // 
-  // The proper fix would be to check upon login and only upon login if GCS folders that we're subscribed to are still existent.
-  //
   if (subscribedFolder
+      && [subscribedFolder ocsFolderForPath: [subscribedFolder ocsPath]]
       && ![sm validatePermission: SOGoPerm_AccessObject
 			onObject: subscribedFolder
 		       inContext: context])
@@ -316,16 +310,23 @@ static SoSecurityManager *sm = nil;
   NSMutableDictionary *folderDisplayNames;
   NSMutableArray *subscribedReferences;
   SOGoUserSettings *settings;
-  NSString *currentKey;
+  NSString *activeUser, *currentKey;
   SOGoUser *ownerUser;
   NSException *error;
   id o;
   int i;
   BOOL dirty;
 
+  if (!subscribedSubFolders)
+    subscribedSubFolders = [NSMutableDictionary new];
+
+  if (!subFolderClass)
+    subFolderClass = [[self class] subFolderClass];
+
   error = nil; /* we ignore non-DB errors at this time... */
   dirty = NO;
 
+  activeUser = [[context activeUser] login];
   ownerUser = [SOGoUser userWithLogin: owner];
   settings = [ownerUser userSettings];
 
@@ -346,7 +347,9 @@ static SoSecurityManager *sm = nil;
 	  // remove it from the current list.
 	  [subscribedReferences removeObject: currentKey];
 	  [folderDisplayNames removeObjectForKey: currentKey];
-	  dirty = YES;
+          if ([owner isEqualToString: activeUser])
+            // Synchronize settings only if the subscription is owned by the active user
+            dirty = YES;
 	}
     }
   
