@@ -96,8 +96,8 @@
 
 - (int) maximumMessageSizeLimit;
 
-- (int) maximumMessageSubmissionCount;
-- (int) maximumRecipientCount;
+- (NSUInteger) maximumMessageSubmissionCount;
+- (NSUInteger) maximumRecipientCount;
 - (int) maximumSubmissionInterval;
 - (int) messageSubmissionBlockInterval;
 
@@ -111,6 +111,7 @@
 - (int) internalSyncInterval;
 - (int) maximumSyncWindowSize;
 - (int) maximumSyncResponseSize;
+- (int) maximumPictureSize;
 
 @end
 
