@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2006-2015 Inverse inc.
-  Copyright (C) 2005 SKYRIX Software AG
+  Copyright (C) 2006-2016 Inverse inc.
 
   This file is part of SOGo.
 
@@ -15,23 +14,14 @@
   License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with OGo; see the file COPYING.  If not, write to the
+  License along with SOGo; see the file COPYING.  If not, write to the
   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSNull.h>
-#import <Foundation/NSTimeZone.h>
-#import <Foundation/NSValue.h>
-#import <Foundation/NSURL.h>
 #import <NGObjWeb/WOApplication.h>
 #import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/SoObject.h>
 #import <NGExtensions/NSCalendarDate+misc.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
@@ -42,14 +32,10 @@
 #import "NSArray+Utilities.h"
 #import "SOGoCache.h"
 #import "SOGoDateFormatter.h"
-#import "SOGoDomainDefaults.h"
-#import "SOGoObject.h"
 #import "SOGoPermissions.h"
 #import "SOGoSystemDefaults.h"
-#import "SOGoUserDefaults.h"
 #import "SOGoUserFolder.h"
 #import "SOGoUserManager.h"
-#import "SOGoUserProfile.h"
 #import "SOGoUserSettings.h"
 #import "WOResourceManager+SOGo.h"
 
@@ -165,10 +151,9 @@
               // The domain is probably appended to the username;
               // make sure it is defined as a domain in the configuration.
               domain = [newLogin substringFromIndex: (r.location + r.length)];
-              if ([[sd domainIds] containsObject: domain])
+              if ([[SOGoUserManager sharedUserManager] isDomainDefined: domain] &&
+                  ![sd enableDomainBasedUID])
                 newLogin = [newLogin substringToIndex: r.location];
-              else
-                domain = nil;
 
               if (domain != nil && ![sd enableDomainBasedUID])
                 // Login domains are enabled (SOGoLoginDomains) but not
@@ -197,8 +182,25 @@
           // When the user is associated to a domain, the [SOGoUser login]
           // method returns the combination login@domain while
           // [SOGoUser loginInDomain] only returns the login.
-          uid = [NSString stringWithString: realUID];
-          realUID = [NSString stringWithFormat: @"%@@%@", realUID, domain];
+          r = [realUID rangeOfString: domain  options: NSBackwardsSearch|NSCaseInsensitiveSearch];
+
+          // Do NOT strip @domain.com if SOGoEnableDomainBasedUID is enabled since
+          // the real login most likely is the email address.
+          if (r.location != NSNotFound && ![sd enableDomainBasedUID])
+            uid = [realUID substringToIndex: r.location-1];
+          // If we don't have the domain in the UID but SOGoEnableDomainBasedUID is
+          // enabled, let's add it internally so so it becomes unique across
+          // all potential domains.
+          else if (r.location == NSNotFound && [sd enableDomainBasedUID])
+            {
+              uid = [NSString stringWithString: realUID];
+              realUID = [NSString stringWithFormat: @"%@@%@", realUID, domain];
+            }
+          // We found the domain and SOGoEnableDomainBasedUID is enabled,
+          // we keep realUID.. This would happen for example if the user
+          // authenticates with foo@bar.com and the UIDFieldName is also foo@bar.com
+          else if ([sd enableDomainBasedUID])
+            uid = [NSString stringWithString: realUID];
         }
     }
 
@@ -378,6 +380,39 @@
   return dateFormatter;
 }
 
+- (NSDictionary *) currentDay
+{
+  NSCalendarDate *now;
+  NSDictionary *description, *abbr;
+  NSDictionary *locale;
+  SOGoUserDefaults *ud;
+  WOResourceManager *resMgr;
+  NSUInteger seconds;
+
+  now = [NSCalendarDate calendarDate];
+  ud = [self userDefaults];
+  resMgr = [[WOApplication application] resourceManager];
+  locale = [resMgr localeForLanguageNamed: [ud language]];
+
+  [now setTimeZone: [ud timeZone]];
+  seconds = [now hourOfDay]*3600 + [now minuteOfHour]*60 + [now secondOfMinute];
+
+  abbr = [NSDictionary dictionaryWithObjectsAndKeys:
+                 [now descriptionWithCalendarFormat: @"%a" locale: locale], @"weekday",
+                 [now descriptionWithCalendarFormat: @"%b" locale: locale], @"month",
+                       nil];
+  description = [NSDictionary dictionaryWithObjectsAndKeys:
+                        [now descriptionWithCalendarFormat: @"%A" locale: locale], @"weekday",
+                        [now descriptionWithCalendarFormat: @"%B" locale: locale], @"month",
+                        [now descriptionWithCalendarFormat: @"%d" locale: locale], @"day",
+                        [now descriptionWithCalendarFormat: @"%Y" locale: locale], @"year",
+                        abbr, @"abbr",
+                        [NSNumber numberWithInt: (24*3600 - seconds)], @"secondsBeforeTomorrow",
+                        nil];
+
+  return description;
+}
+
 - (SOGoUserDefaults *) userDefaults
 {
   if (!_defaults)
@@ -498,15 +533,26 @@
 
 - (unsigned int) weekNumberForDate: (NSCalendarDate *) date
 {
-  NSCalendarDate *firstWeek;
+  NSCalendarDate *firstWeek, *previousWeek;
   unsigned int weekNumber;
 
   firstWeek = [self firstWeekOfYearForDate: date];
   if ([firstWeek earlierDate: date] == firstWeek)
-    weekNumber = ([date timeIntervalSinceDate: firstWeek]
-		  / (86400 * 7) + 1);
+    {
+      weekNumber = ([date timeIntervalSinceDate: firstWeek] / (86400 * 7) + 1);
+    }
   else
-    weekNumber = 0;
+    {
+      // Date is within the last week of the previous year;
+      // Compute the previous week number to find the week number of the requested date.
+      // The number will either be 52 or 53.
+      previousWeek = [date dateByAddingYears: 0
+                                      months: 0
+                                        days: -7];
+      firstWeek = [self firstWeekOfYearForDate: previousWeek];
+      weekNumber = ([previousWeek timeIntervalSinceDate: firstWeek] / (86400 * 7) + 1);
+      weekNumber += 1;
+    }
 
   return weekNumber;
 }
@@ -574,20 +620,23 @@
     }
 }
 
-- (void) _appendSystemMailAccount
+- (void) _appendSystemMailAccountWithDelegatedIdentities: (BOOL) appendDeletegatedIdentities
 {
   NSString *fullName, *replyTo, *imapLogin, *imapServer, *cImapServer, *signature,
     *encryption, *scheme, *action, *query, *customEmail, *defaultEmail, *sieveServer;
-  NSMutableDictionary *mailAccount, *identity, *mailboxes, *receipts;
+  NSMutableDictionary *mailAccount, *identity, *mailboxes, *receipts, *mailSettings;
   NSNumber *port;
-  NSMutableArray *identities;
-  NSArray *mails;
+  NSMutableArray *identities, *mails;
+  NSArray *delegators, *delegates;
   NSURL *url, *cUrl;
   unsigned int count, max, default_identity;
   NSInteger defaultPort;
+  NSUInteger index;
 
   [self userDefaults];
+  [self userSettings];
 
+  mailSettings = [_settings objectForKey: @"Mail"];
   mailAccount = [NSMutableDictionary new];
 
   // 1. login
@@ -665,14 +714,12 @@
   defaultEmail = [NSString stringWithFormat: @"%@@%@", [self loginInDomain], [self domain]];
   default_identity = 0;
   identities = [NSMutableArray new];
-  mails = [self allEmails];
+  mails = [NSMutableArray arrayWithArray: [self allEmails]];
   [mailAccount setObject: [mails objectAtIndex: 0] forKey: @"name"];
 
   replyTo = [_defaults mailReplyTo];
 
   max = [mails count];
-  if (max > 1)
-    max--;
 
   /* custom from */
   if ([[self domainDefaults] mailCustomFromEnabled])
@@ -684,6 +731,17 @@
         {
           if ([customEmail length] == 0)
             customEmail = [mails objectAtIndex: 0];
+          else if ([fullName length] == 0)
+            {
+              // Custom email but default fullname; if the custom email is
+              // one of the user's emails, remove the duplicated entry
+              index = [mails indexOfObject: customEmail];
+              if (index != NSNotFound)
+                {
+                  [mails removeObjectAtIndex: index];
+                  max--;
+                }
+            }
 
           if ([fullName length] == 0)
             {
@@ -730,13 +788,60 @@
       [identities addObject: identity];
 
       if ([[identity objectForKey: @"email"] caseInsensitiveCompare: defaultEmail] == NSOrderedSame)
-            default_identity = [identities count]-1;
+        default_identity = [identities count]-1;
 
       [identity release];
     }
   [[identities objectAtIndex: default_identity] setObject: [NSNumber numberWithBool: YES]
                                                    forKey: @"isDefault"];
-  
+
+  /* identities from delegators */
+  if (appendDeletegatedIdentities)
+    {
+      delegators = [mailSettings objectForKey: @"DelegateFrom"];
+      if (delegators)
+        {
+          BOOL dirty;
+          NSDictionary *delegatorAccount, *delegatorSettings;
+          NSMutableArray *validDelegators;
+          NSString *delegatorLogin;
+          SOGoUser *delegatorUser;
+
+          dirty = NO;
+          validDelegators = [NSMutableArray array];
+          max = [delegators count];
+          for (count = 0; count < max; count++)
+            {
+              // 1. Verify if delegator is valid
+              delegatorLogin = [delegators objectAtIndex: count];
+              delegatorUser = [SOGoUser userWithLogin: delegatorLogin];
+              if (delegatorUser)
+                {
+                  // 2. Verify if delegator still delegates to user
+                  delegatorSettings = [[delegatorUser userSettings] objectForKey: @"Mail"];
+                  delegates = [delegatorSettings objectForKey: @"DelegateTo"];
+                  if ([delegates containsObject: [self login]])
+                    {
+                      [validDelegators addObject: delegatorLogin];
+                      delegatorAccount = [[delegatorUser mailAccountsWithDelegatedIdentities: NO] objectAtIndex: 0];
+                      [identities addObjectsFromArray: [delegatorAccount objectForKey: @"identities"]];
+                    }
+                  else
+                    dirty = YES;
+                }
+              else
+                dirty = YES;
+            }
+
+          if (dirty)
+            {
+              [mailSettings setObject: validDelegators
+                               forKey: @"DelegateFrom"];
+              [_settings synchronize];
+            }
+        }
+    }
+
   [mailAccount setObject: identities forKey: @"identities"];
   [identities release];
 
@@ -770,21 +875,50 @@
                 forKey: @"Sent"];
   [mailboxes setObject: [_defaults trashFolderName]
                 forKey: @"Trash"];
-  [mailAccount setObject: mailboxes forKey: @"mailboxes"];
+  [mailboxes setObject: [_defaults junkFolderName]
+                forKey: @"Junk"];
+  [mailAccount setObject: mailboxes forKey: @"specialMailboxes"];
   [mailboxes release];
 
+  /* delegates */
+  delegates = [mailSettings objectForKey: @"DelegateTo"];
+  if (!delegates)
+    delegates = [NSArray array];
+  else
+    {
+      NSMutableArray *allDelegates;
+      SOGoUser *delegate;
+
+      allDelegates = [NSMutableArray array];
+      for (count = 0; count < [delegates count]; count++)
+        {
+          delegate = [SOGoUser userWithLogin: [delegates objectAtIndex: count]];
+          [allDelegates addObject: [NSDictionary dictionaryWithObjectsAndKeys: [delegates objectAtIndex: count], @"uid",
+                                                 [delegate cn], @"cn",
+                                                 [delegate systemEmail], @"c_email", nil]];
+        }
+
+      delegates = allDelegates;
+    }
+  [mailAccount setObject: delegates  forKey: @"delegates"];
+
   [mailAccounts addObject: mailAccount];
   [mailAccount release];
 }
 
 - (NSArray *) mailAccounts
+{
+  return [self mailAccountsWithDelegatedIdentities: YES];
+}
+
+- (NSArray *) mailAccountsWithDelegatedIdentities: (BOOL) appendDeletegatedIdentities
 {
   NSArray *auxAccounts;
 
   if (!mailAccounts)
     {
       mailAccounts = [NSMutableArray new];
-      [self _appendSystemMailAccount];
+      [self _appendSystemMailAccountWithDelegatedIdentities: appendDeletegatedIdentities];
       if ([[self domainDefaults] mailAuxiliaryUserAccountsEnabled])
         {
           auxAccounts = [[self userDefaults] auxiliaryMailAccounts];
