@@ -1,8 +1,6 @@
 /* SOGoToolRestore.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2012 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2009-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +18,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSError.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSString.h>
+#import <Foundation/NSUserDefaults.h>
 
 #import <GDLAccess/EOAdaptorChannel.h>
 #import <GDLAccess/EOAdaptorContext.h>
@@ -36,39 +33,30 @@
 
 #import <Appointments/iCalEntityObject+SOGo.h>
 #import <SOGo/NSArray+Utilities.h>
+#import "SOGo/SOGoCredentialsFile.h"
 #import <SOGo/SOGoProductLoader.h>
 #import <SOGo/SOGoUser.h>
-#import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoUserManager.h>
 #import <SOGo/SOGoUserProfile.h>
 #import <SOGo/SOGoUserSettings.h>
+#import <SOGo/SOGoSystemDefaults.h>
 
-#import "SOGoTool.h"
+#import <Mailer/SOGoMailAccounts.h>
+#import <Mailer/SOGoMailAccount.h>
+
+#import <NGCards/iCalCalendar.h>
+#import <NGCards/NGVList.h>
+
+#import <NGObjWeb/WOContext+SoObjects.h>
+
+#import "SOGoToolRestore.h"
 
 /* TODO:
    - respond to "--help restore"
    - handle database connectivity errors
    - handle the case where the restored folder has been deleted
    - write methods in GDLContentStore to get/update displayname
-     and storing roles */ 
-
-typedef enum SOGoToolRestoreMode {
-  SOGoToolRestoreFolderMode,
-  SOGoToolRestoreFolderDestructiveMode,
-  SOGoToolRestoreListFoldersMode,
-  SOGoToolRestorePreferencesMode
-} SOGoToolRestoreMode;
-
-@interface SOGoToolRestore : SOGoTool
-{
-  NSString *directory;
-  NSString *userID;
-  NSString *restoreFolder;
-  BOOL destructive; /* destructive mode not handled */
-  SOGoToolRestoreMode restoreMode;
-}
-
-@end
+     and storing roles */
 
 @implementation SOGoToolRestore
 
@@ -88,6 +76,7 @@ typedef enum SOGoToolRestoreMode {
     {
       directory = nil;
       userID = nil;
+      filename = nil;
       restoreFolder = nil;
       destructive = NO;
     }
@@ -99,23 +88,52 @@ typedef enum SOGoToolRestoreMode {
 {
   [directory release];
   [userID release];
+  [filename release];
   [restoreFolder release];
   [super dealloc];
 }
 
 - (void) usage
 {
-  fprintf (stderr, "restore [-l|-p|-f/-F folder/ALL|-p] directory user\n\n"
+  fprintf (stderr, "restore [-l|-p|-f/-F folder/ALL|-p] [-c credentialFile] directory user\n\n"
 	   "           directory  the directory where backup files were initially stored\n"
 	   "           user       the user of whom to restore the data\n"
 	   "           -l         flag used to list folders to restore\n"
 	   "           -p         flag used to restore only the user's preferences\n"
-	   "           -f/-F      flag used to specify which folder to restore, ALL for everything\n\n"
+	   "           -f/-F      flag used to specify which folder to restore, ALL for everything\n"
+	   "           -c credentialFile    Specify the file containing the sieve admin credentials\n"
+	   "                                If set, it is used automatically to generate the sieve\n"
+           "                                script after a data restore.\n"
+	   "                                The file should contain a single line:\n"
+	   "                                username:password\n\n"
 	   "Examples:   sogo-tool restore -l /tmp/foo bob\n"
 	   "            sogo-tool restore -f Contacts/personal /tmp/foo bob\n"
 	   "            sogo-tool restore -p /tmp/foo bob\n");
 }
 
+- (Class) parsingClassForContent: (NSString *) theContent
+{
+  CardGroup *cardEntry;
+  NSString *firstTag;
+  Class objectClass;
+
+  objectClass = Nil;
+
+  cardEntry = [CardGroup parseSingleFromSource: theContent];
+  if (cardEntry)
+    {
+      firstTag = [[cardEntry tag] uppercaseString];
+      if ([firstTag isEqualToString: @"VCARD"])
+	objectClass = [NGVCard class];
+      else if ([firstTag isEqualToString: @"VLIST"])
+	objectClass = [NGVList class];
+      else
+	objectClass = [iCalCalendar class];
+    }
+
+  return objectClass;
+}
+
 - (BOOL) checkDirectory
 {
   NSFileManager *fm;
@@ -144,13 +162,35 @@ typedef enum SOGoToolRestoreMode {
 
 - (BOOL) fetchUserID: (NSString *) identifier
 {
-  BOOL rc;
+  SOGoSystemDefaults *sd;
   SOGoUserManager *lm;
   NSDictionary *infos;
+  NSString *uid = nil;
+
+  BOOL rc;
 
   lm = [SOGoUserManager sharedUserManager];
   infos = [lm contactInfosForUserWithUIDorEmail: identifier];
-  ASSIGN (userID, [infos objectForKey: @"c_uid"]);
+  uid = nil;
+
+  if (infos)
+    {
+      sd = [SOGoSystemDefaults sharedSystemDefaults];
+      uid = [infos objectForKey: @"c_uid"];
+
+      if ([sd enableDomainBasedUID] && [uid rangeOfString: @"@"].location == NSNotFound)
+        uid = [NSString stringWithFormat: @"%@@%@",
+                     [infos objectForKey: @"c_uid"],
+                     [infos objectForKey: @"c_domain"]];
+
+      if ([[infos objectForKey: @"DomainLessLogin"] boolValue])
+        ASSIGN(filename, [infos objectForKey: @"c_uid"]);
+      else
+        ASSIGN(filename, uid);
+    }
+
+  ASSIGN (userID, uid);
+
   if (userID)
     rc = YES;
   else
@@ -214,14 +254,22 @@ typedef enum SOGoToolRestoreMode {
   BOOL rc;
   NSString *identifier;
   NSArray *newArguments;
-  int count, max;
+  int count, max, v;
 
   count = [self parseModeArguments];
   max = [arguments count] - count;
-  if (max == 2)
+  v = 2;
+
+  if ([[NSUserDefaults standardUserDefaults] stringForKey: @"c"])
+    {
+      count = 3;
+      v = 4;
+    }
+
+  if (max == v)
     {
       newArguments
-        = [arguments subarrayWithRange: NSMakeRange (count, max)];
+        = [arguments subarrayWithRange: NSMakeRange (count, 2)];
       ASSIGN (directory, [newArguments objectAtIndex: 0]);
       identifier = [newArguments objectAtIndex: 1];
       rc = ([self checkDirectory]
@@ -347,11 +395,12 @@ typedef enum SOGoToolRestoreMode {
 - (BOOL) restoreRecords: (NSArray *) records
                ofFolder: (GCSFolder *) gcsFolder
 {
-  NSAutoreleasePool *pool;
   NSDictionary *existingRecords, *currentRecord;
   NSString *cName, *cContent;
-  int count, max;
+  NSAutoreleasePool *pool;
+
   unsigned int version;
+  int count, max;
   BOOL rc;
 
   if (records)
@@ -374,7 +423,10 @@ typedef enum SOGoToolRestoreMode {
             {
               NSLog (@"restoring record '%@'", cName);
               cContent = [currentRecord objectForKey: @"c_content"];
-              [gcsFolder writeContent: cContent toName: cName
+              [gcsFolder writeContent: cContent
+                        fromComponent: [[self parsingClassForContent: cContent] parseSingleFromSource: cContent]
+                            container: nil
+                               toName: cName
                           baseVersion: &version];
             }
         }
@@ -529,15 +581,16 @@ typedef enum SOGoToolRestoreMode {
   if (tables)
     {
       NSLog (@"Restorable folders:");
-      folderPrefixLen = 1 + [userID length];
-      tableKeys = [[tables allKeys] objectEnumerator];
+      folderPrefixLen = 8 + [userID length]; // tables keys start with /Users/<userID>
+      tableKeys = [[[tables allKeys] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] objectEnumerator];
       while ((key = [tableKeys nextObject]))
         {
           currentFolder = [tables objectForKey: key];
           folderKey = [key substringFromIndex: folderPrefixLen];
-          NSLog (@"  '%@': %@",
-                 [currentFolder objectForKey: @"displayname"], folderKey);
+          NSLog (@"  %@ (%@)",
+                 folderKey, [currentFolder objectForKey: @"displayname"]);
         }
+      rc = YES;
     }
   else
     {
@@ -548,6 +601,51 @@ typedef enum SOGoToolRestoreMode {
   return rc;
 }
 
+//
+// We regenerate the Sieve script
+//
+- (BOOL) _updateSieveScripsForLogin: (NSString *) theLogin
+{
+  /* credentials file handling */
+  NSString *credsFilename, *authname=nil, *authpwd=nil;
+  SOGoCredentialsFile *cf;
+  SOGoUser *user;
+  SOGoUserFolder *home;
+  SOGoMailAccounts *folder;
+  SOGoMailAccount *account;
+  WOContext *localContext;
+  Class SOGoMailAccounts_class;
+
+  credsFilename = [[NSUserDefaults standardUserDefaults] stringForKey: @"c"];
+  if (credsFilename)
+    {
+      cf = [SOGoCredentialsFile credentialsFromFile: credsFilename];
+      authname = [cf username];
+      authpwd = [cf password];
+    }
+
+  if (authname == nil || authpwd == nil)
+    {
+      NSLog(@"To update Sieve scripts, you must provide the \"-p credentialFile\" parameter");
+      return NO;
+    }
+
+  /* update sieve script */
+  [[SOGoProductLoader productLoader] loadProducts: [NSArray arrayWithObject: @"Mailer.SOGo"]];
+  SOGoMailAccounts_class = NSClassFromString(@"SOGoMailAccounts");
+
+  user = [SOGoUser userWithLogin: theLogin];
+  localContext = [WOContext context];
+  [localContext setActiveUser: user];
+
+  home = [user homeFolderInContext: localContext];
+  folder = [SOGoMailAccounts_class objectWithName: @"Mail" inContainer: home];
+  account = [folder lookupName: @"0" inContext: localContext acquire: NO];
+  [account setContext: localContext];
+
+  return [account updateFiltersWithUsername: authname  andPassword: authpwd];
+}
+
 - (BOOL) restoreUserPreferencesFromUserRecord: (NSDictionary *) userRecord
 {
   SOGoUser *sogoUser;
@@ -563,7 +661,14 @@ typedef enum SOGoToolRestoreMode {
 
       up = [[sogoUser userDefaults] source];
       [up setValues: [preferences objectAtIndex: 0]];
-      [up synchronize];
+
+      if ([[NSUserDefaults standardUserDefaults] stringForKey: @"c"])
+	{
+	  if ([self _updateSieveScripsForLogin: userID])
+	    [up synchronize];
+	}
+      else
+	[up synchronize];
 
       up = [[sogoUser userSettings] source];
       [up setValues: [preferences objectAtIndex: 1]];
@@ -584,7 +689,7 @@ typedef enum SOGoToolRestoreMode {
   NSString *importPath;
   BOOL rc;
 
-  importPath = [directory stringByAppendingPathComponent: userID];
+  importPath = [directory stringByAppendingPathComponent: filename];
   userRecord = [NSDictionary dictionaryWithContentsOfFile: importPath];
   if (userRecord)
     {
@@ -602,7 +707,7 @@ typedef enum SOGoToolRestoreMode {
   else
     {
       rc = NO;
-      NSLog (@"user backup file could not be loaded");
+      NSLog(@"user backup (%@) file could not be loaded", importPath);
     }
 
   return rc;
