@@ -20,7 +20,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WORequest.h>
 
 #import <SoObjects/SOGo/SOGoUser.h>
@@ -46,6 +45,11 @@
   [super dealloc];
 }
 
+- (NSString *) modulePath
+{
+  return @"Administration";
+}
+
 - (BOOL) shouldTakeValuesFromRequest: (WORequest *) request
                            inContext: (WOContext*) context
 {
