@@ -22,12 +22,10 @@
 
 #import <Foundation/NSKeyValueCoding.h>
 #import <NGExtensions/NGExtensions.h>
-#import <NGObjWeb/NGObjWeb.h>
 #import <NGObjWeb/SoObjects.h>
 
 #import <SOGoUI/UIxComponent.h>
 
-#import <NGObjWeb/SoComponent.h>
 
 @class NSArray, NSDictionary;
 
@@ -113,8 +111,9 @@
   
   rm = [self pageResourceManager];
 
-  return [rm pathForResourceNamed: rn inFramework: fw 
-	     languages: [[self context] resourceLookupLanguages]];
+  return [rm pathForResourceNamed: rn
+                      inFramework: fw
+                        languages: languages];
 }
 
 - (id)loadToolbarConfigFromResourceNamed:(NSString *)_name {
@@ -166,7 +165,7 @@
     [self errorWithFormat:
             @"not toolbar configuration found on SoObject: %@ (%@)",
             [self clientObject], [[self clientObject] soClass]];
-    toolbarConfig = [[NSNull null] retain];
+    toolbarConfig = (NSArray *)[[NSNull null] retain];
     return nil;
   }
 
