@@ -1,8 +1,6 @@
 /* UIxContactActions.m - this file is part of SOGo
  *
- * Copyright (C) 2010 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2010-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +19,10 @@
  */
 
 #import <Foundation/NSArray.h>
-#import <Foundation/NSString.h>
+
+#import <NGExtensions/NSString+misc.h>
 
 #import <NGObjWeb/WOContext.h>
-#import <NGObjWeb/WODirectAction.h>
 #import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WOResponse.h>
 
@@ -32,7 +30,7 @@
 
 #import <Contacts/SOGoContactGCSEntry.h>
 
-#import <Common/WODirectAction+SOGo.h>
+#import <SOGoUI/SOGoDirectAction.h>
 
 @interface NGVCard (SOGoActionCategory)
 
@@ -82,7 +80,7 @@
 
 @end
 
-@interface UIxContactActions : WODirectAction
+@interface UIxContactActions : SOGoDirectAction
 
 - (WOResponse *) setCategoryAction;
 - (WOResponse *) unsetCategoryAction;
@@ -141,10 +139,10 @@
   content = [NSMutableString string];
   response = [context response];
 
-  [content appendFormat: [[self clientObject] contentAsString]];
+  [content appendFormat: @"%@", [[self clientObject] contentAsString]];
   [response setHeader: @"text/plain; charset=utf-8" 
             forKey: @"content-type"];
-  [response appendContentString: content];
+  [response appendContentString: [content stringByEscapingHTMLString]];
 
   return response;
 }
