@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2005-2015 Inverse inc.
+  Copyright (C) 2005-2017 Inverse inc.
 
   This file is part of SOGo
 
@@ -337,11 +337,13 @@ static Class SOGoContactGCSEntryK = Nil;
 
 - (void) setAttributes: (NSDictionary *) attributes
 {
-  CardElement *element;
+  NSMutableArray *orgs, *categories;
+  NSDictionary *customFields;
   NSArray *elements, *values;
-  NSMutableArray *units, *categories;
   NSCalendarDate *date;
+  CardElement *element;
   id o;
+
   unsigned int i, year, month, day;
 
   [card setNWithFamily: [attributes objectForKey: @"c_sn"]
@@ -398,27 +400,13 @@ static Class SOGoContactGCSEntryK = Nil;
         }
     }
 
-  if ([[attributes objectForKey: @"orgUnits"] isKindOfClass: [NSArray class]])
-    {
-      elements = [card childrenWithTag: @"org"];
-      [card removeChildren: elements];
-      values = [attributes objectForKey: @"orgUnits"];
-      units = [NSMutableArray arrayWithCapacity: [values count]];
-      for (i = 0; i < [values count]; i++)
-        {
-          o = [values objectAtIndex: i];
-          if ([o isKindOfClass: [NSDictionary class]])
-            {
-              [units addObject: [o objectForKey: @"value"]];
-            }
-        }
-    }
+  if ([[attributes objectForKey: @"orgs"] isKindOfClass: [NSArray class]])
+    orgs = [NSMutableArray arrayWithArray: [attributes objectForKey: @"orgs"]];
   else
-    {
-      units = nil;
-    }
-  [card setOrg: [attributes objectForKey: @"c_org"]
-         units: units];
+    orgs = [NSMutableArray array];
+  if ([[attributes objectForKey: @"org"] length])
+    [orgs insertObject: [attributes objectForKey: @"org"] atIndex: 0];
+  [card setOrganizations: orgs];
 
   elements = [card childrenWithTag: @"tel"];
   [card removeChildren: elements];
@@ -499,6 +487,38 @@ static Class SOGoContactGCSEntryK = Nil;
       [card setCategories: categories];
     }
 
+  customFields = [attributes objectForKey: @"customFields"];
+  if (customFields && [customFields isKindOfClass: [NSDictionary class]])
+    {
+      if ((o = [customFields objectForKey: @"1"]))
+        {
+          elements = [card childrenWithTag: @"custom1"];
+          [card removeChildren: elements];
+          [card addElementWithTag: @"custom1"  ofType: nil  withValue: o];
+        }
+
+      if ((o = [customFields objectForKey: @"2"]))
+        {
+          elements = [card childrenWithTag: @"custom2"];
+          [card removeChildren: elements];
+          [card addElementWithTag: @"custom2"  ofType: nil  withValue: o];
+        }
+
+      if ((o = [customFields objectForKey: @"3"]))
+        {
+          elements = [card childrenWithTag: @"custom3"];
+          [card removeChildren: elements];
+          [card addElementWithTag: @"custom1"  ofType: nil  withValue: o];
+        }
+
+      if ((o = [customFields objectForKey: @"4"]))
+        {
+          elements = [card childrenWithTag: @"custom4"];
+          [card removeChildren: elements];
+          [card addElementWithTag: @"custom1"  ofType: nil  withValue: o];
+        }
+    }
+
   [card cleanupEmptyChildren];
 }
 
@@ -520,7 +540,9 @@ static Class SOGoContactGCSEntryK = Nil;
  * @apiParam {String} c_cn                 Fullname
  * @apiParam {String} c_screenname         Screen Name (X-AIM for now)
  * @apiParam {String} tz                   Timezone
- * @apiParam {String} note                 Note
+ * @apiParam {String} org                  Main organization
+ * @apiParam {String[]} orgs               Additional organizations
+ * @apiParam {String[]} notes              Notes
  * @apiParam {String[]} allCategories      All available categories
  * @apiParam {Object[]} categories         Categories assigned to the card
  * @apiParam {String} categories.value     Category name
