@@ -1,14 +1,14 @@
 /*
- Copyright (C) 2004-2005 SKYRIX Software AG
+ Copyright (C) 2006-2016 Inverse inc.
+
+ This file is part of SOGo
  
- This file is part of OpenGroupware.org.
- 
- OGo is free software; you can redistribute it and/or modify it under
+ SOGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.
  
- OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+ SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.
@@ -19,8 +19,8 @@
  02111-1307, USA.
 */
 
-#ifndef __UIxContactsListView_H__
-#define __UIxContactsListView_H__
+#ifndef __UIxContactFolderActions_H__
+#define __UIxContactFolderActions_H__
 
 #import <SOGoUI/UIxComponent.h>
 
@@ -30,8 +30,6 @@
 @protocol SOGoContactObject;
 
 @interface UIxContactFolderActions : UIxComponent
-{
-}
 
 - (int) importLdifData: (NSString *) ldifData;
 - (int) importVcardData: (NSString *) vcardData;
@@ -39,4 +37,4 @@
 
 @end
 
-#endif /* __UIxContactsListView_H__ */
+#endif /* __UIxContactFolderActions_H__ */
