@@ -77,6 +77,7 @@ _xmlCharsetForCharset (NSString *charset)
     { @"koi8-r", XML_CHAR_ENCODING_ERROR},       // unsupported, will trigger koi8-r -> utf8 conversion
     { @"windows-1250", XML_CHAR_ENCODING_ERROR}, // unsupported, will trigger windows-1250 -> utf8 conversion
     { @"windows-1251", XML_CHAR_ENCODING_ERROR}, // unsupported, will trigger windows-1251 -> utf8 conversion
+    { @"windows-1255", XML_CHAR_ENCODING_ERROR}, // unsupported, will trigger windows-1255 -> utf8 conversion
     { @"windows-1257", XML_CHAR_ENCODING_ERROR}, // unsupported, will trigger windows-1257 -> utf8 conversion
     { @"gb2312", XML_CHAR_ENCODING_ERROR},       // unsupported, will trigger gb2312 -> utf8 conversion
     { @"gbk", XML_CHAR_ENCODING_ERROR},          // unsupported, will trigger gb2312 -> utf8 conversion
@@ -685,8 +686,7 @@ static NSData* _sanitizeContent(NSData *theData)
               
               if (!skipAttribute)
                 [resultPart appendFormat: @" %@=\"%@\"",
-                            name, [value stringByReplacingString: @"\""
-                                                      withString: @"\\\""]];
+                            name, [value safeStringByEscapingXMLString: NO]];
             }
           
           if ([VoidTags containsObject: lowerName])
