@@ -1,4 +1,5 @@
 /*
+  Copyright (C) 2015 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of OpenGroupware.org.
@@ -14,7 +15,7 @@
   License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with OGo; see the file COPYING.  If not, write to the
+  License along with SOGo; see the file COPYING.  If not, write to the
   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 */
@@ -44,6 +45,7 @@
 @class NSArray;
 @class NSData;
 @class NSFormatter;
+@class NSMutableDictionary;
 
 @class SOGoMailBodyPart;
 
@@ -52,6 +54,7 @@
   NSArray *partPath;
   id      bodyInfo;
   NSData  *flatContent;
+  NSDictionary *attachmentIds;
 }
 
 /* accessors */
@@ -63,6 +66,9 @@
 - (id)bodyInfo;
 
 - (SOGoMailBodyPart *) clientPart;
+- (id) renderedPart;
+
+- (void) setAttachmentIds: (NSDictionary *) newAttachmentIds;
 
 - (NSData *)flatContent;
 - (NSData *)decodedFlatContent;
