@@ -654,6 +654,8 @@
 
   // Retrieve messages UIDs using form parameters "sort" and "asc"
   uids = [self getSortedUIDsInFolder: folder];
+  if (uids == nil)
+    return nil;
   
   // Get rid of the extra parenthesis
    // uids = [[[[uids stringValue] stringByReplacingOccurrencesOfString:@"(" withString:@""] stringByReplacingOccurrencesOfString:@")" withString:@""] componentsSeparatedByString:@","];
@@ -670,7 +672,6 @@
       r = NSMakeRange(0, count);
       headers = [self getHeadersForUIDs: [a subarrayWithRange: r]
                                inFolder: folder];
-
       [data setObject: headers forKey: @"headers"];
     }
 
@@ -764,7 +765,19 @@
   data = [self getUIDsInFolder: folder
                    withHeaders: !noHeaders];
 
-  response = [self responseWithStatus: 200 andJSONRepresentation: data];
+  if (data != nil)
+    response = [self responseWithStatus: 200 andJSONRepresentation: data];
+  else if ([folder isSpecialFolder])
+    {
+      response = [self responseWithStatus: 204];
+    }
+  else
+    {
+      data = [NSDictionary dictionaryWithObjectsAndKeys:
+                             @"An error occured while communicating with the mail server", @"message", nil];
+      response = [self responseWithStatus: 500 /* Error */
+                    andJSONRepresentation: data];
+    }
 
   return response;
 }
@@ -905,8 +918,16 @@
   uids = [data objectForKey: @"uids"];
   headers = [self getHeadersForUIDs: uids
 			   inFolder: [self clientObject]];
-  response = [self responseWithStatus: 200
-                andJSONRepresentation: headers];
+  if (headers)
+    response = [self responseWithStatus: 200
+                  andJSONRepresentation: headers];
+  else
+    {
+      data = [NSDictionary dictionaryWithObjectsAndKeys:
+                             @"An error occured while communicating with the mail server", @"message", nil];
+      response = [self responseWithStatus: 500 /* Error */
+                    andJSONRepresentation: data];
+    }
 
   return response;
 }
