@@ -19,22 +19,15 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSEnumerator.h>
 #import <Foundation/NSValue.h>
 
-#import <EOControl/EOQualifier.h>
 
-#import <NGCards/NGVCard.h>
 #import <NGCards/NGVCardReference.h>
 #import <NGCards/NGVList.h>
-#import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
-#import <NGObjWeb/SoComponent.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
 
-#import <Contacts/SOGoContactObject.h>
 #import <Contacts/SOGoContactGCSList.h>
 #import <Contacts/SOGoContactGCSEntry.h>
 #import <Contacts/SOGoContactFolders.h>
@@ -43,9 +36,7 @@
 #import <Mailer/SOGoDraftsFolder.h>
 #import <Mailer/SOGoMailAccount.h>
 #import <Mailer/SOGoMailAccounts.h>
-#import <Mailer/SOGoMailFolder.h>
 #import <Mailer/SOGoMailLabel.h>
-#import <Mailer/SOGoMailObject.h>
 #import <Mailer/SOGoSentFolder.h>
 #import <SOGo/NSDictionary+URL.h>
 #import <SOGo/NSArray+Utilities.h>
@@ -53,13 +44,10 @@
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoUser.h>
-#import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoUserSettings.h>
-#import <SOGoUI/UIxComponent.h>
 
 #import "UIxMailMainFrame.h"
-#import "UIxMailListActions.h"
 
 // Avoid compilation warnings
 @interface SOGoUserFolder (private)
@@ -87,6 +75,11 @@
   [super dealloc];
 }
 
+- (NSString *) modulePath
+{
+  return @"Mail";
+}
+
 - (void) _setupContext
 {
   SOGoUser *activeUser;
@@ -111,12 +104,23 @@
 
 - (NSString *) mailAccounts
 {
-  NSArray *accounts, *names;
+  NSArray *accounts;
 
   accounts = [[self clientObject] mailAccounts];
-  names = [accounts objectsForKey: @"name" notFoundMarker: nil];
 
-  return [names jsonRepresentation];
+  return [accounts jsonRepresentation];
+}
+
+- (WOResponse *) mailAccountsAction
+{
+  WOResponse *response;
+  NSString *s;
+
+  s = [self mailAccounts];
+  response = [self responseWithStatus: 200
+                            andString: s];
+
+  return response;
 }
 
 - (NSString *) userNames
@@ -127,7 +131,6 @@
   userNames = [accounts objectsForKey: @"userName" notFoundMarker: nil];
   
   return [userNames jsonRepresentation];
-  
 }
 
 - (NSString *) pageFormURL
@@ -175,28 +178,6 @@
   return [u hasSuffix:@"/"] ? @"view" : @"#";
 }
 
-- (NSString *) inboxData
-{
-  SOGoMailAccounts *accounts;
-  SOGoMailAccount *account;
-  SOGoMailFolder *inbox;
-  NSDictionary *data;
-  UIxMailListActions *actions;
-
-  [self _setupContext];
-  
-#warning this code is dirty: we should not invoke UIxMailListActions from here!
-  actions = [[[UIxMailListActions new] initWithRequest: [context request]] autorelease];
-  accounts = [self clientObject];
-  
-  account = [accounts lookupName: @"0" inContext: context acquire: NO];
-  inbox = [account inboxFolderInContext: context];
-
-  data = [actions getUIDsInFolder: inbox withHeaders: YES];
-
-  return [data jsonRepresentation];
-}
-
 - (id <WOActionResults>) composeAction
 {
   id contact;
@@ -335,12 +316,17 @@
 
 - (WOResponse *) getFoldersStateAction
 {
-  NSString *expandedFolders;
+  id o;
+  NSArray *expandedFolders;
 
   [self _setupContext];
-  expandedFolders = [moduleSettings objectForKey: @"ExpandedFolders"];
+  o = [moduleSettings objectForKey: @"ExpandedFolders"];
+  if ([o isKindOfClass: [NSString class]])
+    expandedFolders = [o componentsSeparatedByString: @","];
+  else
+    expandedFolders = o;
 
-  return [self responseWithStatus: 200 andString: expandedFolders];
+  return [self responseWithStatus: 200 andJSONRepresentation: expandedFolders];
 }
 
 - (NSString *) verticalDragHandleStyle
@@ -398,14 +384,14 @@
 - (WOResponse *) saveFoldersStateAction
 {
   WORequest *request;
-  NSString *expandedFolders;
+  NSArray *expandedFolders;
   
   [self _setupContext];
   request = [context request];
-  expandedFolders = [request formValueForKey: @"expandedFolders"];
+  expandedFolders = [[request contentAsString] objectFromJSONString];
 
   [moduleSettings setObject: expandedFolders
-		  forKey: @"ExpandedFolders"];
+                     forKey: @"ExpandedFolders"];
 
   [us synchronize];
 
@@ -627,7 +613,7 @@
 
 - (NSString *) columnsDisplayCount
 {
-  return [NSString stringWithFormat: @"%d", [[self columnsDisplayOrder] count]];
+  return [NSString stringWithFormat: @"%d", (int)[[self columnsDisplayOrder] count]];
 }
 
 - (void) setCurrentColumn: (NSDictionary *) newCurrentColumn
@@ -645,6 +631,50 @@
   return [self labelForKey: [currentColumn objectForKey: @"value"]];
 }
 
+- (unsigned int) _unseenCountForFolder: (NSString *) theFolder
+{
+  NSArray *pathComponents;
+  SOGoMailAccount *account;
+  SOGoMailFolder *folder;
+
+  pathComponents = [theFolder pathComponents];
+  account = [[self clientObject] lookupName: [pathComponents objectAtIndex: 0]
+                                  inContext: context
+                                    acquire: YES];
+  folder = [account lookupNameByPaths: [pathComponents subarrayWithRange: NSMakeRange(1, [pathComponents count]-1)]
+                            inContext: context
+                              acquire: YES];
+
+  return [folder unseenCount];
+}
+
+- (WOResponse *) unseenCountAction
+{
+  NSMutableDictionary *data;
+  WOResponse *response;
+  NSArray *folders;
+  NSString *folder;
+  int i;
+
+  folders = [[[[context request] contentAsString] objectFromJSONString] objectForKey: @"mailboxes"];
+  data = [NSMutableDictionary dictionary];
+
+  for (i = 0; i < [folders count]; i++)
+    {
+      folder = [folders objectAtIndex: i];
+      [data setObject: [NSNumber numberWithUnsignedInt: [self _unseenCountForFolder: folder]]
+               forKey: folder];
+    }
+
+  response = [self responseWithStatus: 200];
+
+  [response setHeader: @"text/plain; charset=utf-8"
+	    forKey: @"content-type"];
+  [response appendContentString: [data jsonRepresentation]];
+
+  return response;
+}
+
 - (NSString *) unseenCountFolders
 {
   NSArray *pathComponents, *filters, *actions;
@@ -688,13 +718,13 @@
 
 		  for (k = 0; k < [pathComponents count]; k++)
 		    {
-                      component = [NSString stringWithFormat: @"folder%@", [pathComponents objectAtIndex: k]];
-                      [path appendString: [component asCSSIdentifier]];
+                      component = [[pathComponents objectAtIndex: k] asCSSIdentifier];
+                      [path appendString: [NSString stringWithFormat: @"folder%@", component]];
 		      if (k < [pathComponents count] - 1)
 			[path appendString: @"/"];
 		    }
 		  
-		  [folders addObject: [NSString stringWithFormat: @"/0/%@", path]];
+		  [folders addObject: [NSString stringWithFormat: @"0/%@", path]];
 		}
 	    }
 	}
@@ -732,3 +762,15 @@
 }
 
 @end /* UIxMailMainFrame */
+
+@interface UIxMailFolderTemplate : UIxComponent
+@end
+
+@implementation UIxMailFolderTemplate
+@end
+
+@interface UIxMailViewTemplate : UIxComponent
+@end
+
+@implementation UIxMailViewTemplate
+@end
