@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2005-2013 Inverse inc.
+  Copyright (C) 2005-2015 Inverse inc.
 
   This file is part of SOGo.
 
@@ -15,16 +14,15 @@
   License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with OGo; see the file COPYING.  If not, write to the
+  License along with SOGo; see the file COPYING.  If not, write to the
   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 */
 
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSException.h>
+#import <Foundation/NSValue.h>
+
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSException+misc.h>
 #import <NGExtensions/NGHashMap.h>
 #import <NGExtensions/NSString+misc.h>
@@ -36,35 +34,37 @@
 #import <NGMail/NGMimeMessage.h>
 #import <NGMail/NGMimeMessageGenerator.h>
 
-#import <NGImap4/NGImap4Client.h>
-#import <NGImap4/NGImap4Connection.h>
-#import <NGImap4/NGImap4Envelope.h>
 #import <NGImap4/NGImap4EnvelopeAddress.h>
 
+#import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoBuild.h>
 #import <SOGo/SOGoMailer.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
-#import <SOGo/SOGoUserManager.h>
-#import <SOGoUI/UIxComponent.h>
 #import <Mailer/SOGoMailObject.h>
 #import <Mailer/SOGoMailAccount.h>
 #import <Mailer/SOGoMailFolder.h>
 #import <MailPartViewers/UIxMailRenderingContext.h> // cyclic
 #import <MailPartViewers/UIxMailSizeFormatter.h>
+#import <MailPartViewers/UIxMailPartViewer.h>
 
 #import "WOContext+UIxMailer.h"
+#import "UIxMailFormatter.h"
 
 @interface UIxMailView : UIxComponent
 {
   id currentAddress;
-  NSString *shouldAskReceipt;
+  NSNumber *shouldAskReceipt;
   NSString *matchingIdentityEMail;
   NSDictionary *attachment;
   NSArray *attachmentAttrs;
 }
 
+- (BOOL) mailIsDraft;
+- (NSNumber *) shouldAskReceipt;
+- (NSString *) formattedDate;
+
 @end
 
 @implementation UIxMailView
@@ -183,6 +183,15 @@ static NSString *mailETag = nil;
   return [UIxMailSizeFormatter sharedMailSizeFormatter];
 }
 
+- (NSString *) formattedDate
+{
+  NSFormatter *formatter;
+
+  formatter = [[self context] mailDateFormatter];
+
+  return [formatter stringForObjectValue: [[self clientObject] date]];
+}
+
 - (NSString *) attachmentsText
 {
   if ([[self attachmentAttrs] count] > 1)
@@ -197,19 +206,53 @@ static NSString *mailETag = nil;
 {
   // TODO: I would prefer to flatten the body structure prior rendering,
   //       using some delegate to decide which parts to select for alternative.
-  id info;
+  id info, viewer;
   
   info = [[self clientObject] bodyStructure];
 
-  return [[context mailRenderingContext] viewerForBodyInfo:info];
+  viewer = [[context mailRenderingContext] viewerForBodyInfo: info];
+  [viewer setBodyInfo: info];
+
+  NSMutableDictionary *attachmentIds;
+  NSDictionary *attributes;
+  NSString *filename;
+  unsigned int count, max;
+
+  max = [[self attachmentAttrs] count];
+  attachmentIds = [NSMutableDictionary dictionaryWithCapacity: max];
+  for (count = 0; count < max; count++)
+    {
+      attributes = [[self attachmentAttrs] objectAtIndex: count];
+      filename = [NSString stringWithFormat: @"<%@>", [attributes objectForKey: @"filename"]];
+      [attachmentIds setObject: [attributes objectForKey: @"url"]
+                        forKey: filename];
+      if ([[attributes objectForKey: @"bodyId"] length])
+        [attachmentIds setObject: [attributes objectForKey: @"url"]
+                          forKey: [attributes objectForKey: @"bodyId"]];
+    }
+  [viewer setAttachmentIds: attachmentIds];
+
+  return viewer;
 }
 
 /* actions */
 
-- (id) defaultAction
+- (id <WOActionResults>) defaultAction
 {
   WOResponse *response;
   NSString *s;
+  NSMutableDictionary *data;
+  NSArray *addresses;
+  SOGoMailObject *co;
+  UIxEnvelopeAddressFormatter *addressFormatter;
+  UIxMailRenderingContext *mctx;
+
+  co = [self clientObject];
+  addressFormatter = [context mailEnvelopeAddressFormatter];
+
+  mctx = [[UIxMailRenderingContext alloc] initWithViewer: self context: context];
+  [context pushMailRenderingContext: mctx];
+  [mctx release];
 
   /* check etag to see whether we really must rerender */
   /*
@@ -217,32 +260,87 @@ static NSString *mailETag = nil;
     those are the IMAP4 flags (and annotations, which we do not use).
     Since we don't render the flags, it should be OK, if this changes
     we must embed the flagging into the etag.
+
+    2015-12-09: We disable caching for now. Let's do this right soon
+    by taking into account IMAP flags and the Accepted/Declined/etc.
+    state of an even with an IMIP invitation. We should perhaps even
+    store the state as an IMAP flag.
   */
   s = [[context request] headerForKey: @"if-none-match"];
-  if (s)
+  //if (s)
+  if (0)
     {
       if ([s rangeOfString:mailETag].length > 0) /* not perfectly correct */
         { 
           /* client already has the proper entity */
           // [self logWithFormat:@"MATCH: %@ (tag %@)", s, mailETag];
 	  
-          if (![[self clientObject] doesMailExist]) {
-            return [NSException exceptionWithHTTPStatus:404 /* Not Found */
-                                                 reason:@"message got deleted"];
-          }
+          if (![co doesMailExist])
+            {
+              data = [NSDictionary dictionaryWithObject: [self labelForKey: @"Message got deleted"]
+                                                 forKey: @"message"];
+              return [self responseWithStatus: 404 /* Not Found */
+                        andJSONRepresentation: data];
+            }
           
-          response = [context response];
-          [response setStatus: 304 /* Not Modified */];
+          response = [self responseWithStatus: 304];
 
           return response;
         }
     }
   
   if (![self message]) // TODO: redirect to proper error
-    return [NSException exceptionWithHTTPStatus:404 /* Not Found */
-			reason:@"did not find specified message!"];
+    {
+      data = [NSDictionary dictionaryWithObject: [self labelForKey: @"Did not find specified message"]
+                                         forKey: @"message"];
+      return [self responseWithStatus: 404 /* Not Found */
+                            andJSONRepresentation: data];
+    }
 
-  return self;
+  data = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                       [self attachmentAttrs], @"attachmentAttrs",
+                       [self shouldAskReceipt], @"shouldAskReceipt",
+                       [NSNumber numberWithBool: [self mailIsDraft]], @"isDraft",
+                       [[self contentViewerComponent] renderedPart], @"parts",
+                       nil];
+  if ([self formattedDate])
+    [data setObject: [self formattedDate] forKey: @"date"];
+  if ([self messageSubject])
+    [data setObject: [self messageSubject] forKey: @"subject"];
+  if ((addresses = [addressFormatter dictionariesForArray: [co fromEnvelopeAddresses]]))
+    [data setObject: addresses forKey: @"from"];
+  if ((addresses = [addressFormatter dictionariesForArray: [co toEnvelopeAddresses]]))
+    [data setObject: addresses forKey: @"to"];
+  if ((addresses = [addressFormatter dictionariesForArray: [co ccEnvelopeAddresses]]))
+    [data setObject: addresses forKey: @"cc"];
+  if ((addresses = [addressFormatter dictionariesForArray: [co bccEnvelopeAddresses]]))
+    [data setObject: addresses forKey: @"bcc"];
+  if ((addresses = [addressFormatter dictionariesForArray: [co replyToEnvelopeAddresses]]))
+    [data setObject: addresses forKey: @"reply-to"];
+
+  // Mark message as read
+  [co addFlags: @"seen"];
+
+  response = [self responseWithStatus: 200
+                andJSONRepresentation: data];
+
+  [response setHeader: mailETag forKey: @"etag"];
+
+  [[context popMailRenderingContext] reset];
+
+  return response;
+}
+
+- (id <WOActionResults>) archiveAttachmentsAction
+{
+  NSString *name;
+  SOGoMailObject *co;
+
+  co = [self clientObject];
+  name = [NSString stringWithFormat: @"%@-%@.zip",
+                  [self labelForKey: @"attachments"], [co nameInContainer]];
+
+  return [co archiveAllFilesinArchiveNamed: name];
 }
 
 /* MDN */
@@ -549,7 +647,7 @@ static NSString *mailETag = nil;
     [self _flagMessageWithMDNSent];
 }
 
-- (NSString *) shouldAskReceipt
+- (NSNumber *) shouldAskReceipt
 {
   NGMailAddress *mailAddress;
   NSDictionary *mailHeaders;
@@ -557,7 +655,7 @@ static NSString *mailETag = nil;
 
   if (!shouldAskReceipt)
     {
-      shouldAskReceipt = @"false";
+      shouldAskReceipt = [NSNumber numberWithBool: NO];
       mailHeaders = [[self clientObject] mailHeaders];
       email = [mailHeaders objectForKey: @"disposition-notification-to"];
       if (!email)
@@ -584,7 +682,7 @@ static NSString *mailETag = nil;
               action = [self _receiptAction];
               if ([action isEqualToString: @"ask"])
                 {
-                  shouldAskReceipt = @"true";
+                  shouldAskReceipt = [NSNumber numberWithBool: YES];
                   [self _flagMessageWithMDNSent];
                 }
               else if ([action isEqualToString: @"send"])
@@ -599,7 +697,7 @@ static NSString *mailETag = nil;
 - (WOResponse *) sendMDNAction
 {
   WOResponse *response;
-  NSDictionary *mailHeaders;
+  NSDictionary *mailHeaders, *jsonResponse;
   NSString *email, *action;
 
   mailHeaders = [[self clientObject] mailHeaders];
@@ -616,9 +714,11 @@ static NSString *mailETag = nil;
   if (email)
     {
       if ([self _userHasEMail: email])
-        response = [self responseWithStatus: 403
-                                  andString: (@"One cannot send an MDN to"
-                                              @" oneself.")];
+        {
+          jsonResponse = [NSDictionary dictionaryWithObject: [self labelForKey: @"One cannot send an MDN to oneself."]
+                                                     forKey: @"message"];
+          response = [self responseWithStatus: 403 andJSONRepresentation: jsonResponse];
+        }
       else
         {
           action = [self _receiptAction];
@@ -628,16 +728,20 @@ static NSString *mailETag = nil;
               response = [self responseWithStatus: 204];
             }
           else
-            response = [self responseWithStatus: 403
-                                      andString: (@"No notification header found in"
-                                                  @" original message.")];
+            {
+              jsonResponse = [NSDictionary dictionaryWithObject: [self labelForKey: @"No notification header found in original message."]
+                                                         forKey: @"message"];
+              response = [self responseWithStatus: 403 andJSONRepresentation: jsonResponse];
+            }
         }
     }
   else
-    response = [self responseWithStatus: 403
-                              andString: (@"No notification header found in"
-                                          @" original message.")];
-  
+    {
+      jsonResponse = [NSDictionary dictionaryWithObject: [self labelForKey: @"No notification header found in original message."]
+                                                 forKey: @"message"];
+      response = [self responseWithStatus: 403 andJSONRepresentation: jsonResponse];
+    }
+
   return response;
 }
 
