@@ -72,6 +72,11 @@
 
 /* accessors */
 
+- (NSString *) modulePath
+{
+  return @"";
+}
+
 //- (NSString *) connectURL
 //{
 //  return [NSString stringWithFormat: @"%@/connect", [self applicationPath]];
@@ -525,6 +530,11 @@
   return item;
 }
 
+- (NSString *) language
+{
+  return [[context resourceLookupLanguages] objectAtIndex: 0];
+}
+
 - (NSArray *) languages
 {
   return [[SOGoSystemDefaults sharedSystemDefaults] supportedLanguages];
