@@ -1,6 +1,6 @@
 /* UIxJSONPreferences.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2016 Inverse inc.
+ * Copyright (C) 2007-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoUserProfile.h>
 #import <SOGo/WOResourceManager+SOGo.h>
+#import <SOGoUI/UIxComponent.h>
 #import <Mailer/SOGoMailLabel.h>
 
 #import "UIxJSONPreferences.h"
@@ -71,13 +72,18 @@ static SoProduct *preferencesProduct = nil;
 }
 
 - (WOResponse *) jsonDefaultsAction
+{
+  return [self responseWithStatus: 200
+                        andString: [self jsonDefaults]];
+}
+
+- (NSString *) jsonDefaults
 {
   NSMutableDictionary *values, *account, *vacation;
   SOGoUserDefaults *defaults;
   SOGoDomainDefaults *domainDefaults;
   NSMutableArray *accounts;
   NSDictionary *categoryLabels, *vacationOptions;
-  NSDictionary *locale;
 
   if (!preferencesProduct)
     {
@@ -126,6 +132,9 @@ static SoProduct *preferencesProduct = nil;
       [[defaults source] removeObjectForKey: @"SOGoAlternateAvatar"];
     }
 
+  if (![[defaults source] objectForKey: @"SOGoAnimationMode"])
+    [[defaults source] setObject: [defaults animationMode]  forKey: @"SOGoAnimationMode"];
+
   //
   // Default Calendar preferences
   //
@@ -259,6 +268,9 @@ static SoProduct *preferencesProduct = nil;
       [defaults setContactsCategories: categoryLabels];
     }
 
+  if (![[defaults source] objectForKey: @"SOGoMailAddOutgoingAddresses"])
+    [[defaults source] setObject: [NSNumber numberWithBool: [defaults mailAddOutgoingAddresses]] forKey: @"SOGoMailAddOutgoingAddresses"];
+
   //
   // Default Mail preferences
   //
@@ -283,6 +295,9 @@ static SoProduct *preferencesProduct = nil;
   if (![[defaults source] objectForKey: @"SOGoMailDisplayRemoteInlineImages"])
     [[defaults source] setObject: [defaults mailDisplayRemoteInlineImages] forKey: @"SOGoMailDisplayRemoteInlineImages"];
 
+  if (![[defaults source] objectForKey: @"SOGoMailAutoSave"])
+    [[defaults source] setObject: [defaults mailAutoSave] forKey: @"SOGoMailAutoSave"];
+
   // Populate default mail labels, based on the user's preferred language
   if (![[defaults source] objectForKey: @"SOGoMailLabelsColors"])
     {
@@ -320,7 +335,6 @@ static SoProduct *preferencesProduct = nil;
              forKey: @"SOGoAppointmentSendEMailNotifications"];
 
   // Add locale code (used by CK Editor)
-  locale = [[preferencesProduct resourceManager] localeForLanguageNamed: [defaults language]];
   [values setObject: [locale objectForKey: @"NSLocaleCode"] forKey: @"LocaleCode"];
   [values setObject: [NSDictionary dictionaryWithObjectsAndKeys:
                                            [locale objectForKey: @"NSMonthNameArray"], @"months",
@@ -340,7 +354,8 @@ static SoProduct *preferencesProduct = nil;
                                         @"ignore", @"receiptAnyAction", nil]
                   forKey:  @"receipts"];
     }
-  [accounts insertObject: account  atIndex: 0];
+  if (account)
+    [accounts insertObject: account  atIndex: 0];
   [values setObject: accounts  forKey: @"AuxiliaryMailAccounts"];
 
   // Add the domain's default vacation subject if user has not specified a custom subject
@@ -356,10 +371,16 @@ static SoProduct *preferencesProduct = nil;
       [values setObject: vacation forKey: @"Vacation"];
     }
 
-  return [self responseWithStatus: 200 andJSONRepresentation: values];
+  return [values jsonRepresentation];
 }
 
 - (WOResponse *) jsonSettingsAction
+{
+  return [self responseWithStatus: 200
+                        andString: [self jsonSettings]];
+}
+
+- (NSString *) jsonSettings
 {
   SOGoUserSettings *settings;
   id v;
@@ -387,7 +408,7 @@ static SoProduct *preferencesProduct = nil;
   if (![settings objectForKey: @"Mail"])
     [settings setObject: [NSMutableDictionary dictionary]  forKey: @"Mail"];
 
-  return [self responseWithStatus: 200 andJSONRepresentation: [[settings source] values]];
+  return [[[settings source] values] jsonRepresentation];
 }
 
 @end
