@@ -1,6 +1,6 @@
 /* UIxPreferences.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 
 @class NSString;
 
-@class SOGoMailLabel;
 @class SOGoUser;
 
 @interface UIxPreferences : UIxComponent
@@ -44,27 +43,21 @@
   NSDictionary *calendarCategoriesColors;
   
   NSArray *contactsCategories;
-  NSString *defaultCategoryColor;
   NSCalendarDate *today;
 
-  // Mail labels/tags
-  SOGoMailLabel *label;
-  NSArray *mailLabels;
-  
   // Sieve filtering
   NSArray *daysOfWeek, *daysBetweenResponsesList;
   NSArray *sieveFilters;
   NSMutableDictionary *vacationOptions, *forwardOptions;
 
   BOOL mailCustomFromEnabled;
+  BOOL forwardEnabled;
   BOOL hasChanged;
-
-  
 }
 
-- (NSString *) userLongDateFormat;
-- (BOOL) isSieveServerAvailable;
-- (id) sieveClient;
+- (BOOL) _isSieveServerAvailable;
+- (id) _sieveClient;
+- (NSString *) _vacationTextForTemplate: (NSString *) templateFilePath;
 
 @end
 
