@@ -267,6 +267,12 @@
 @implementation SOGoFolderGermanRemovalAdvisory
 @end
 
+@implementation SOGoFolderHebrewAdditionAdvisory
+@end
+
+@implementation SOGoFolderHebrewRemovalAdvisory
+@end
+
 @implementation SOGoFolderHungarianAdditionAdvisory
 @end
 
@@ -285,6 +291,18 @@
 @implementation SOGoFolderItalianRemovalAdvisory
 @end
 
+@implementation SOGoFolderLatvianAdditionAdvisory
+@end
+
+@implementation SOGoFolderLatvianRemovalAdvisory
+@end
+
+@implementation SOGoFolderLithuanianAdditionAdvisory
+@end
+
+@implementation SOGoFolderLithuanianRemovalAdvisory
+@end
+
 @implementation SOGoFolderNorwegianBokmalAdditionAdvisory
 @end
 
