@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2007-2012 Inverse inc.
-  Copyright (C) 2004 SKYRIX Software AG
+  Copyright (C) 2007-2016 Inverse inc.
 
   This file is part of SOGo
 
@@ -20,18 +19,11 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSException.h>
-#import <Foundation/NSEnumerator.h>
 #import <Foundation/NSKeyValueCoding.h>
-#import <Foundation/NSPathUtilities.h>
 #import <Foundation/NSUserDefaults.h> /* for locale strings */
-#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#import <NGObjWeb/SoHTTPAuthenticator.h>
 #import <NGObjWeb/SoObjects.h>
-#import <NGObjWeb/SoProduct.h>
-#import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WOResponse.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGExtensions/NSObject+Logs.h>
@@ -39,21 +31,21 @@
 #import <NGExtensions/NSURL+misc.h>
 
 #import <SOGo/NSCalendarDate+SOGo.h>
+#import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSObject+Utilities.h>
+#import <SOGo/NSString+Crypto.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoBuild.h>
-#import <SOGo/SOGoContentObject.h>
-#import <SOGo/SOGoObject.h>
-#import <SOGo/SOGoPermissions.h>
+#import <SOGo/SOGoSession.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserFolder.h>
-#import <SOGo/SOGoUserDefaults.h>
+#import <SOGo/SOGoWebAuthenticator.h>
+#import <SOGo/WOContext+SOGo.h>
 #import <SOGo/WOResourceManager+SOGo.h>
 
 #import "UIxJSClose.h"
 
-#import "UIxComponent.h"
 
 @interface UIxComponent (PrivateAPI)
 - (void)_parseQueryString:(NSString *)_s;
@@ -62,6 +54,7 @@
 
 @implementation UIxComponent
 
+static NSMutableArray *amPmLabelKeys      = nil;
 static NSMutableArray *dayLabelKeys       = nil;
 static NSMutableArray *abbrDayLabelKeys   = nil;
 static NSMutableArray *monthLabelKeys     = nil;
@@ -70,6 +63,10 @@ static SoProduct      *commonProduct      = nil;
 
 + (void)initialize {
   if (dayLabelKeys == nil) {
+    amPmLabelKeys = [[NSMutableArray alloc] initWithCapacity:2];
+    [amPmLabelKeys addObject:@"AM"];
+    [amPmLabelKeys addObject:@"PM"];
+
     dayLabelKeys = [[NSMutableArray alloc] initWithCapacity:7];
     [dayLabelKeys addObject:@"Sunday"];
     [dayLabelKeys addObject:@"Monday"];
@@ -122,6 +119,11 @@ static SoProduct      *commonProduct      = nil;
   }
 }
 
++ (NSArray *) amPmLabelKeys
+{
+  return (NSArray *) amPmLabelKeys;
+}
+
 + (NSArray *) abbrDayLabelKeys
 {
   return (NSArray *) abbrDayLabelKeys;
@@ -149,6 +151,7 @@ static SoProduct      *commonProduct      = nil;
       if (!userDefaults)
         ASSIGN (userDefaults, [SOGoSystemDefaults sharedSystemDefaults]);
       language = [userDefaults language];
+      ASSIGN (languages, [context resourceLookupLanguages]);
       ASSIGN (locale,
               [[self resourceManager] localeForLanguageNamed: language]);
     }
@@ -357,40 +360,22 @@ static SoProduct      *commonProduct      = nil;
 
 - (NSString *) modulePath
 {
-  SOGoObject *currentClient, *parent;
-  BOOL found;
-  NSString *hostLessURL;
-  Class objectClass, userFolderClass;
-// , groupFolderClass
-
-  currentClient = [self clientObject];
-  if (currentClient
-      && [currentClient isKindOfClass: [SOGoObject class]])
+  if ([[self parent] respondsToSelector: @selector(modulePath)])
     {
-//       groupFolderClass = [SOGoCustomGroupFolder class];
-      userFolderClass = [SOGoUserFolder class];
+      NSString *baseURL;
 
-      objectClass = [currentClient class];
-//       found = (objectClass == groupFolderClass || objectClass == userFolderClass);
-      found = (objectClass == userFolderClass);
-      while (!found && currentClient)
-	{
-	  parent = [currentClient container];
-	  objectClass = [parent class];
-	  if (// objectClass == groupFolderClass
-// 	      || 
-	      objectClass == userFolderClass)
-	    found = YES;
-	  else
-	    currentClient = parent;
-	}
+      baseURL = [[self clientObject] baseURLInContext: context];
+
+      if (!baseURL)
+        baseURL = @"/SOGo/so/";
+
+      if ([baseURL hasSuffix: [NSString stringWithFormat: @"%@/", [[self parent] modulePath]]])
+        return baseURL;
+
+      return [NSString stringWithFormat: @"%@%@", baseURL, [[self parent] modulePath]];
     }
-  else
-    currentClient = [WOApplication application];
-  
-  hostLessURL = [[currentClient baseURLInContext: context] hostlessURL];
 
-  return [hostLessURL substringToIndex: [hostLessURL length] - 1];
+  return @"SOGo";
 }
 
 - (NSString *) ownPath
@@ -411,11 +396,15 @@ static SoProduct      *commonProduct      = nil;
 
 - (NSString *) relativePathToUserFolderSubPath: (NSString *) _sub
 {
-  NSString *dst, *rel;
+  NSString *dst, *rel, *theme;
 
   dst = [[self userFolderPath] stringByAppendingPathComponent: _sub];
   rel = [dst urlPathRelativeToPath:[self ownPath]];
 
+  theme = [[context request] formValueForKey: @"theme"];
+  if ([theme length])
+    rel = [NSString stringWithFormat: @"%@?theme=%@", rel, theme];
+
   return rel;
 }
 
@@ -434,6 +423,20 @@ static SoProduct      *commonProduct      = nil;
   return _selectedDate;
 }
 
+- (NSString *) currentDayDescription
+{
+  NSDictionary *currentDay;
+  SOGoUser *user;
+
+  user = [context activeUser];
+  if (user)
+    currentDay = [user currentDay];
+  else
+    currentDay = [NSDictionary dictionary];
+
+  return [currentDay jsonRepresentation];
+}
+
 - (NSString *) dateStringForDate: (NSCalendarDate *) _date
 {
   [_date setTimeZone: [userDefaults timeZone]];
@@ -452,7 +455,7 @@ static SoProduct      *commonProduct      = nil;
 
   jsClose = [UIxJSClose new];
   [jsClose autorelease];
-  [jsClose setRefreshMethod: methodName];
+  [jsClose setRefreshMethod: [methodName doubleQuotedString]];
 
   return jsClose;
 }
@@ -512,6 +515,22 @@ static SoProduct      *commonProduct      = nil;
   return [[context activeUser] login];
 }
 
+/* Common defaults and settings */
+
+- (int) minimumSearchLength
+{
+  return [[[context activeUser] domainDefaults] searchMinimumWordLength];
+}
+
+- (NSString *) minimumSearchLengthLabel
+{
+  NSDictionary *defaults;
+
+  defaults = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: [self minimumSearchLength]]
+                                         forKey: @"minimumSearchLength"];
+  return [defaults keysWithFormat: [self commonLabelForKey: @"Enter at least %{minimumSearchLength} characters"]];
+}
+
 /* labels */
 
 - (NSString *) labelForKey: (NSString *) _str
@@ -536,7 +555,6 @@ static SoProduct      *commonProduct      = nil;
 - (NSString *) labelForKey: (NSString *) _str
        withResourceManager: (WOResourceManager *) rm
 {
-  NSArray *languages;
   NSString *lKey, *lTable, *lVal;
   NSRange r;
 
@@ -546,10 +564,6 @@ static SoProduct      *commonProduct      = nil;
   if (rm == nil)
     [self warnWithFormat:@"missing resource manager!"];
 
-  /* lookup languages */
-
-  languages = [context resourceLookupLanguages];
-
   /* get parameters */
     
   r = [_str rangeOfString:@"/"];
@@ -710,9 +724,27 @@ static SoProduct      *commonProduct      = nil;
 - (WOResponse *) redirectToLocation: (NSString *) newLocation
 {
   WOResponse *response;
+  NSURL *url;
+  NSMutableString *location;
+  NSString *theme, *query;
+
+  location = [NSMutableString stringWithString: newLocation];
+  theme = [[context request] formValueForKey: @"theme"];
+  if ([theme length])
+    {
+      url = [NSURL URLWithString: newLocation];
+      query = [url query];
+      if ([query length])
+        {
+          if ([query rangeOfString: @"theme="].length == 0)
+            [location appendFormat: @"&theme=%@", theme];
+        }
+      else
+        [location appendFormat: @"?theme=%@", theme];
+    }
 
   response = [self responseWithStatus: 302];
-  [response setHeader: newLocation forKey: @"location"];
+  [response setHeader: location forKey: @"location"];
 
   return response;
 }
@@ -733,4 +765,44 @@ static SoProduct      *commonProduct      = nil;
   return [sd uixDebugEnabled];
 }
 
+//
+// Protection against XSRF
+//
+- (id<WOActionResults>)performActionNamed:(NSString *)_actionName
+{
+  SOGoWebAuthenticator *auth;
+  NSString *value, *token;
+  NSArray *creds;
+
+  auth = [[WOApplication application]
+           authenticatorInContext: context];
+
+  if (![[SOGoSystemDefaults sharedSystemDefaults] xsrfValidationEnabled] ||
+      ![auth isKindOfClass: [SOGoWebAuthenticator class]])
+    return [super performActionNamed: _actionName];
+
+  // If the action is 'connect' (or 'logoff'), we let it go as the token
+  // needs to be created (or destroyed) during the session initialization
+  if ([_actionName isEqualToString: @"connect"] ||
+      [_actionName isEqualToString: @"logoff"])
+    {
+      return [super performActionNamed: _actionName];
+    }
+
+  // We grab the X-XSRF-TOKEN header
+  token = [[context request] headerForKey: @"X-XSRF-TOKEN"];
+
+  // We compare it with our session key
+  value = [[context request]
+           cookieValueForKey: [auth cookieNameInContext: context]];
+  creds = [auth parseCredentials: value];
+
+  value = [SOGoSession valueForSessionKey: [creds lastObject]];
+
+  if ([token isEqualToString: [value asSHA1String]])
+    return [super performActionNamed: _actionName];
+
+  return nil;
+}
+
 @end /* UIxComponent */
