@@ -1,8 +1,6 @@
 /* UIxAppointmentActions.m - this file is part of SOGo
  *
- * Copyright (C) 2011-2014 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2011-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,10 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSArray.h>
+#import <Foundation/NSValue.h>
 
 #import <NGExtensions/NSObject+Values.h>
 
@@ -34,10 +29,10 @@
 #import <NGObjWeb/WORequest.h>
 
 #import <NGCards/iCalCalendar.h>
-#import <NGCards/iCalEvent.h>
 
 #import <SOGo/NSCalendarDate+SOGo.h>
 #import <SOGo/NSDictionary+Utilities.h>
+#import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
 #import <Appointments/iCalEvent+SOGo.h>
@@ -45,8 +40,6 @@
 #import <Appointments/SOGoAppointmentFolder.h>
 #import <Appointments/SOGoAppointmentFolders.h>
 
-#import <Common/WODirectAction+SOGo.h>
-
 #import "UIxAppointmentActions.h"
 
 @implementation UIxAppointmentActions
@@ -59,23 +52,28 @@
   SoSecurityManager *sm;
   iCalEvent *event;
   NSCalendarDate *start, *newStart, *end, *newEnd;
+  NSDictionary *params, *jsonResponse;
   NSTimeInterval newDuration;
   SOGoUserDefaults *ud;
-  NSString *daysDelta, *startDelta, *durationDelta, *destionationCalendar;
+  NSNumber *daysDelta, *startDelta, *durationDelta;
+  NSString *destionationCalendar;
   NSTimeZone *tz;
   NSException *ex;
   SOGoAppointmentFolder *targetCalendar, *sourceCalendar;
   SOGoAppointmentFolders *folders;
+  BOOL forceSave;
+  id error;
 
   rq = [context request];
+  params = [[rq contentAsString] objectFromJSONString];
 
-  daysDelta = [rq formValueForKey: @"days"];
-  startDelta = [rq formValueForKey: @"start"];
-  durationDelta = [rq formValueForKey: @"duration"];
-  destionationCalendar = [rq formValueForKey: @"destination"];
+  daysDelta = [params objectForKey: @"days"];
+  startDelta = [params objectForKey: @"start"];
+  durationDelta = [params objectForKey: @"duration"];
+  destionationCalendar = [params objectForKey: @"destination"];
+  forceSave = [[params objectForKey: @"ignoreConflicts"] boolValue];
 
-  if ([daysDelta length] > 0
-      || [startDelta length] > 0 || [durationDelta length] > 0)
+  if (daysDelta || startDelta || durationDelta)
     {
       co = [self clientObject];
       event = (iCalEvent *) [[self clientObject] occurence];
@@ -117,9 +115,10 @@
         [event updateRecurrenceRulesUntilDate: end];
 
       [event setLastModified: [NSCalendarDate calendarDate]];
-      ex = [co saveComponent: event];
+      ex = [co saveComponent: event  force: forceSave];
+
       // This condition will be executed only if the event is moved from a calendar to another. If destionationCalendar == 0; there is no calendar change
-      if (![destionationCalendar isEqualToString:@"0"])
+      if ([destionationCalendar length] > 0)
         {
           folders = [[self->context activeUser] calendarsFolderInContext: self->context];
           sourceCalendar = [co container];
@@ -135,44 +134,57 @@
                               inContext: context])
               ex = [co moveToFolder: targetCalendar];
           }
-
-    
         }
+
       if (ex)
         {
-          NSDictionary *jsonResponse;
+          unsigned int httpStatus;
+
+          httpStatus = 500;
+
+          if ([ex respondsToSelector: @selector(httpStatus)])
+            httpStatus = [ex httpStatus];
+
+          error = [[ex reason] objectFromJSONString];
+          if (error == nil)
+            error = [ex reason];
           jsonResponse = [NSDictionary dictionaryWithObjectsAndKeys:
-                                       [ex reason], @"message",
+                                         error, @"message",
                                        nil];
-          response = [self responseWithStatus: 403
-                                    andString: [jsonResponse jsonRepresentation]];
+
+          response = [self responseWithStatus: httpStatus
+                        andJSONRepresentation: jsonResponse];
         }
       else
         response = [self responseWith204];
     }
   else
-    response
-      = (WOResponse *) [NSException exceptionWithHTTPStatus: 400
-                                                     reason: @"missing 'days', 'start' and/or 'duration' parameters"];
+    {
+      jsonResponse = [NSDictionary dictionaryWithObjectsAndKeys:
+                                     @"missing 'days', 'start' and/or 'duration' parameters", @"message",
+                                   nil];
+      response = [self responseWithStatus: 400
+                    andJSONRepresentation: jsonResponse];
+    }
 
   return response;
 }
 
-- (WOResponse *) copyAction
+- (WOResponse *) _copyOrMoveAction: (BOOL) moving
 {
-  NSString *destination;
-  NSArray *events;
-  iCalCalendar *calendar;
-  iCalRepeatableEntityObject *masterOccurence;
-  SOGoAppointmentObject *thisEvent;
   SOGoAppointmentFolder *sourceCalendar, *destinationCalendar;
+  SOGoAppointmentObject *thisEvent;
+  iCalCalendar *calendar;
+  NSString *destination;
   SoSecurityManager *sm;
+  NSDictionary *params;
   WOResponse *response;
   WORequest *rq;
 
   rq = [context request];
+  params = [[rq contentAsString] objectFromJSONString];
+  destination = [params objectForKey: @"destination"];
 
-  destination = [rq formValueForKey: @"destination"];
   if (![destination length])
     destination = @"personal";
   
@@ -192,40 +204,50 @@
 	  response = [NSException exceptionWithHTTPStatus: 403
 						   reason: @"Can't add event to destination calendar."];
 	}
-      // Verify that the destination calendar is not the source calendar
-      else if ([[destinationCalendar nameInContainer] isEqualToString: [sourceCalendar nameInContainer]])
-	{
-	  response = [NSException exceptionWithHTTPStatus: 400
-						   reason: @"Destination calendar is the source calendar."];
-	}
       else
 	{
-	  // Remove attendees, recurrence exceptions and single occurences from the event
-	  calendar = [thisEvent calendar: NO secure: NO];
-	  events = [calendar events];
-	  masterOccurence = [events objectAtIndex: 0];
-	  
-	  if ([masterOccurence hasAlarms])
-	    [masterOccurence removeAllAlarms];
-	  if ([masterOccurence hasRecurrenceRules])
-	    {
-	      [masterOccurence removeAllExceptionRules];
-	      [masterOccurence removeAllExceptionDates];
-	    }
-	  if ([[masterOccurence attendees] count] > 0)
-	    {
-	      [masterOccurence setOrganizer: nil];
-	      [masterOccurence removeAllAttendees];
-	    }
-	  [calendar setUniqueChild: masterOccurence];
+          calendar = [thisEvent calendar: NO secure: NO];
 
-	  // Perform the copy
-	  if ([thisEvent copyComponent: calendar
-			      toFolder: (SOGoGCSFolder *) destinationCalendar])
-	    response = [NSException exceptionWithHTTPStatus: 500
-						     reason: @"Can't copy event to destination calendar."];
-	  else
-	    response = [self responseWith204];
+          if (moving)
+            {
+              // Perform the move
+              if ([thisEvent moveToFolder: (SOGoGCSFolder *) destinationCalendar])
+                response = [NSException exceptionWithHTTPStatus: 500
+                                                         reason: @"Can't move event to destination calendar."];
+              else
+                response = [self responseWith204];
+            }
+          else
+            {
+              iCalRepeatableEntityObject *masterOccurence;
+              NSArray *events;
+
+              // Remove attendees, recurrence exceptions and single occurences from the event
+              events = [calendar events];
+              masterOccurence = [events objectAtIndex: 0];
+
+              if ([masterOccurence hasAlarms])
+                [masterOccurence removeAllAlarms];
+              if ([masterOccurence hasRecurrenceRules])
+                {
+                  [masterOccurence removeAllExceptionRules];
+                  [masterOccurence removeAllExceptionDates];
+                }
+              if ([[masterOccurence attendees] count] > 0)
+                {
+                  [masterOccurence setOrganizer: nil];
+                  [masterOccurence removeAllAttendees];
+                }
+              [calendar setUniqueChild: masterOccurence];
+
+              // Perform the copy
+              if ([thisEvent copyComponent: calendar
+                                  toFolder: (SOGoGCSFolder *) destinationCalendar])
+                response = [NSException exceptionWithHTTPStatus: 500
+                                                         reason: @"Can't copy event to destination calendar."];
+              else
+                response = [self responseWith204];
+            }
 	}
     }
   else
@@ -237,4 +259,14 @@
   return response;
 }
 
+- (WOResponse *) copyAction
+{
+  return [self _copyOrMoveAction: NO];
+}
+
+- (WOResponse *) moveAction
+{
+  return [self _copyOrMoveAction: YES];
+}
+
 @end
