@@ -20,12 +20,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSKeyValueCoding.h>
 
-#import <NGObjWeb/WOContext.h>
 
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserSettings.h>
