@@ -46,6 +46,7 @@
 #import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSObject+Utilities.h>
 #import <SOGo/WOResourceManager+SOGo.h>
+#import <Appointments/iCalToDo+SOGo.h>
 #import <Appointments/SOGoAppointmentFolders.h>
 #import <Appointments/SOGoWebAppointmentFolder.h>
 
@@ -315,6 +316,7 @@ static NSArray *tasksFields = nil;
 {
   NSArray *mails, *states;
   NSMutableArray *statesDescription;
+  NSString *description;
   iCalPersonPartStat stat;
   NSUInteger count, max;
 
@@ -336,7 +338,9 @@ static NSArray *tasksFields = nil;
       for (count = 0; count < max; count++)
         {
           stat = (states ? [[states objectAtIndex: count] intValue] : 0);
-          [statesDescription addObject: [[iCalPerson descriptionForParticipationStatus: stat] lowercaseString]];
+          description = [iCalPerson descriptionForParticipationStatus: stat];
+          if (description == nil) description = @"";
+          [statesDescription addObject: [description lowercaseString]];
         }
 
       if (max > 0)
@@ -1590,6 +1594,8 @@ _computeBlocksPosition (NSArray *blocks)
       statusFlag = [self _getStatusClassForStatusCode: statusCode
                                       andEndDateStamp: endDateStamp];
       [filteredTask addObject: statusFlag];
+      statusFlag = [iCalToDo statusForCode: statusCode];
+      [filteredTask replaceObjectAtIndex: taskStatusIndex withObject: statusFlag];
       if (endDateStamp > 0)
         [filteredTask addObject: [self _formattedDateForSeconds: endDateStamp
                                                       forAllDay: NO]];
