@@ -1,6 +1,6 @@
 /* UIxCalMainView.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2015 Inverse inc.
+ * Copyright (C) 2006-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,31 +18,32 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSKeyValueCoding.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSTimeZone.h>
 #import <Foundation/NSValue.h>
 
-#import <NGObjWeb/SoSecurityManager.h>
 #import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
+
+#import <NGCards/iCalPerson.h>
 
 #import <SOGo/NSArray+Utilities.h>
-#import <SOGo/SOGoPermissions.h>
+#import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoUserSettings.h>
 
-#import <Appointments/SOGoAppointmentFolder.h>
 #import <Appointments/SOGoAppointmentFolders.h>
 
 #import "UIxCalMainView.h"
 
 @implementation UIxCalMainView
 
+- (NSString *) modulePath
+{
+  return @"Calendar";
+}
+
 - (void) checkDefaultModulePreference
 {
   SOGoUserDefaults *ud;
@@ -259,12 +260,14 @@
 - (WOResponse *) saveSelectedListAction
 {
   WORequest *request;
+  NSDictionary *params;
   NSString *selectedList;
   
   [self _setupContext];
   request = [context request];
+  params = [[request contentAsString] objectFromJSONString];
 
-  selectedList = [request formValueForKey: @"list"];
+  selectedList = [params objectForKey: @"list"];
   [moduleSettings setObject: selectedList
                      forKey: @"SelectedList"];
   [us synchronize];
@@ -275,12 +278,14 @@
 - (WOResponse *) saveListStateAction
 {
   WORequest *request;
+  NSDictionary *params;
   NSString *state;
 
   [self _setupContext];
   request = [context request];
+  params = [[request contentAsString] objectFromJSONString];
 
-  state = [request formValueForKey: @"state"];
+  state = [params objectForKey: @"state"];
   [moduleSettings setObject: state
                      forKey: @"ListState"];
   [us synchronize];
@@ -288,14 +293,35 @@
   return [self responseWithStatus: 204];
 }
 
-- (NSString *) listStateStyle
+- (BOOL) listIsCollapsed
 {
   NSString *state;
 
   [self _setupContext];
   state = [moduleSettings objectForKey: @"ListState"];
 
-  return (state && [state compare: @"collapse"] == NSOrderedSame)? @"display: none;" : @"";
+  return (state && [state compare: @"collapse"] == NSOrderedSame);
+}
+
+- (WOResponse *) saveFoldersOrderAction
+{
+  WORequest *request;
+  NSDictionary *params;
+  NSArray *list;
+
+  [self _setupContext];
+  request = [context request];
+  params = [[request contentAsString] objectFromJSONString];
+
+  list = [params objectForKey: @"folders"];
+  if (list)
+    [moduleSettings setObject: list
+                       forKey: @"FoldersOrder"];
+  else
+    [moduleSettings removeObjectForKey: @"FoldersOrder"];
+  [us synchronize];
+
+  return [self responseWithStatus: 204];
 }
 
 - (unsigned int) firstDayOfWeek
@@ -328,3 +354,178 @@
 }
 
 @end
+
+/* Component Viewer, parent class of Appointment Viewer and Task Viewer */
+
+@interface UIxComponentViewTemplate : UIxComponent
+{
+  id item;
+}
+@end
+
+@implementation UIxComponentViewTemplate
+
+- (id) init
+{
+  if ((self = [super init]))
+    {
+      item = nil;
+    }
+
+  return self;
+}
+
+- (void) dealloc
+{
+  [item release];
+  [super dealloc];
+}
+
+- (void) setItem: (id) _item
+{
+  ASSIGN (item, _item);
+}
+
+- (id) item
+{
+  return item;
+}
+
+- (NSArray *) replyList
+{
+  return [NSArray arrayWithObjects:
+                    [NSNumber numberWithInt: iCalPersonPartStatAccepted],
+	   [NSNumber numberWithInt: iCalPersonPartStatDeclined],
+	   [NSNumber numberWithInt: iCalPersonPartStatNeedsAction],
+	   [NSNumber numberWithInt: iCalPersonPartStatTentative],
+	   [NSNumber numberWithInt: iCalPersonPartStatDelegated],
+		  nil];
+}
+
+- (NSString *) itemReplyText
+{
+  NSString *word;
+
+  word = [iCalPerson descriptionForParticipationStatus: [item intValue]];
+
+  return [self labelForKey: [NSString stringWithFormat: @"partStat_%@", word]];
+}
+
+@end
+
+/* Appointment Viewer */
+
+@interface UIxAppointmentViewTemplate : UIxComponentViewTemplate
+@end
+
+@implementation UIxAppointmentViewTemplate
+@end
+
+/* Task Viewer */
+
+@interface UIxTaskViewTemplate : UIxComponentViewTemplate
+@end
+
+@implementation UIxTaskViewTemplate
+@end
+
+/* Component Editor, parent class of Appointment Editor and Task Editor */
+
+@interface UIxComponentEditorTemplate : UIxComponent
+{
+  id item;
+}
+@end
+
+@implementation UIxComponentEditorTemplate
+
+- (id) init
+{
+  if ((self = [super init]))
+    {
+      item = nil;
+    }
+
+  return self;
+}
+
+- (void) dealloc
+{
+  [item release];
+  [super dealloc];
+}
+
+- (void) setItem: (id) _item
+{
+  ASSIGN (item, _item);
+}
+
+- (id) item
+{
+  return item;
+}
+
+- (NSArray *) repeatList
+{
+  static NSArray *repeatItems = nil;
+
+  if (!repeatItems)
+    {
+      repeatItems = [NSArray arrayWithObjects: @"never",
+                             @"daily",
+                             @"weekly",
+                             @"monthly",
+                             @"yearly",
+                             nil];
+      [repeatItems retain];
+    }
+
+  return repeatItems;
+}
+
+- (NSString *) itemRepeatText
+{
+  return [self labelForKey: [NSString stringWithFormat: @"repeat_%@", [item uppercaseString]]];
+}
+
+@end
+
+/* Appointment Editor */
+
+@interface UIxAppointmentEditorTemplate : UIxComponentEditorTemplate
+@end
+
+@implementation UIxAppointmentEditorTemplate
+@end
+
+/* Task Editor */
+
+@interface UIxTaskEditorTemplate : UIxComponentEditorTemplate
+@end
+
+@implementation UIxTaskEditorTemplate
+
+- (NSArray *) statusList
+{
+  static NSArray *statusItems = nil;
+
+  if (!statusItems)
+    {
+      statusItems = [NSArray arrayWithObjects: @"not-specified",
+                             @"needs-action",
+                             @"in-process",
+                             @"completed",
+                             @"cancelled",
+                             nil];
+      [statusItems retain];
+    }
+
+  return statusItems;
+}
+
+- (NSString *) itemStatusText
+{
+  return [self labelForKey: [NSString stringWithFormat: @"status_%@", [item uppercaseString]]];
+}
+
+@end
