@@ -1,8 +1,6 @@
 /* UIxCalMonthView.m - this file is part of SOGo
  *
- * Copyright (C) 2006, 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2006-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,15 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSString.h>
 
-#import <NGObjWeb/WOApplication.h>
 
 #import <NGExtensions/NSCalendarDate+misc.h>
 #import <SoObjects/SOGo/NSCalendarDate+SOGo.h>
 
+#import <SOPE/NGCards/iCalRecurrenceRule.h>
+
 #import <SOGoUI/SOGoAptFormatter.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
@@ -66,6 +62,8 @@
   [weeksToDisplay release];
   [currentTableDay release];
   [currentWeek release];
+  [enabledWeekDays release];
+  free(daysNumbersToDisplay);
   [super dealloc];
 }
 
@@ -78,16 +76,19 @@
 
 - (NSArray *) headerDaysToDisplay
 {
-  NSMutableArray *headerDaysToDisplay;
-  unsigned int counter;
   NSCalendarDate *currentDate;
+  NSMutableArray *headerDaysToDisplay;
+  NSString *weekDay;
+  unsigned int counter;
 
   headerDaysToDisplay = [NSMutableArray arrayWithCapacity: 7];
-  currentDate
-    = [[context activeUser] firstDayOfWeekForDate: [self selectedDate]];
+  currentDate = [[context activeUser] firstDayOfWeekForDate: [self selectedDate]];
   for (counter = 0; counter < 7; counter++)
     {
-      [headerDaysToDisplay addObject: currentDate];
+      // Skip days that match disabled weekdays in user's defaults
+      weekDay = iCalWeekDayString[[currentDate dayOfWeek]];
+      if ([enabledWeekDays count] == 0 || [enabledWeekDays containsObject: weekDay])
+        [headerDaysToDisplay addObject: currentDate];
       currentDate = [currentDate tomorrow];
     }
 
@@ -96,31 +97,37 @@
 
 - (NSArray *) weeksToDisplay
 {
-  NSMutableArray *week;
-  unsigned int counter, day;
   NSCalendarDate *currentDate, *selectedDate, *lastDayOfMonth, *firstOfAllDays;
+  NSMutableArray *week;
+  NSString *weekDay;
+  unsigned int counter, day, enabledCounter;
   unsigned int firstToLast, weeks;
 
   if (!weeksToDisplay)
     {
       selectedDate = [self selectedDate];
-      firstOfAllDays
-	= [[context activeUser] firstDayOfWeekForDate:
-				  [selectedDate firstDayOfMonth]];
+      firstOfAllDays = [[context activeUser] firstDayOfWeekForDate:
+                                               [selectedDate firstDayOfMonth]];
       lastDayOfMonth = [selectedDate lastDayOfMonth];
-      firstToLast = ([lastDayOfMonth timeIntervalSinceDate: firstOfAllDays]
-		     / 86400) + 1;
+      firstToLast = ([lastDayOfMonth timeIntervalSinceDate: firstOfAllDays] / 86400) + 1;
       weeks = firstToLast / 7;
       if ((firstToLast % 7))
 	weeks++;
       weeksToDisplay = [NSMutableArray arrayWithCapacity: weeks];
+      daysNumbersToDisplay = malloc (weeks * 7 * sizeof (unsigned int));
       currentDate = firstOfAllDays;
-      for (counter = 0; counter < weeks; counter++)
+      for (counter = 0, enabledCounter = 0; counter < weeks; counter++)
 	{
 	  week = [NSMutableArray arrayWithCapacity: 7];
 	  for (day = 0; day < 7; day++)
 	    {
-	      [week addObject: currentDate];
+              weekDay = iCalWeekDayString[[currentDate dayOfWeek]];
+              if ([enabledWeekDays count] == 0 || [enabledWeekDays containsObject: weekDay])
+                {
+                  [week addObject: currentDate];
+                  daysNumbersToDisplay[enabledCounter] = (counter * 7) + day;
+                  enabledCounter++;
+                }
 	      currentDate = [currentDate tomorrow];
 	    }
 	  [weeksToDisplay addObject: week];
@@ -144,7 +151,7 @@
   date = [firstDay dateByAddingYears: 0 months: monthsOffset
 		   days: 0 hours: 0 minutes: 0 seconds: 0];
 
-  return [self queryParametersBySettingSelectedDate: date];
+  return [self queryParametersBySettingSelectedDate: [self _nextValidDate: date]];
 }
 
 - (NSDictionary *) monthBeforePrevMonthQueryParameters
@@ -221,8 +228,10 @@
 
 - (int) currentDayNumber
 {
-  return ([currentWeek indexOfObject: currentTableDay]
-          + [weeksToDisplay indexOfObject: currentWeek] * 7);
+  int i = [currentWeek indexOfObject: currentTableDay] +
+    ([weeksToDisplay indexOfObject: currentWeek] * [currentWeek count]);
+
+  return daysNumbersToDisplay[i];
 }
 
 - (void) setCurrentWeek: (NSArray *) newCurrentWeek
@@ -235,27 +244,6 @@
   return currentWeek;
 }
 
-- (NSString *) labelForCurrentDayCell
-{
-  NSCalendarDate *lastDayOfMonth;
-  NSString *label, *monthOfYear;
-  int dayOfMonth;
-
-  dayOfMonth = [currentTableDay dayOfMonth];
-  lastDayOfMonth = [currentTableDay lastDayOfMonth];
-  if (dayOfMonth == 1
-      || [currentTableDay isDateOnSameDay: lastDayOfMonth])
-    {
-      monthOfYear
-        = [monthNames objectAtIndex: [currentTableDay monthOfYear]];
-      label = [NSString stringWithFormat: @"%d %@", dayOfMonth, monthOfYear];
-    }
-  else
-    label = [NSString stringWithFormat: @"%d", dayOfMonth];
-
-  return label;
-}
-
 - (NSString *) headerDayCellClasses
 {
   unsigned int dayOfWeek;
@@ -265,24 +253,19 @@
   return [NSString stringWithFormat: @"headerDay day%d", dayOfWeek];
 }
 
-- (NSString *) dayHeaderNumber
+- (NSString *) dayHeaderMonth
 {
-  NSString *nameOfMonth, *dayHeaderNumber;
+  NSString *nameOfMonth = nil;
   unsigned int dayOfMonth;
 
   dayOfMonth = [currentTableDay dayOfMonth];
   if (dayOfMonth == 1
       || [currentTableDay isDateOnSameDay: [currentTableDay lastDayOfMonth]])
     {
-      nameOfMonth
-        = [self localizedNameForMonthOfYear: [currentTableDay monthOfYear]];
-      dayHeaderNumber = [NSString stringWithFormat: @"%d %@", dayOfMonth,
-                                  nameOfMonth];
+      nameOfMonth = [self localizedNameForMonthOfYear: [currentTableDay monthOfYear]];
     }
-  else
-    dayHeaderNumber = [NSString stringWithFormat: @"%d", dayOfMonth];
 
-  return dayHeaderNumber;
+  return nameOfMonth;
 }
 
 - (NSString *) dayCellClasses
@@ -299,7 +282,7 @@
 
   [classes appendFormat: @"day weekOf%d week%dof%d day%d",
            numberOfWeeks,
-           [weeksToDisplay indexOfObject: currentWeek],
+           (int)[weeksToDisplay indexOfObject: currentWeek],
            numberOfWeeks, dayOfWeek];
   if (realDayOfWeek == 0 || realDayOfWeek == 6)
     [classes appendString: @" weekEndDay"];
@@ -318,8 +301,9 @@
   NSCalendarDate *firstDayOfMonth;
 
   firstDayOfMonth = [[self selectedDate] firstDayOfMonth];
+  firstDayOfMonth = [[context activeUser] firstDayOfWeekForDate: firstDayOfMonth];
 
-  return [[context activeUser] firstDayOfWeekForDate: firstDayOfMonth];
+  return [self _nextValidDate: firstDayOfMonth];
 }
 
 - (NSCalendarDate *) endDate
