@@ -19,11 +19,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <SOGoUI/UIxComponent.h>
+#include <UIxCalendarSelector.h>
 
-@interface UIxCalViewPrint : UIxComponent
+@interface UIxCalViewPrint : UIxCalendarSelector
 {
   id item;
+  SOGoUserSettings *us;
 }
 
 @end
