@@ -18,8 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSString.h>
-#import <Foundation/NSArray.h>
 
 #import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoUser.h>
@@ -49,6 +47,7 @@ static NSArray *layoutItems = nil;
 - (void) dealloc
 {
   [item release];
+  [super dealloc];
 }
 
 - (void) setItem: (NSString *) newItem
@@ -68,7 +67,7 @@ static NSArray *layoutItems = nil;
 
 - (NSString *) itemPrintLayoutText
 {
-  return [self labelForKey: [NSString stringWithFormat: item]];
+  return [self labelForKey: [NSString stringWithFormat: @"%@", item]];
 }
 
 //
