@@ -255,13 +255,29 @@
   /**
    * @ngInject
    */
-  ErrorInterceptor.$inject = ['$rootScope', '$q'];
-  function ErrorInterceptor($rootScope, $q) {
+  ErrorInterceptor.$inject = ['$rootScope', '$window', '$q', '$injector'];
+  function ErrorInterceptor($rootScope, $window, $q, $injector) {
     return {
       responseError: function(rejection) {
+        var deferred, iframe;
         if (/^application\/json/.test(rejection.config.headers.Accept)) {
-          // Broadcast the response error
-          $rootScope.$broadcast('http:Error', rejection);
+          // Handle CAS ticket renewal
+          if ($window.usesCASAuthentication && rejection.status == -1) {
+            deferred = $q.defer();
+            iframe = angular.element('<iframe class="ng-hide" src="' + UserFolderURL + 'recover"></iframe>');
+            iframe.on('load', function() {
+              // Once the browser has followed the redirection, send the initial request
+              var $http = $injector.get('$http');
+              $http(rejection.config).then(deferred.resolve, deferred.reject);
+              iframe.remove();
+            });
+            document.body.appendChild(iframe[0]);
+            return deferred.promise;
+          }
+          else {
+            // Broadcast the response error
+            $rootScope.$broadcast('http:Error', rejection);
+          }
         }
         return $q.reject(rejection);
       }
