@@ -17,7 +17,8 @@
     $scope.activeUser = sgSettings.activeUser();
     $scope.baseURL = sgSettings.baseURL();
     $scope.leftIsClose = !$mdMedia(sgConstant['gt-md']);
-    $scope.centerIsClose = !!$window.centerIsClose;
+    // Don't hide the center list when on a small device
+    $scope.centerIsClose = !!$window.centerIsClose && !$scope.leftIsClose;
 
     // Show current day in top bar
     $scope.currentDay = window.currentDay;
@@ -109,7 +110,10 @@
     // Listen to HTTP errors broadcasted from HTTP interceptor
     $rootScope.$on('http:Error', onHttpError);
 
-    Alarm.getAlarms();
+    if (sgSettings.activeUser('path').calendar) {
+      // Fetch Calendar alarms
+      Alarm.getAlarms();
+    }
   }
 
   angular.module('SOGo.Common')
