@@ -46,13 +46,11 @@
       $Preferences: Preferences
     });
     // Initialize categories from user's defaults
-    Preferences.ready().then(function() {
-      if (Preferences.defaults.SOGoContactsCategories) {
-        Card.$categories = Preferences.defaults.SOGoContactsCategories;
-      }
-      if (Preferences.defaults.SOGoAlternateAvatar)
-        Card.$alternateAvatar = Preferences.defaults.SOGoAlternateAvatar;
-    });
+    if (Preferences.defaults.SOGoContactsCategories) {
+      Card.$categories = Preferences.defaults.SOGoContactsCategories;
+    }
+    if (Preferences.defaults.SOGoAlternateAvatar)
+      Card.$alternateAvatar = Preferences.defaults.SOGoAlternateAvatar;
 
     return Card; // return constructor
   }];
@@ -138,8 +136,10 @@
   Card.prototype.init = function(data, partial) {
     var _this = this;
 
-    this.refs = [];
-    this.categories = [];
+    if (angular.isUndefined(this.refs))
+      this.refs = [];
+    if (angular.isUndefined(this.categories))
+      this.categories = [];
     this.c_screenname = null;
     angular.extend(this, data);
     if (!this.$$fullname)
@@ -149,18 +149,24 @@
     if (!this.$$image)
       this.$$image = this.image;
     if (!this.$$image)
-      Card.$Preferences.avatar(this.$$email, 32, {no_404: true}).then(function(url) {
-        _this.$$image = url;
-      });
+      this.$$image = Card.$Preferences.avatar(this.$$email, 32, {no_404: true});
     if (this.hasphoto)
       this.photoURL = Card.$$resource.path(this.pid, this.id, 'photo');
     if (this.isgroup)
       this.c_component = 'vlist';
     this.$avatarIcon = this.$isList()? 'group' : 'person';
+    if (data.orgs && data.orgs.length)
+      this.orgs = _.map(data.orgs, function(org) { return { 'value': org }; });
     if (data.notes && data.notes.length)
       this.notes = _.map(data.notes, function(note) { return { 'value': note }; });
     else if (!this.notes || !this.notes.length)
       this.notes = [ { value: '' } ];
+    // Lowercase the type of specific fields
+    angular.forEach(['addresses', 'phones', 'urls'], function(key) {
+      angular.forEach(_this[key], function(o) {
+        if (o.type) o.type = o.type.toLowerCase();
+      });
+    });
     // Instanciate Card objects for list members
     angular.forEach(this.refs, function(o, i) {
       if (o.email) o.emails = [{value: o.email}];
@@ -169,11 +175,9 @@
     });
     // Instanciate date object of birthday
     if (this.birthday) {
-      Card.$Preferences.ready().then(function() {
-        var dlp = Card.$Preferences.$mdDateLocaleProvider;
-        _this.birthday = _this.birthday.parseDate(dlp, '%Y-%m-%d');
-        _this.$birthday = dlp.formatDate(_this.birthday);
-      });
+      var dlp = Card.$Preferences.$mdDateLocaleProvider;
+      this.birthday = this.birthday.parseDate(dlp, '%Y-%m-%d');
+      this.$birthday = dlp.formatDate(this.birthday);
     }
 
     this.$loaded = angular.isDefined(this.c_name)? Card.STATUS.LOADED : Card.STATUS.NOT_LOADED;
@@ -294,8 +298,8 @@
         names.push(this.c_sn);
       if (names.length > 0)
         fn = names.join(' ');
-      else if (this.c_org && this.c_org.length > 0) {
-        fn = this.c_org;
+      else if (this.org && this.org.length > 0) {
+        fn = this.org;
       }
       else if (this.emails && this.emails.length > 0) {
         email = _.find(this.emails, function(i) { return i.value !== ''; });
@@ -303,6 +307,8 @@
           fn = email.value;
       }
     }
+    if (this.contactinfo)
+      fn += ' (' + this.contactinfo.split("\n").join("; ") + ')';
 
     return fn;
   };
@@ -311,12 +317,8 @@
     var description = [];
     if (this.title) description.push(this.title);
     if (this.role) description.push(this.role);
-    if (this.orgUnits && this.orgUnits.length > 0)
-      _.forEach(this.orgUnits, function(unit) {
-        if (unit.value !== '')
-          description.push(unit.value);
-      });
-    if (this.c_org) description.push(this.c_org);
+    if (this.org) description.push(this.org);
+    if (this.orgs) description = _.concat(description, _.map(this.orgs, 'value'));
     if (this.description) description.push(this.description);
 
     return description.join(', ');
@@ -385,20 +387,14 @@
     return this.c_component == 'vlist' && condition;
   };
 
-  Card.prototype.$addOrgUnit = function(orgUnit) {
-    if (angular.isUndefined(this.orgUnits)) {
-      this.orgUnits = [{value: orgUnit}];
+  Card.prototype.$addOrg = function(org) {
+    if (angular.isUndefined(this.orgs)) {
+      this.orgs = [org];
     }
-    else {
-      for (var i = 0; i < this.orgUnits.length; i++) {
-        if (this.orgUnits[i].value == orgUnit) {
-          break;
-        }
-      }
-      if (i == this.orgUnits.length)
-        this.orgUnits.push({value: orgUnit});
+    else if (org != this.org && !_.includes(this.orgs, org)) {
+      this.orgs.push(org);
     }
-    return this.orgUnits.length - 1;
+    return this.orgs.length - 1;
   };
 
   // Card.prototype.$addCategory = function(category) {
@@ -498,16 +494,20 @@
   Card.prototype.explode = function() {
     var _this = this, cards = [], data;
 
-    if (this.emails.length > 1) {
-      data = this.$omit();
-      _.forEach(this.emails, function(email) {
-        var card = new Card(angular.extend({}, data, {emails: [email]}));
-        cards.push(card);
-      });
-      return cards;
+    if (this.emails) {
+      if (this.emails.length > 1) {
+        data = this.$omit();
+        _.forEach(this.emails, function(email) {
+          var card = new Card(angular.extend({}, data, {emails: [email]}));
+          cards.push(card);
+        });
+        return cards;
+      }
+      else
+        return [this];
     }
-    else
-      return [this];
+
+    return [];
   };
 
   /**
@@ -609,6 +609,10 @@
         card.birthday = '';
     }
 
+    // We flatten the organizations to an array of strings
+    if (this.orgs)
+      card.orgs = _.map(this.orgs, 'value');
+
     // We flatten the notes to an array of strings
     if (this.notes)
       card.notes = _.map(this.notes, 'value');
