@@ -44,15 +44,14 @@
       selectedFolder: null,
       $refreshTimeout: null,
       $virtualMode: false,
+      $virtualPath: false,
       PRELOAD: PRELOAD
     });
     // Initialize sort parameters from user's settings
-    Preferences.ready().then(function() {
-      if (Preferences.settings.Mail.SortingState) {
-        Mailbox.$query.sort = Preferences.settings.Mail.SortingState[0];
-        Mailbox.$query.asc = parseInt(Preferences.settings.Mail.SortingState[1]);
-      }
-    });
+    if (Preferences.settings.Mail.SortingState) {
+      Mailbox.$query.sort = Preferences.settings.Mail.SortingState[0];
+      Mailbox.$query.asc = parseInt(Preferences.settings.Mail.SortingState[1]);
+    }
 
     return Mailbox; // return constructor
   }];
@@ -157,16 +156,47 @@
    */
   Mailbox.prototype.init = function(data) {
     var _this = this;
-    this.$isLoading = true;
-    this.$messages = [];
-    this.uidsMap = {};
+    if (angular.isUndefined(this.uidsMap) || data.headers) {
+      this.$isLoading = true;
+      this.$messages = [];
+      this.uidsMap = {};
+    }
     angular.extend(this, data);
     if (this.path) {
       this.id = this.$id();
       this.$acl = new Mailbox.$$Acl('Mail/' + this.id);
     }
+    this.$displayName = this.name;
     if (this.type) {
       this.$isEditable = this.isEditable();
+      this.$isSpecial = true;
+      if (this.type == 'inbox') {
+        this.$displayName = l('InboxFolderName');
+        this.$icon = 'inbox';
+      }
+      else if (this.type == 'draft') {
+        this.$displayName = l('DraftsFolderName');
+        this.$icon = 'drafts';
+      }
+      else if (this.type == 'sent') {
+        this.$displayName = l('SentFolderName');
+        this.$icon = 'send';
+      }
+      else if (this.type == 'trash') {
+        this.$displayName = l('TrashFolderName');
+        this.$icon = 'delete';
+      }
+      else if (this.type == 'junk') {
+        this.$displayName = l('JunkFolderName');
+        this.$icon = 'thumb_down';
+      }
+      else if (this.type == 'additional') {
+        this.$icon = 'folder_shared';
+      }
+      else {
+        this.$isSpecial = false;
+        this.$icon = 'folder_open';
+      }
     }
     this.$isNoInferiors = this.isNoInferiors();
     if (angular.isUndefined(this.$shadowData)) {
@@ -311,46 +341,43 @@
       _this.$isLoading = true;
     });
 
-    return Mailbox.$Preferences.ready().then(function() {
+    if (Mailbox.$refreshTimeout)
+      Mailbox.$timeout.cancel(Mailbox.$refreshTimeout);
 
-      if (Mailbox.$refreshTimeout)
-        Mailbox.$timeout.cancel(Mailbox.$refreshTimeout);
+    if (sortingAttributes)
+      // Sorting preferences are common to all mailboxes
+      angular.extend(Mailbox.$query, sortingAttributes);
 
-      if (sortingAttributes)
-        // Sorting preferences are common to all mailboxes
-        angular.extend(Mailbox.$query, sortingAttributes);
-
-      angular.extend(options, { sortingAttributes: Mailbox.$query });
-      if (angular.isDefined(filters)) {
-        options.filters = _.reject(filters, function(filter) {
-          return !filter.searchInput || filter.searchInput.length === 0;
-        });
-        // Decompose filters that match two fields
-        _.forEach(options.filters, function(filter) {
-          var secondFilter,
-              match = filter.searchBy.match(/(\w+)_or_(\w+)/);
-          if (match) {
-            options.sortingAttributes.match = 'OR';
-            filter.searchBy = match[1];
-            secondFilter = angular.copy(filter);
-            secondFilter.searchBy = match[2];
-            options.filters.push(secondFilter);
-          }
-        });
-      }
-
-      // Restart the refresh timer, if needed
-      if (!Mailbox.$virtualMode) {
-        var refreshViewCheck = Mailbox.$Preferences.defaults.SOGoRefreshViewCheck;
-        if (refreshViewCheck && refreshViewCheck != 'manually') {
-          var f = angular.bind(_this, Mailbox.prototype.$filter, null, filters);
-          Mailbox.$refreshTimeout = Mailbox.$timeout(f, refreshViewCheck.timeInterval()*1000);
+    angular.extend(options, { sortingAttributes: Mailbox.$query });
+    if (angular.isDefined(filters)) {
+      options.filters = _.reject(filters, function(filter) {
+        return !filter.searchInput || filter.searchInput.length === 0;
+      });
+      // Decompose filters that match two fields
+      _.forEach(options.filters, function(filter) {
+        var secondFilter,
+            match = filter.searchBy.match(/(\w+)_or_(\w+)/);
+        if (match) {
+          options.sortingAttributes.match = 'OR';
+          filter.searchBy = match[1];
+          secondFilter = angular.copy(filter);
+          secondFilter.searchBy = match[2];
+          options.filters.push(secondFilter);
         }
-      }
+      });
+    }
 
-      var futureMailboxData = Mailbox.$$resource.post(_this.id, 'view', options);
-      return _this.$unwrap(futureMailboxData);
-    });
+    // Restart the refresh timer, if needed
+    if (!Mailbox.$virtualMode) {
+      var refreshViewCheck = Mailbox.$Preferences.defaults.SOGoRefreshViewCheck;
+      if (refreshViewCheck && refreshViewCheck != 'manually') {
+        var f = angular.bind(this, Mailbox.prototype.$filter, null, filters);
+        Mailbox.$refreshTimeout = Mailbox.$timeout(f, refreshViewCheck.timeInterval()*1000);
+      }
+    }
+
+    var futureMailboxData = Mailbox.$$resource.post(this.id, 'view', options);
+    return this.$unwrap(futureMailboxData);
   };
 
   /**
@@ -445,6 +472,16 @@
     return this.flags.indexOf('noselect') >= 0;
   };
 
+  /**
+   * @function getClassName
+   * @memberof Mailbox.prototype
+   * @desc Not used but defined because it is called from UIxAclEditor.wox.
+   * @returns a string representing the foreground CSS class name
+   */
+  Mailbox.prototype.getClassName = function(base) {
+    return false;
+  };
+
   /**
    * @function $rename
    * @memberof AddressBook.prototype
@@ -494,9 +531,8 @@
     i = _.indexOf(_.map(children, 'id'), this.id);
 
     return this.$save().then(function(data) {
-      var sibling;
-      angular.extend(_this, data); // update the path attribute
-      _this.id = _this.$id();
+      var sibling, oldPath = _this.path;
+      _this.init(data); // update the path and id
 
       // Move mailbox among its siblings according to its new name
       children.splice(i, 1);
@@ -511,6 +547,17 @@
         i = children.length;
       }
       children.splice(i, 0, _this);
+
+      // Update the path and id of children
+      var pathRE = new RegExp('^' + oldPath);
+      var _updateChildren = function(mailbox) {
+        _.forEach(mailbox.children, function(child) {
+          child.path = child.path.replace(pathRE, _this.path);
+          child.id = child.$id();
+          _updateChildren(child);
+        });
+      };
+      _updateChildren(_this);
     });
   };
 
@@ -530,6 +577,16 @@
       });
   };
 
+  /**
+   * @function $canFolderAs
+   * @memberof Mailbox.prototype
+   * @desc Check if the folder can be set as Drafts/Sent/Trash
+   * @returns true if folder is eligible
+   */
+  Mailbox.prototype.$canFolderAs = function() {
+    return this.type == 'folder' && this.level === 0;
+  };
+
   /**
    * @function $setFolderAs
    * @memberof Mailbox.prototype
@@ -572,7 +629,14 @@
    * @returns a promise of the HTTP operation
    */
   Mailbox.prototype.$markAsRead = function() {
-    return Mailbox.$$resource.post(this.id, 'markRead');
+    var _this = this;
+
+    return Mailbox.$$resource.post(this.id, 'markRead').then(function() {
+      _this.unseenCount = 0;
+      _.forEach(_this.$messages, function(message) {
+        message.isread = true;
+      });
+    });
   };
 
   /**
@@ -838,7 +902,7 @@
         _this.init(data);
 
         if (_this.uids) {
-          Mailbox.$log.debug('unwrapping ' + data.uids.length + ' messages');
+          Mailbox.$log.debug('unwrapping ' + _this.uids.length + ' messages');
 
           // First entry of 'headers' are keys
           headers = _.invokeMap(_this.headers[0], 'toLowerCase');
@@ -880,6 +944,7 @@
     }, function(data) {
       angular.extend(_this, data);
       _this.isError = true;
+      _this.$isLoading = false;
       deferred.reject();
     });
 
