@@ -48,16 +48,15 @@
       $$resource: new Resource(Settings.activeUser('folderURL') + 'Mail', Settings.activeUser()),
       $avatar: angular.bind(Preferences, Preferences.avatar)
     });
+
     // Initialize tags form user's defaults
-    Preferences.ready().then(function() {
-      if (Preferences.defaults.SOGoMailLabelsColors) {
-        Message.$tags = Preferences.defaults.SOGoMailLabelsColors;
-      }
-      if (Preferences.defaults.SOGoMailDisplayRemoteInlineImages &&
-          Preferences.defaults.SOGoMailDisplayRemoteInlineImages == 'always') {
-        Message.$displayRemoteInlineImages = true;
-      }
-    });
+    if (Preferences.defaults.SOGoMailLabelsColors) {
+      Message.$tags = Preferences.defaults.SOGoMailLabelsColors;
+    }
+    if (Preferences.defaults.SOGoMailDisplayRemoteInlineImages &&
+        Preferences.defaults.SOGoMailDisplayRemoteInlineImages == 'always') {
+      Message.$displayRemoteInlineImages = true;
+    }
 
     return Message; // return constructor
   }];
@@ -129,6 +128,9 @@
     if (options && options.asDraft && this.draftId) {
       id = buildPath() + '/' + this.draftId; // add draft ID
     }
+    if (options && options.withResourcePath) {
+      id = Message.$$resource.path(id); // return absolute URL
+    }
 
     return id;
   };
@@ -474,9 +476,9 @@
    * @param {string} filename - the filename of the attachment to delete
    */
   Message.prototype.$deleteAttachment = function(filename) {
-    var action = 'deleteAttachment?filename=' + filename;
+    var data = { 'filename': filename };
     var _this = this;
-    Message.$$resource.post(this.$absolutePath({asDraft: true}), action).then(function(data) {
+    Message.$$resource.fetch(this.$absolutePath({asDraft: true}), 'deleteAttachment', data).then(function(data) {
       Message.$timeout(function() {
         _this.editable.attachmentAttrs = _.filter(_this.editable.attachmentAttrs, function(attachment) {
           return attachment.filename != filename;
