@@ -52,28 +52,26 @@
       $refreshTimeout: null,
       $ghost: {}
     });
-    Preferences.ready().then(function() {
-      // Initialize filter parameters from user's settings
-      if (Preferences.settings.Calendar.EventsFilterState)
-        Component.$queryEvents.filterpopup = Preferences.settings.Calendar.EventsFilterState;
-      if (Preferences.settings.Calendar.TasksFilterState)
-        Component.$queryTasks.filterpopup = Preferences.settings.Calendar.TasksFilterState;
-      if (Preferences.settings.Calendar.EventsSortingState) {
-        Component.$queryEvents.sort = Preferences.settings.Calendar.EventsSortingState[0];
-        Component.$queryEvents.asc = parseInt(Preferences.settings.Calendar.EventsSortingState[1]);
-      }
-      if (Preferences.settings.Calendar.TasksSortingState) {
-        Component.$queryTasks.sort = Preferences.settings.Calendar.TasksSortingState[0];
-        Component.$queryTasks.asc = parseInt(Preferences.settings.Calendar.TasksSortingState[1]);
-      }
-      Component.$queryTasks.show_completed = parseInt(Preferences.settings.ShowCompletedTasks);
-      // Initialize categories from user's defaults
-      Component.$categories = Preferences.defaults.SOGoCalendarCategoriesColors;
-      // Initialize time format from user's defaults
-      if (Preferences.defaults.SOGoTimeFormat) {
-        Component.timeFormat = Preferences.defaults.SOGoTimeFormat;
-      }
-    });
+    // Initialize filter parameters from user's settings
+    if (Preferences.settings.Calendar.EventsFilterState)
+      Component.$queryEvents.filterpopup = Preferences.settings.Calendar.EventsFilterState;
+    if (Preferences.settings.Calendar.TasksFilterState)
+      Component.$queryTasks.filterpopup = Preferences.settings.Calendar.TasksFilterState;
+    if (Preferences.settings.Calendar.EventsSortingState) {
+      Component.$queryEvents.sort = Preferences.settings.Calendar.EventsSortingState[0];
+      Component.$queryEvents.asc = parseInt(Preferences.settings.Calendar.EventsSortingState[1]);
+    }
+    if (Preferences.settings.Calendar.TasksSortingState) {
+      Component.$queryTasks.sort = Preferences.settings.Calendar.TasksSortingState[0];
+      Component.$queryTasks.asc = parseInt(Preferences.settings.Calendar.TasksSortingState[1]);
+    }
+    Component.$queryTasks.show_completed = parseInt(Preferences.settings.ShowCompletedTasks);
+    // Initialize categories from user's defaults
+    Component.$categories = Preferences.defaults.SOGoCalendarCategoriesColors;
+    // Initialize time format from user's defaults
+    if (Preferences.defaults.SOGoTimeFormat) {
+      Component.timeFormat = Preferences.defaults.SOGoTimeFormat;
+    }
 
     return Component; // return constructor
   }];
@@ -124,19 +122,15 @@
    * current view.
    */
   Component.$startRefreshTimeout = function(type) {
-    var _this = this;
-
     if (Component.$refreshTimeout)
       Component.$timeout.cancel(Component.$refreshTimeout);
 
-    Component.$Preferences.ready().then(function() {
-      // Restart the refresh timer, if needed
-      var refreshViewCheck = Component.$Preferences.defaults.SOGoRefreshViewCheck;
-      if (refreshViewCheck && refreshViewCheck != 'manually') {
-        var f = angular.bind(Component.$rootScope, Component.$rootScope.$emit, 'calendars:list');
-        Component.$refreshTimeout = Component.$timeout(f, refreshViewCheck.timeInterval()*1000);
-      }
-    });
+    // Restart the refresh timer, if needed
+    var refreshViewCheck = Component.$Preferences.defaults.SOGoRefreshViewCheck;
+    if (refreshViewCheck && refreshViewCheck != 'manually') {
+      var f = angular.bind(Component.$rootScope, Component.$rootScope.$emit, 'calendars:list');
+      Component.$refreshTimeout = Component.$timeout(f, refreshViewCheck.timeInterval()*1000);
+    }
   };
 
   /**
@@ -166,44 +160,41 @@
         queryKey = '$query' + type.capitalize(),
         params = {
           day: '' + year + (month < 10?'0':'') + month + (day < 10?'0':'') + day,
-        };
+        },
+        futureComponentData,
+        dirty = false,
+        otherType;
 
     Component.$startRefreshTimeout(type);
 
-    return this.$Preferences.ready().then(function() {
-      var futureComponentData,
-          dirty = false,
-          otherType;
+    angular.extend(this.$query, params);
 
-      angular.extend(_this.$query, params);
+    if (options) {
+      _.forEach(_.keys(options), function(key) {
+        // Query parameters common to events and tasks are compared
+        dirty |= (_this.$query[key] && options[key] != Component.$query[key]);
+        if (key == 'reload' && options[key])
+          dirty = true;
+        // Update either the common parameters or the type-specific parameters
+        else if (angular.isDefined(_this.$query[key]))
+          _this.$query[key] = options[key];
+        else
+          _this[queryKey][key] = options[key];
+      });
+    }
 
-      if (options) {
-        _.forEach(_.keys(options), function(key) {
-          // Query parameters common to events and tasks are compared
-          dirty |= (_this.$query[key] && options[key] != Component.$query[key]);
-          if (key == 'reload' && options[key])
-            dirty = true;
-          // Update either the common parameters or the type-specific parameters
-          else if (angular.isDefined(_this.$query[key]))
-            _this.$query[key] = options[key];
-          else
-            _this[queryKey][key] = options[key];
-        });
-      }
+    // Perform query with both common and type-specific parameters
+    futureComponentData = this.$$resource.fetch(null, type + 'list',
+                                                angular.extend(this[queryKey], this.$query));
 
-      // Perform query with both common and type-specific parameters
-      futureComponentData = _this.$$resource.fetch(null, type + 'list',
-                                                   angular.extend(_this[queryKey], _this.$query));
+    // Invalidate cached results of other type if $query has changed
+    if (dirty) {
+      otherType = (type == 'tasks')? '$events' : '$tasks';
+      delete Component[otherType];
+      Component.$log.debug('force reload of ' + otherType);
+    }
 
-      // Invalidate cached results of other type if $query has changed
-      if (dirty) {
-        otherType = (type == 'tasks')? '$events' : '$tasks';
-        delete Component[otherType];
-        Component.$log.debug('force reload of ' + otherType);
-      }
-
-      return _this.$unwrapCollection(type, futureComponentData);
-    });
+    return this.$unwrapCollection(type, futureComponentData);
   };
 
   /**
@@ -256,40 +247,36 @@
    * @returns a promise of a collection of objects describing the events blocks
    */
   Component.$eventsBlocksForView = function(view, date) {
-    var _this = this;
+    var firstDayOfWeek, viewAction, startDate, endDate, params;
 
-    return Component.$Preferences.ready().then(function(data) {
-      var firstDayOfWeek, viewAction, startDate, endDate, params;
-      firstDayOfWeek = Component.$Preferences.defaults.SOGoFirstDayOfWeek;
-
-      if (view == 'day') {
-        viewAction = 'dayView';
-        startDate = endDate = date;
-      }
-      else if (view == 'multicolumnday') {
-        viewAction = 'multicolumndayView';
-        startDate = endDate = date;
-      }
-      else if (view == 'week') {
-        viewAction = 'weekView';
-        startDate = date.beginOfWeek(firstDayOfWeek);
-        endDate = new Date();
-        endDate.setTime(startDate.getTime());
-        endDate.addDays(6);
-      }
-      else if (view == 'month') {
-        viewAction = 'monthView';
-        startDate = date;
-        startDate.setDate(1);
-        startDate = startDate.beginOfWeek(firstDayOfWeek);
-        endDate = new Date();
-        endDate.setTime(date.getTime());
-        endDate.setMonth(endDate.getMonth() + 1);
-        endDate.addDays(-1);
-        endDate = endDate.endOfWeek(firstDayOfWeek);
-      }
-      return _this.$eventsBlocks(viewAction, startDate, endDate);
-    });
+    firstDayOfWeek = Component.$Preferences.defaults.SOGoFirstDayOfWeek;
+    if (view == 'day') {
+      viewAction = 'dayView';
+      startDate = endDate = date;
+    }
+    else if (view == 'multicolumnday') {
+      viewAction = 'multicolumndayView';
+      startDate = endDate = date;
+    }
+    else if (view == 'week') {
+      viewAction = 'weekView';
+      startDate = date.beginOfWeek(firstDayOfWeek);
+      endDate = new Date();
+      endDate.setTime(startDate.getTime());
+      endDate.addDays(6);
+    }
+    else if (view == 'month') {
+      viewAction = 'monthView';
+      startDate = date;
+      startDate.setDate(1);
+      startDate = startDate.beginOfWeek(firstDayOfWeek);
+      endDate = new Date();
+      endDate.setTime(date.getTime());
+      endDate.setMonth(endDate.getMonth() + 1);
+      endDate.addDays(-1);
+      endDate = endDate.endOfWeek(firstDayOfWeek);
+    }
+    return this.$eventsBlocks(viewAction, startDate, endDate);
   };
 
   /**
@@ -316,10 +303,6 @@
         componentData.hour = start.getHourString();
         componentData.blocks = [];
         component = new Component(componentData);
-        // Filter out categories for which there's no associated color
-        component.categories = _.filter(component.categories, function(name) {
-          return Component.$Preferences.defaults.SOGoCalendarCategoriesColors[name];
-        });
         objects.push(component);
         return objects;
       };
@@ -437,10 +420,6 @@
         _.reduce(data[type], function(components, componentData, i) {
           var data = _.zipObject(fields, componentData), component;
           component = new Component(data);
-          // Filter out categories for which there's no associated color
-          component.categories = _.filter(component.categories, function(name) {
-            return Component.$Preferences.defaults.SOGoCalendarCategoriesColors[name];
-          });
           components.push(component);
           return components;
         }, components);
@@ -539,8 +518,13 @@
     else if (this.type == 'task')
       this.completed = new Date();
 
-    if (this.c_category)
-      this.categories = _.invokeMap(this.c_category, 'asCSSIdentifier');
+    if (this.c_category) {
+      // c_category is only defined in list mode (when calling $filter)
+      // Filter out categories for which there's no associated color
+      this.categories = _.invokeMap(_.filter(this.c_category, function(name) {
+        return Component.$Preferences.defaults.SOGoCalendarCategoriesColors[name];
+      }), 'asCSSIdentifier');
+    }
 
     // Parse recurrence rule definition and initialize default values
     this.$isRecurrent = angular.isDefined(data.repeat);
@@ -565,11 +549,15 @@
       this.repeat.frequency = 'never';
     if (angular.isUndefined(this.repeat.interval))
       this.repeat.interval = 1;
-    if (angular.isUndefined(this.repeat.month))
-      this.repeat.month = { occurrence: '1', day: 'SU', type: 'bymonthday' };
     if (angular.isUndefined(this.repeat.monthdays))
       // TODO: initialize this.repeat.monthdays with month day of start date
       this.repeat.monthdays = [];
+    else if (this.repeat.monthdays.length > 0)
+      this.repeat.month = { type: 'bymonthday' };
+    if (angular.isUndefined(this.repeat.month))
+      this.repeat.month = {};
+    if (angular.isUndefined(this.repeat.month.occurrence))
+      angular.extend(this.repeat.month, { occurrence: '1', day: 'SU' });
     if (angular.isUndefined(this.repeat.months))
       // TODO: initialize this.repeat.months with month of start date
       this.repeat.months = [];
@@ -588,26 +576,24 @@
 
     if (this.isNew) {
       // Set default values
-      Component.$Preferences.ready().then(function() {
-        var type = (_this.type == 'appointment')? 'Events' : 'Tasks';
+      var type = (this.type == 'appointment')? 'Events' : 'Tasks';
 
-        // Set default classification
-        _this.classification = Component.$Preferences.defaults['SOGoCalendar' + type + 'DefaultClassification'].toLowerCase();
+      // Set default classification
+      this.classification = Component.$Preferences.defaults['SOGoCalendar' + type + 'DefaultClassification'].toLowerCase();
 
-        // Set default alarm
-        var units = { M: 'MINUTES', H: 'HOURS', D: 'DAYS', W: 'WEEKS' };
-        var match = /-PT?([0-9]+)([MHDW])/.exec(Component.$Preferences.defaults.SOGoCalendarDefaultReminder);
-        if (match) {
-          _this.$hasAlarm = true;
-          _this.alarm.quantity = parseInt(match[1]);
-          _this.alarm.unit = units[match[2]];
-        }
+      // Set default alarm
+      var units = { M: 'MINUTES', H: 'HOURS', D: 'DAYS', W: 'WEEKS' };
+      var match = /-PT?([0-9]+)([MHDW])/.exec(Component.$Preferences.defaults.SOGoCalendarDefaultReminder);
+      if (match) {
+        this.$hasAlarm = true;
+        this.alarm.quantity = parseInt(match[1]);
+        this.alarm.unit = units[match[2]];
+      }
 
-        // Set notitifications
-        _this.sendAppointmentNotifications = Component.$Preferences.defaults.SOGoAppointmentSendEMailNotifications;
-      });
+      // Set notitifications
+      this.sendAppointmentNotifications = Component.$Preferences.defaults.SOGoAppointmentSendEMailNotifications;
     }
-    else {
+    else if (angular.isUndefined(data.$hasAlarm)) {
       this.$hasAlarm = angular.isDefined(data.alarm);
     }
 
@@ -639,10 +625,10 @@
   Component.prototype.hasCustomRepeat = function() {
     var b = angular.isDefined(this.repeat) &&
         (this.repeat.interval > 1 ||
-         this.repeat.days && this.repeat.days.length > 0 ||
-         this.repeat.monthdays && this.repeat.monthdays.length > 0 ||
-         this.repeat.months && this.repeat.months.length > 0 ||
-         this.repeat.month && this.repeat.month.day);
+         angular.isDefined(this.repeat.days) && this.repeat.days.length > 0 ||
+         angular.isDefined(this.repeat.monthdays) && this.repeat.monthdays.length > 0 ||
+         angular.isDefined(this.repeat.months) && this.repeat.months.length > 0 ||
+         angular.isDefined(this.repeat.month) && angular.isDefined(this.repeat.month.type));
     return b;
   };
 
@@ -687,13 +673,16 @@
   };
 
   /**
-   * @function isReadOnly
+   * @function showPercentComplete
    * @memberof Component.prototype
-   * @desc Check if the component is not editable and not an invitation
-   * @returns true or false
+   * @desc Check if the percent completion should be displayed with respect to the
+   *       component's type and status.
+   * @returns true if the percent completion should be displayed
    */
-  Component.prototype.isReadOnly = function() {
-    return (this.isReadOnly && !this.userHasRSVP);
+  Component.prototype.showPercentComplete = function() {
+    return (this.type == 'task' &&
+            this.percentComplete > 0 &&
+            this.status != 'cancelled');
   };
 
   /**
@@ -1257,7 +1246,7 @@
     var component = {};
     angular.forEach(this, function(value, key) {
       if (key != 'constructor' &&
-          key[0] != '$' &&
+          (key == '$hasAlarm' || key[0] != '$') &&
           key != 'blocks') {
         component[key] = angular.copy(value);
       }
