@@ -14,6 +14,7 @@
     vm.service = Component;
     vm.component = stateComponent;
     vm.close = close;
+    vm.highPriority = highPriority;
     vm.cardFilter = cardFilter;
     vm.newMessageWithAllRecipients = newMessageWithAllRecipients;
     vm.newMessageWithRecipient = newMessageWithRecipient;
@@ -27,19 +28,19 @@
     vm.copySelectedComponent = copySelectedComponent;
     vm.moveSelectedComponent = moveSelectedComponent;
 
-    // Load all attributes of component
-    if (angular.isUndefined(vm.component.$futureComponentData)) {
-      component = Calendar.$get(vm.component.pid).$getComponent(vm.component.id, vm.component.occurrenceId);
-      component.$futureComponentData.then(function() {
-        vm.component = component;
-        vm.organizer = [vm.component.organizer];
-      });
-    }
+    // Put organizer in an array to display it as an mdChip
+    vm.organizer = [stateComponent.organizer];
 
     function close() {
       $mdDialog.hide();
     }
 
+    function highPriority() {
+      return (vm.component &&
+              vm.component.priority &&
+              vm.component.priority < 5);
+    }
+
     // Autocomplete cards for attendees
     function cardFilter($query) {
       return AddressBook.$filterAll($query);
@@ -122,8 +123,8 @@
 
       c.$reply().then(function() {
         $rootScope.$emit('calendars:list');
-        $mdDialog.hide();
         Alarm.getAlarms();
+        $mdDialog.hide();
       });
     }
 
@@ -208,8 +209,8 @@
   /**
    * @ngInject
    */
-  ComponentEditorController.$inject = ['$rootScope', '$scope', '$log', '$timeout', '$mdDialog', 'User', 'CalendarSettings', 'Calendar', 'Component', 'AddressBook', 'Card', 'Alarm', 'stateComponent'];
-  function ComponentEditorController($rootScope, $scope, $log, $timeout, $mdDialog, User, CalendarSettings, Calendar, Component, AddressBook, Card, Alarm, stateComponent) {
+  ComponentEditorController.$inject = ['$rootScope', '$scope', '$log', '$timeout', '$mdDialog', 'sgFocus', 'User', 'CalendarSettings', 'Calendar', 'Component', 'AddressBook', 'Card', 'Alarm', 'stateComponent'];
+  function ComponentEditorController($rootScope, $scope, $log, $timeout, $mdDialog, focus, User, CalendarSettings, Calendar, Component, AddressBook, Card, Alarm, stateComponent) {
     var vm = this, component, oldStartDate, oldEndDate, oldDueDate;
 
     vm.service = Calendar;
@@ -217,7 +218,8 @@
     vm.categories = {};
     vm.showRecurrenceEditor = vm.component.$hasCustomRepeat;
     vm.toggleRecurrenceEditor = toggleRecurrenceEditor;
-    vm.showAttendeesEditor = false;
+    vm.recurrenceMonthDaysAreRequired = recurrenceMonthDaysAreRequired;
+    vm.showAttendeesEditor = vm.component.attendees && vm.component.attendees.length;
     vm.toggleAttendeesEditor = toggleAttendeesEditor;
     //vm.searchText = null;
     vm.cardFilter = cardFilter;
@@ -263,6 +265,12 @@
       vm.showAttendeesEditor = !vm.showAttendeesEditor;
     }
 
+    function recurrenceMonthDaysAreRequired() {
+      return vm.component &&
+        vm.component.repeat.frequency == 'monthly' &&
+        vm.component.repeat.month.type == 'bymonthday';
+    }
+
     // Autocomplete cards for attendees
     function cardFilter($query) {
       AddressBook.$filterAll($query);
@@ -270,15 +278,18 @@
     }
 
     function addAttendee(card) {
+      var automaticallyExapand = (!vm.component.attendees || vm.component.attendees.length === 0);
       if (angular.isString(card)) {
         // User pressed "Enter" in search field, adding a non-matching card
         if (card.isValidEmail()) {
           vm.component.addAttendee(new Card({ emails: [{ value: card }] }));
+          vm.showAttendeesEditor |= automaticallyExapand;
           vm.searchText = '';
         }
       }
       else {
         vm.component.addAttendee(card);
+        vm.showAttendeesEditor |= automaticallyExapand;
       }
     }
 
@@ -292,11 +303,11 @@
     function priorityLevel() {
       if (vm.component && vm.component.priority) {
         if (vm.component.priority > 5)
-          return l('low');
+          return l('low');                   // 6-7-8-9
         else if (vm.component.priority > 4)
-          return l('normal');
+          return l('normal');                // 5
         else
-          return l('high');
+          return l('high');                  // 1-2-3-4
       }
     }
 
@@ -308,8 +319,11 @@
             Alarm.getAlarms();
             $mdDialog.hide();
           }, function(response) {
-            if (response.status == CalendarSettings.ConflictHTTPErrorCode)
+            if (response.status == CalendarSettings.ConflictHTTPErrorCode &&
+                _.isObject(response.data.message))
               vm.attendeeConflictError = response.data.message;
+            else
+              edit(form);
           });
       }
     }
